/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18, 5737-M66
 * (C) Copyright IBM Corp. 2016,2022 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication, or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
Polymer({
	is: 'maximo-inspector-container',
	
  behaviors: [BaseComponent, Inspections, QuestionGroups, CascadeConditions, ResizeBehavior],

  listeners: {
    'showQuestion'              : '_showQuestion',
    'show-question-panel'       : '_showQuestionPanel',
    'element-status-change'     : '_elementStatusChange',
    'open-completion-summary'   : '_openCompletionSummary',
    'inspections-screen-resize' : '_screenResize',
  },

    properties: {

      /**
       * List of inspection results
       */
      resultSet: {
        type: Array,
        value: [],
        observer: '_resultSetChanged'
      },

			/**
       * List of inspection forms
       */
      inspFormSetPreview: {
        type: Array,
        value: [],
        observer: '_inspFormSetPreviewChanged'
      },

      /**
       * Processed preview inspform set
       * with additional attributes
       */
      preparedPreviewSet: {
        type: Array,
        value: null,
      },

      /**
       * DEPRECATED
       * Property set only after meters fetch
       * Use/observer this property to know when meters are fetched 
       */
      syncResultSet: {
        type: Array,
        value: null,
        observer: 'syncResultSetChanged'
      },

      /**
       * Processed result set
       * with additional attributes
       */
      preparedResultSet: {
        type: Array,
        value: null,
        observer: 'preparedResultSetChanged'
      },

    	/**
    	 * available meters
    	 */
    	availableMeters: {
    		type: Object,
        value: null,
        observer: 'availableMetersChanged'
    	},

    	/**
    	 * Indicate when to show complete panel
    	 * if all inspresults are complete
    	 */
    	openCompleteSummaryPending: {
    	  type: Boolean,
    	  value: function() {
    	    return false;
    	  }
      },
      
      /**
       * Store list of conditions by form
       * using formnum as key
       * includes new structure of responses 
       * and matches by result num
       * formnum => [{
       *  srcfield: ,
       *  tgtfield: ,
       * }, conditions...]
       * OPERATED by CascadeConditions Behavior
       */
      storedFormConditions: {
        type: Object,
        value: {}
      },

      /**
       * Store field default attributes
       * Used to revert changes
       */
      defaultFieldStorage: {
        type: Array,
        value: []
      },

      /**
       * Define if panels can toggle
       * based on device type
       */
      isToggleablePanel: {
        type: Boolean,
        value: true,
        observer: '_isToggleablePanelChanged'
      }

    },

	  ready: function() {},

	  /**
     * Observer resultset 
     */
    _resultSetChanged: function(newResultSet, oldResultSet) {
      //console.log('result set changed ');
      if (newResultSet && newResultSet.length) {
        
        var pending = newResultSet.filter(function(inspResult){
          return inspResult.status_maxvalue === status_.PENDING;
        });

        if(pending.length === 0){
          this.processResultSetChange(newResultSet)
        }
      }
    },

	//split out processResultSetChange functionality in order to
	//call it when we click the start button for a pending inspection.
	//Addresses APAR 322468.
    processResultSetChange: function(newResultSet){
	  // Filter completed inspections
      var completed = newResultSet.filter(function(inspResult){
        return inspResult.status_maxvalue === status_.COMPLETED;
      });

      // Check if openCompleteSummaryPending
      // Check if all inspections are complete
      if (this.openCompleteSummaryPending && 
        newResultSet.length === completed.length){
        this.openCompletionSummaryModal();
      } else {
        this._getAvailableMeters(newResultSet);
      }

      this.openCompleteSummaryPending = false;
    },

    /**
     * Observer inspection form set preview
     */
    _inspFormSetPreviewChanged: function(newSet, oldSet) {
      
      var preparedPreviewSet = [];
      preparedPreviewSet = this.preparePreviewSet(newSet);

      if (preparedPreviewSet && preparedPreviewSet.length) {

        //var inspectionSetCopy = JSON.parse(JSON.stringify(newSet));

        // Sort result set by form name - Alphabetical sort
        preparedPreviewSet.sort(function (a, b) {
          if (a.name > b.name) {
            return 1;
          }
          if (a.name < b.name) {
            return -1;
          }
          // a must be equal to b
          return 0;
        });

      }

      this.set('preparedPreviewSet', preparedPreviewSet);
    },

    /**
     * Observer avaialbleMeters
     */
    availableMetersChanged: function(newAvailableMeters) {
      if (newAvailableMeters) {
        this.prepareResultSet(this.resultSet ,newAvailableMeters);
      }
    },

    /**
     * DEPRECATED
     * Observer syncresultset
     * Create to open completion summary
     */
    syncResultSetChanged: function(changeRecord, oldRecord) {

      if (changeRecord) {

        // Filter completed inspections
        var completed = changeRecord.filter(function(inspResult){
          return inspResult.status_maxvalue === 'COMPLETED';
        });

        // Check if openCompleteSummaryPending
        // Check if all inspections are complete
        if (this.openCompleteSummaryPending && 
           changeRecord.length === completed.length){

          this.openCompletionSummaryModal();

        }
        this.openCompleteSummaryPending = false;
      }
    },

    /**
     * Observer preparedResultSet
     */
    preparedResultSetChanged: function(newPreparedResultSet){
      //TODO remove this return
      // return;
      if (newPreparedResultSet) {

        // Filter completed inspections
        var completed = newPreparedResultSet.filter(function(inspResult){
          return inspResult.status_maxvalue === 'COMPLETED';
        });

        // Check if openCompleteSummaryPending
        // Check if all inspections are complete
        if (this.openCompleteSummaryPending && 
          newPreparedResultSet.length === completed.length){

          this.openCompletionSummaryModal();
        }
        this.openCompleteSummaryPending = false;
      }
    },

    /**
     * Observer isToggleablePanel
     */
    _isToggleablePanelChanged: function(newIsToggleableValue) {

      this.toggleClass('desktop', !newIsToggleableValue, this.$.shell);
      this.toggleClass('desktop', !newIsToggleableValue, this.$.leftPanel);
      this.toggleClass('desktop', !newIsToggleableValue, this.$.rightPanel);
      
	  this.toggleClass('mobile', newIsToggleableValue, this.$.completionSummaryModalContent);

    },

    /**
     * Get all fields from inspection result form 
     * with type equals to meter
     * @return array with fields
     */
    getMeterFields: function(inspectionResult) {
      let inspectionMeterFields = [];
      
      if (inspectionResult && inspectionResult.inspectionform 
          && inspectionResult.inspectionform.length) {
          let form = inspectionResult.inspectionform[0];
          if (form && form.inspquestion && form.inspquestion.length) {
              let questions = form.inspquestion;
              if (questions && typeof questions[Symbol.iterator] === 'function') {
                  // Iterate over inspection questions
                  for (let question of questions) {
                      let fields = question.inspfield;
                      if (fields && typeof fields[Symbol.iterator] === 'function') {
                          // Iterate over question fields
                          for (let field of fields) {
                              if (field.fieldtype_maxvalue && field.fieldtype_maxvalue === fieldTypes_.MM) {
                                  inspectionMeterFields.push(Object.assign({},field));
                              }
                          }
                      }
                  }
              }
          }
      }
      return inspectionMeterFields;
    },

    /**
     * Request available meters for
     * assets and locations found in result set
     */
    _getAvailableMeters: function(resultList) {

      // Reset available meters
      this.set('availableMeters', null);

      var self = this;
      var assetList = [];
      var locationList = [];
      var meterFields = [];
      // Get assets and locations from result set
      resultList.forEach(function(inspResult) {
        // Include asset num in list to query        
        if (inspResult.assets && inspResult.assets.assetnum) {
          assetList.push(inspResult.assets.assetnum);
        }
        // Include location id in list to query
        if (inspResult.locations && inspResult.locations.location) {
          locationList.push(inspResult.locations.location);
        }

        let inspMeterFields = this.getMeterFields(inspResult);
        meterFields = meterFields.concat(inspMeterFields);
      }, this);

      var promiseList = [];
      var maximoQbe, qbeObject;
      // Request assetmeter
      if (assetList.length && meterFields.length) {
        maximoQbe = new MaximoQbe();
        maximoQbe.setIn('assetnum',assetList).setEquals('active','true');
        qbeObject = maximoQbe.toJSON();
        this.$.assetMeterCollection.queryWhere = qbeObject;
        promiseList.push(this.$.assetMeterCollection.refreshRecords());
      } else {
        promiseList.push(Promise.resolve(false));
      }

      // Request locationmeter
      if (locationList.length && meterFields.length) {
        maximoQbe = new MaximoQbe();
        maximoQbe.setIn('location',locationList).setEquals('active','true');
        qbeObject = maximoQbe.toJSON();
        this.$.locationMeterCollection.queryWhere = qbeObject;
        promiseList.push(this.$.locationMeterCollection.refreshRecords());
      } else {
        promiseList.push(Promise.resolve(false));
      }

      // Wait for all requests to get resolved
      Promise.all(promiseList).then(function(values) {
        var availableMeters = {};
        if (values.length === 2) {
          availableMeters.assetMeters = values[0] ? values[0].response.member : [];
          availableMeters.locationMeters = values[1] ? values[1].response.member : [];
        }
        // Set available meters
        self.set('availableMeters', availableMeters);

      }).catch(function (reason) {
        console.error('An error occurred', reason);
      });

    },

    /**********************************************
      MIGRATE TO BEHAVIOR - Begin
     **********************************************/

    /**
      * Only preview mode
      * Add helper attributes to inspection form entities
      * Apply sort and 
      * set property preparedResultSet
      */
    preparePreviewSet: function(inspformSet) {

      var previewSetCopy = JSON.parse(JSON.stringify(inspformSet));

      if ( previewSetCopy && previewSetCopy.length ) {
        for ( var form of previewSetCopy ) {

          if ( form && form.inspquestion && form.inspquestion.length ) {
            for ( var question of form.inspquestion ) {
              // Iterate over inspection fields
              if ( question.inspfield && question.inspfield.length ) {
                question.inspfield.sort((a, b) => a.sequence - b.sequence);
                for ( var field of question.inspfield ) {
                  this.ensureFieldVisibleProperty(field);
                }
              }

              // Ensure all fields have auxiliary attributes
              this.setQuestionVisibleProperty(question);
            }

            // Iterate over inspection questions again
            // Now looking for question groups to apply auxiliary attributes
            // Ensure all questions have auxiliary attributes first
            for ( var question of form.inspquestion ) {
              // Check if is group
              if ( this._isGroupHeader(question) ) {
                // Find all questions belonging the group
                var children = this._getQuestionChildrenOutOfSet(question.groupid, form.inspquestion);
                this.setQuestionGroupVisibleProperty(question, children);
              }
            }
            
            // Ensure all questions have auxiliary attributes
            var children = this._getFormDependentQuestions(form.inspquestion);
            this.setFormVisibleProperty(form, children);
          }
        }
      }
      return previewSetCopy;
    },

    /**
     * Add helper attributes to inspection form entities
     * Apply sort and 
     * set property preparedResultSet
     */
    prepareResultSet: function(resultSet, meters) {

      var resultSetCopy = JSON.parse(JSON.stringify(resultSet));

      // Add _invalid and _completed to fields
      // Add _required and _completed to question
      // Add _required and _completed to question group
      // Add _required and _completed to form

      // Add auxiliary attributes to fields and questions
      // Need to perform only in those elements before 
      // compute auxiliary attributes for question groups and forms

      try {
        // Iterate over inspection results
        if ( resultSetCopy && resultSet.length ) {
          for ( var inspResult of resultSetCopy ) {
            // Get available meters for inspection result
            var availableMeters = this.getAvailableMeters(inspResult, meters);

            const formConditions = this.extractFormConditions(inspResult.inspectionform[0]);
            // Conditions already checked against field responses
            let metConditions = this.findMetConditions(inspResult);

            let formFields = [];

            // Iterate over inspection questions
            let form = inspResult.inspectionform[0];
            if ( form && form.inspquestion && form.inspquestion.length ) {

              let fieldResults = inspResult.inspfieldresult || [];
              // Store field conditions in a library by form.
              // Used during inspection execution
              this.storeFormConditions(this.storedFormConditions, form, fieldResults);

              /**
               * Apply auxiliary attributes to fields
               * Concat fields in list
               */
              for ( var question of form.inspquestion ) {
                // Iterate over inspection fields
                if ( question.inspfield && question.inspfield.length ) {
                  question.inspfield.sort((a, b) => a.sequence - b.sequence);
                  for ( var field of question.inspfield ) {
                    this.clearResponseIntruders(field, inspResult.resultnum, inspResult.orgid);
                    this.setFieldInvalidProperty(field, availableMeters);
                    this.setFieldCompleteProperty(field);
                    this.ensureFieldVisibleProperty(field);

                    // Store field default attributes before applying it
                    if (this.isFieldATarget(field, formConditions)) {
                      this.defaultFieldStorage = this.addTargetDefaults(field, this.defaultFieldStorage);
                    }  
                  }
                  formFields = formFields.concat(question.inspfield);
                }
              }

              // Order questions before processing conditions
              form.inspquestion.sort( (questionA, questionB) => {
            	 return questionA.groupseq - questionB.groupseq;
              });

              /**
               * Apply cascade conditions to fields
               * Apply auxiliary attributes to questions
               */
              for ( var question of form.inspquestion ) {
                // Iterate over fields applying conditions
                if ( question.inspfield && question.inspfield.length ) {
                  for ( var field of question.inspfield ) {
                    this.applyConditions(field, metConditions, formFields, this.defaultFieldStorage);
                  }
                }
                // Ensure all fields have auxiliary attributes
                this.setQuestionRequiredProperty(question);
                this.setQuestionCompleteProperty(question);
                this.setQuestionVisibleProperty(question);
              }

              // Iterate over inspection questions again
              // Now looking for question groups to apply auxiliary attributes
              // Ensure all questions have auxiliary attributes first
              for ( var question of form.inspquestion ) {
                // Check if is group
                if ( this._isGroupHeader(question) ) {
                  // Find all questions belonging the group
                  var children = this._getQuestionChildrenOutOfSet(question.groupid, form.inspquestion);
                  this.setQuestionGroupRequiredProperty(question, children);
                  this.setQuestionGroupCompleteProperty(question, children);
                  this.setQuestionGroupVisibleProperty(question, children);
                }
              }
              
              // Ensure all questions have auxiliary attributes
              var children = this._getFormDependentQuestions(form.inspquestion);
              this.setFormRequiredProperty(form, children);
              this.setFormCompleteProperty(form, children);
              this.setFormVisibleProperty(form, children);
            }
          }
        }
      } catch (error) {
        console.error('Error processing execution page', error);
      }

      this.set('preparedResultSet', resultSetCopy);
      $M.toggleWait(false);
    },

    /**
     * Retrieve questions for the same group id
     */
    _getQuestionChildrenOutOfSet: function (groupid, set) {
      var children = [];

      if (set && set.length){
        children = set.filter(function(item) {
          return item.groupid === groupid 
            && this._isGroupChild(item)
            && item[this._visibleProperty];
        }, this);
      }
      return children;
    },

    /**
     * Retrieve 1st level questions from form
     * Question Group or Question (if no child belonging to it)
     */
    _getFormDependentQuestions: function(questionSet) {

      var firstLevelQuestions = [];
      if (questionSet && questionSet.length) {
        firstLevelQuestions = questionSet.filter(function(element){
          // Check groupseq has decimal value
          return element.groupseq%1===0 
            && element[this._visibleProperty];
        }, this);
      }
      return firstLevelQuestions;
    },

    /**
     * Remove responses from same field but another result
     */
    clearResponseIntruders: function(field, resultNum, resultOrg) {
      if (field && field.inspfieldresult) {
        // Get response field
        var fieldResponse = this.findInnerFieldResponseInField(field, resultNum, resultOrg);
        if (fieldResponse) {
          field.inspfieldresult = [fieldResponse];
        } else {
          delete field.inspfieldresult;
        }
      }
    },

    /**
     * Set required property for form
     */
    setFormRequiredProperty: function(form, children) {
      var isRequired = false;

      if (children && children.length) {
        for (var child of children) {
          if (child[this._requiredProperty] === true) {
            isRequired = true;
            break;
          }
        }
      }

      form[this._requiredProperty] = isRequired;
    },

    /**
     * Set required property for question group
     */
    setQuestionGroupRequiredProperty: function(question, children) {
      var isRequired = false;

      if (children && children.length) {
        for (var child of children) {
          if (child[this._requiredProperty] === true) {
            isRequired = true;
            break;
          }
        }
      }

      question[this._requiredProperty] = isRequired;
    },

    /**
     * Set required property of question
     */
    setQuestionRequiredProperty: function( question) {

      var isRequired = false;

      if (question && question.inspfield && question.inspfield.length) {
        for (var field of question.inspfield) {
          if (field.required && field[this._invalidProperty] === false) {
            isRequired = true;
            break;
          }
        }
      }

      question[this._requiredProperty] = isRequired;
    },

    /**
     * Set complete property of form
     */
    setFormCompleteProperty: function(form, children) {
      var isCompleted = false;
      if (children && children.length) {
        isCompleted = this.computeCompleteProperty( children );
      }
      form[this._completeProperty] = isCompleted;
    },
    
    /**
     * Set complete property of question group
     */
    setQuestionGroupCompleteProperty: function(question, children) {
      var isCompleted = false;
      if (children && children.length) {
        isCompleted = this.computeCompleteProperty( children );
      }
      question[this._completeProperty] = isCompleted;
    },

    /**
     * Set complete property of question
     */
    setQuestionCompleteProperty: function( question ) {
      var isCompleted = false;
      if (question && question.inspfield) {

        var visibleFields = question.inspfield.filter(function(field) {
          return field.visible;
        }, this);

        // Filter required inspfields
        var requiredFields = visibleFields.filter(function(field) {
          return (field[this._invalidProperty] === false && field.required === true);
        }, this);
        
        var fieldSet = requiredFields.length ? requiredFields : visibleFields;
        isCompleted = this.computeCompleteProperty( fieldSet );
      }
      question[this._completeProperty] = isCompleted;
    },

    /**
     * Based on a set check if all 
     * has _completeProperty equals true
     */
    computeCompleteProperty: function ( itemSet ) {
      var isCompleted = false;

      if (itemSet && itemSet.length) {
        let completedItems = itemSet.filter( function(item){
          return (item[this._completeProperty] === true || item[this._invalidProperty])
            && (item[this._visibleProperty] || item.visible);
        }, this);

        isCompleted = (itemSet.length === completedItems.length);
      }

      return isCompleted;
    },

    /**
     * Set complete property of inspfield
     */
    setFieldCompleteProperty: function( inspField ) {

      var isCompleted = false;

      // Get field type
      var responseFieldType = this.getResponseFieldType(
        inspField.fieldtype_maxvalue, inspField.metertype_maxvalue);

      if (inspField.inspfieldresult) {
        var fieldResponse = inspField.inspfieldresult[0];
        isCompleted = fieldResponse ? this.checkFieldCompletion( responseFieldType, fieldResponse ) : false;
      }

      inspField[this._completeProperty] = isCompleted;
    },

    /**
     * Set visible property of form
     */
    setFormVisibleProperty: function(form, children) {
      var isVisible = false;
      if (form && typeof form === 'object') {
        if (children && children.length) {
          for (var child of children) {
            if (child[this._visibleProperty] === true) {
              isVisible = true;
              break;
            }
          }
        }
        form[this._visibleProperty] = isVisible;
      }
    },

    /**
     * Set visible property for question group
     */
    setQuestionGroupVisibleProperty: function(question, children) {
      var isVisible = false;

      if (question && typeof question === 'object') {
        if (children && children.length) {
          for (var child of children) {
            if (child[this._visibleProperty] === true) {
              isVisible = true;
              break;
            }
          }
        }
        question[this._visibleProperty] = isVisible;
      }

    },

    /**
     * Set visible property of question
     */
    setQuestionVisibleProperty: function( question ) {

      var isVisible = false;

      if (question && typeof question === 'object') {
        if (question.inspfield && question.inspfield.length) {
          for (var field of question.inspfield) {
            if (field.visible === true) {
              isVisible = true;
              break;
            }
          }
        } else {
        	isVisible = true;
        }
        question[this._visibleProperty] = isVisible;
      }
    },

    /**
     * Ensure all fields to have visible property
     * Set default value case prop is not present
     * @param {*} inspField 
     */
    ensureFieldVisibleProperty: function( inspField ) {
      var isVisibleDefault = true;
      if (inspField && typeof inspField === 'object' 
        && !('visible' in inspField)) {
        inspField.visible = isVisibleDefault;
      }
    },

    /**
     * Filter valid response returned inside inspfield
     * @returns {Object | null}
     */
    findInnerFieldResponseInField: function( inspField, resultNum, resultOrg ) {
      var responseArray = [];
      var response = null;
      if (inspField && inspField.inspfieldresult && inspField.inspfieldresult.length) {
        responseArray = inspField.inspfieldresult.filter(function(el) {
          return el.orgid === resultOrg && el.resultnum === resultNum;
        });

        if (responseArray.length) {
          if (responseArray.length === 1) {
            response = responseArray[0];
          } else {
            console.log(`[CRITICAL] ${responseArray.length} responses found with resultnum ${resultNum}.`);
          }
        }
      }

      return response;
    },

    /**
     * Conditions to verify each type of field
     * @returns {Boolean}
     */
    checkFieldCompletion: function( respType, response ) {

      var isCompleted = false;

      if (respType === 'numresponse' && response[respType] || response[respType] === 0) {
        isCompleted = true;

      } else if (respType === 'document' && response.doclinks) {
        let attachments = [];

        if (response.doclinks.member) {
          //Update attachments
          attachments = response.doclinks.member;
        }
        isCompleted = Boolean(attachments.length);
        
      } else if(response.fieldtype_maxvalue==='DT'){
          if(response.dateresponse!==undefined && response.timeresponse!==undefined){
            isCompleted = true;
          }     
      }
      else if (response[respType] && response[respType].length) {
        isCompleted = true;
      }

      return isCompleted;
    },

    /**
     * Set invalid property of inspfield
     */
    setFieldInvalidProperty: function( inspField, metersAvailable ) {

      var isInvalid = false;

      if (inspField.fieldtype_maxvalue && 
        inspField.fieldtype_maxvalue === 'MM') {

        let metersFound = this.findInspectionMeters(metersAvailable, inspField.metername);
        if (!metersFound.length) {
          isInvalid = true;
        }

      }
      inspField[this._invalidProperty] = isInvalid;

    },

    /**
     * Find meters matching current inspection field
     * @return {Array}
     */
    findInspectionMeters: function(availableMeters, fieldMeter) {
      
      return availableMeters.filter(function(meter) {
        return meter.metername === fieldMeter;
      });
    },

    /**
     * Try sorting by sequence otherwise sort by inspection form name
     * param Array resultset
     */
    sortResultSet: function(resultSet) {

      var hasSequence = resultSet.every(function(curVal){ return curVal.hasOwnProperty('sequence'); });

      if (hasSequence) {

        // Sort result set by sequence - Num sort
        resultSet.sort(function (a, b) {
          if (a.sequence > b.sequence) {
            return 1;
          }
          if (a.sequence < b.sequence) {
            return -1;
          }
          return 0;
        });
      } else {
        
        // Sort result set by form name - Alphabetical sort
        resultSet.sort(function (a, b) {
          var formA = a.inspectionform[0];
          var formB = b.inspectionform[0];
          if (formA.name > formB.name) {
            return 1;
          }
          if (formA.name < formB.name) {
            return -1;
          }
          // a must be equal to b
          return 0;
        });
      }
    },

    /**********************************************
      MIGRATE TO BEHAVIOR - End
     **********************************************/

	  /**
	   * Clear question, filters and progressbar
	   */
	  resetPanels: function() {
	    this.$.questionPanel.reset();
	    this.$.execution.reset();
	  },
	  
	  /**
	   * showQuestion Listener
     * show question in execution panel
	   */
	  _showQuestion: function(event) {
	    var element = event.detail;
      this.$.execution.showElement(element);
      
      if (this.isToggleablePanel) {
        this.toggleClass('hide', true, this.$.leftPanel);
      }
	  },
    
    /**
     * showQuestionPanel Listener
     * Transit to question panel
     * @param {*} event 
     */
    _showQuestionPanel: function(event) {
      event.stopPropagation();

      if (this.isToggleablePanel) {
        this.toggleClass('hide', false, this.$.leftPanel);
      }
    },

	  /**
	   * Listen to event fired by execution
	   */
	  _elementStatusChange: function(event) {
	    var payload = event.detail;
	    var leftPanel = this.$.questionPanel;
	    leftPanel.updateElements(payload);
	  },

	  /**
	   * Listen to event fired by execution
	   */
	  _openCompletionSummary: function(event) {
	    this.openCompletionSummaryModal();
	  },

    /**
     * Listen to screen resize
     * @param {*} ev 
     */
    _screenResize: function(ev) {

      ev.stopPropagation();
      // console.log('screen resize capture at container');

      const isToggleable = (this.deviceType === deviceTypes_.PHONE && this.orientation === orientationTypes_.PORTRAIT)
        || (this.deviceType === deviceTypes_.TABLET && this.orientation === orientationTypes_.PORTRAIT);

      this.set('isToggleablePanel', isToggleable);

    },

    /**
     * Show left panel/question panel
     */
    showLP: function(ev) {
      //TODO toggle class from left panel
      this.toggleClass('hide', false, this.$.leftPanel);
    },
    
	  /**
	   * Notify panels about existing error 
	   */
	  revealErrorInPanels: function(inspFieldResult, error) {

	    var formsPanel = this.$.execution;
	    formsPanel.revealError(inspFieldResult, error);
	  },

	  /**
	   * Open completion summary modal
	   */
	  openCompletionSummaryModal: function() {
	    var modal = this.$.completionSummaryModal;
	    modal.style.display = 'block';
	    
	  },

	  /**
	   * Close completion summary modal
	   */
	  _closeCompletionSummaryModal: function(event) {
	    var modal = this.$.completionSummaryModal;
	    if (modal.style.display !== 'none') {
	      modal.style.display = 'none';
	    }

	    this.closeExecution();
	  },

	  /**
	   * Handler closing inspection execution
	   */
	  _closeExecution: function(event) {

	    var previewMode = event.detail.previewMode;

	    this.closeExecution(previewMode);

	  },

	  /**
	   * Closing inspection execution
	   */
	  closeExecution: function(isPreviewMode) {

	    if (isPreviewMode) {
        window.close();
        return;
      }

      this.$.execution.resetRepeaters();

      // Stop audio from inspection execution
      this.$.execution.stopAudioGuided();

      var inspresultlistcollection = $M.getGlobalResource('inspresultlistcollection');
      inspresultlistcollection.refreshRecords();
      var pageName = 'inspResultList';
      this.fire('changePage', pageName);

	  }
});
