/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18, 5737-M66
 * (C) Copyright IBM Corp. 2016,2021 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication, or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
Polymer({
	
	is: 'maximo-inspection-main',
	
  	behaviors: [BaseComponent, QuestionGroups],
  	
  	listeners: {
  		'buildNewForm': '_showCleanForm',
  		'editForm': '_editForm',
  		'goback':'_goBack',
		'done':'_done',
		'formSaveButton':'_savebutton',
		'saveForm':'_saveForm',
		'saveQuestionDescriptions':'_saveQuestionDescriptions',
		'preview':'_openPreview',
		'updateButtons':'_updateNavButtons',
		'createUpdate':'_createUpdate',
		'changeStatus':'_changeStatus',
		'record-updatewsmethod:changeFormStatus':'_changeStatusSuccess',
		'delete':'_deleteForm',
		'deleteQuestion':'_deleteQuestion',
		'createRevision':'_getRevision',
		'collapseInstructions':'_collapseInstructionInputArea',
		'duplicate':'_duplicateRecord',
		'addUpdateQuestion':'_addUpdateQuestion',
		'updateInspectionForm':'_updateInspectionForm',
		'resequenceQuestions':'_resequenceQuestionsHandler',
		'setNextEvent': '_setNextEvent',
		'processNextEvent': '_processNextEvent',
		'open-revision': '_openRevision',
		'refresh-revision': '_refreshRevision',
		'record-getwsmethod:getRevision': 'createRevisionSuccessHandler',
		'record-getwsmethod:getRevision-error': 'createRevisionErrorHandler',
		'saveReason': '_saveReason'
  	},
  	
    properties: {
    	
    	/**
    	 * Inspetion form set grouped by revision
    	 */
    	groupedData: {
    		type: Array,
    		value: []
    	},
    	
    	/**
    	 * Inspection form with details
    	 */
    	currentInspForm: {
    		type: Object,
    		value: null,
    		observer: '_currentInspFormChanged'
    	},
    	
    	/**
    	 * Fetched attributes have to match for these collections
    	 * inspformstacked, inspformrevisioncollection
    	 */
    	formAttributes: {
    		type: String,
    		readOnly: true,
    		value: function() {
    			return '*,inspectionformstatus{href},inspectionresult{href},inspquestion{href},personchange.displayname,personcreate.displayname,inspformscript{*}';
    		}
    	},
    	
    	/**
    	 * component selected in iron-pages 
    	 */
    	selectedPage: {
    		type: String,
    		value: 'list'    	
    	},
    	
    	/**
    	 * Current object hreference
    	 */
    	inspecFormHref: {
    		type: String
    	},
    	
    	
    	/**
    	 * Last revised form pending deactivation
    	 */
    	revisedFrom: {
    		type: Object,
    		value: null
    	},
    	
    	/**
    	 * Synonym domain collection
    	 */
    	formStatusSet: {
    		type: Array,
    		observer: '_statusSetChange'
    	},
    	    
    	/**
    	 * Synonym domain collection for form type
    	 */
    	formTypeSet: {
    		type: Array
	},
    	
    	/**
    	 * Inspection form status domain filter
    	 */
    	domainStatusFilter: {
    		type: Array,
    		value: function () {
    			return [{'filtertype': 'SIMPLE', 'field': 'domainid', 'availablevalues': [ {'value': 'INSPECTFORMSTATUS', 'selected': true} ]}];
    		}
    	},
    	
    	/**
    	 * Inspection form input domain filter
    	 */
    	domainInputFilter: {
    		type: Array,
    		value: function() {
    			return [{'filtertype': 'SIMPLE', 'field': 'domainid', 'availablevalues': [ {'value': 'INSPFIELDGROUP', 'selected': true} ]}];
    		}
    	},
    	
		/**
		 * URI to fetch input type with conditions processed
		 */
		 lookupInputTypeURI: {
			type: String,
			value: function() {
				return $M.configureURLContext($M.getMaxauth().baseUri+'/oslc/os/mxapiinspform/newmbo/inspquestion/newmbo/inspfield/newmbo/getlist~fieldtype')
			}
		},

     	/**
    	 * Inspection form type domain filter
    	 */
    	domainFormTypeFilter: {
    		type: Array,
    		value: function () {
    			return [{'filtertype': 'SIMPLE', 'field': 'domainid', 'availablevalues': [ {'value': 'INSPFORMTYPE', 'selected': true} ]}];
    		}
    	},
    	
    	isNewRecord: {
    		type: Boolean,
    		value: false
    	},
    	
    	saveClicked: {
    		type: Boolean,
    		value: false
    	},
    	
    	questionSort: {
    		type: Array,
    		value: function () {
    			return [{collection: 'inspectionform.inspquestion', attributes: '%2Bgroupseq'}];
    		}
    	},
    	
    	nextEvent: {
    		type: Object, // {event, target}
    		value: null
    	},

    	/**
    	 * saved query to fetch inspection
    	 */
    	queryName: {
    	  type: String,
    	  value: 'INSPECTIONALL',
    	  readOnly: true
    	},
    	/**
    	 * Flag for available question.
    	 */
    	questionFlag: {
    		type: Boolean,
    		value: false
    	},

		/**
		 * INSPFIELDGROUP domain values prop
		 */
		domainInputGroupCollection: {
			type: Array,
			value: []
		},

		/**
		 * INSPFIELDTYPE domain values prop
		 */
		domainInputTypeCollection: {
			type: Array,
			value: []
		},

		/**
		 * Combined input groups and input types
		 */
		formFieldSet: {
			type: Array,
			value: []
		},

		/*
		* Property used to check Mobile Features enabled
		*/
	    mobileFeatures: {
		   type: Boolean,
		   value: false,
		   notify: true
	    },

		mviEnabled:{
			type: Boolean,
			value: false,
			notify:true
		},

		/**
		 * Property used to check multi select feature enabled
		 */
		multiSelectFeature: {
			type: Boolean,
			value:false,
			notify: true
		},

		/**
		 * Property used to check signature feature enabled
		 */
		 signatureFeature: {
			type: Boolean,
			value:false,
			notify: true
		},

		/**
		 * Property used to check input validation feature enabled
		 */
		inputValidationFeature: {
			type: Boolean,
			value:false,
			notify: true
		},

		/**
		 * Property used to check input validation feature enabled
		 */
		lookupResponseFeature: {
			type: Boolean,
			value:false,
			notify: true
		},

		/**
		 * Property used to check enablereview feature enabled
		 */
		 formReviewFeature: {
			type: Boolean,
			value: false,
			notify: true
		},
	},

	observers: [
		'inputFieldTypeChanged(domainInputGroupCollection, domainInputTypeCollection)'
	],
	
	/**********  OBSERVERS Begin  *********/
	attached: function (){
		this.$.mobileFeaturesProp?.fetchProperty();
		this.$.mobileSignatureProp?.fetchProperty();
		this.$.mobileMultiSelectProp?.fetchProperty();
		this.$.mobileInputValidationProp?.fetchProperty();
		this.$.mobileLookupResponseProp?.fetchProperty();
		this.$.mobileFormReviewProp?.fetchProperty();
		this.$.mviEnabledProp.fetchProperty();
	},
	/**
	 * Complex observer
	 * Combine both input group and input type and sets formFieldSet property
	 */
	inputFieldTypeChanged: function(inputGroup, inputType) {
		if (inputGroup?.length && inputType?.length) {
			const inputTypeDomain = {
				domainid: 'INSPFIELDTYPE',
				synonymdomain: inputType
			}
			const domainsCombined = [inputGroup[0], inputTypeDomain];
			this.set('formFieldSet', domainsCombined);
		}
	},

	/**
	 * currentInspForm observer
	 */
	_currentInspFormChanged: function (newv) {
	
		if (!newv) {
			return;
		}
		
		if(newv.status_maxvalue=='PNDREV'){
			this._setRevisionForm();			
		}
	
		if(newv.inspquestion){
			this.$.formEdition.questions = newv.inspquestion;
		} else {
			this.$.formEdition.questions = [];
		}

	},
	
	/**********  OBSERVERS End  *********/

	
	/**********  LISTENERS Begin  *********/
	
	_showCleanForm: function (e) {
		//show form page 
		this.$.formEdition.clearForm();
		this.currentInspForm = null;
		
		//clear UI search value in search text box
		$j(e.target).find('#formList_searchbar_inputval')[0].value='';
		
		//this.$.inspformstacked.queryWhere = '';
		this.$.inspformstacked.getNewRecordData();
		
		$M.toggleWait(true);
		//this._changePage(1);
		this.$.formEdition.fire('setFormNameFocus');
	},
	
	_editForm: function(e){
		
		this.$.formEdition.clearForm();
		var record = e.detail;
		this.currentInspForm = null;
		//this.currentInspForm = record;
		this.$.formEdition.isNewRecord = false;
		this.inspecFormHref = record.href;
		this.questionFlag = false;
		this.$.inspecFormResource.loadRecord().then(function() {
			this.questionFlag = true;
		});
		
		this._changePage('form');
	},
	
	_goBack: function (e) {		
		this._collapseInstructionInputArea(e);
		this._collapseGroupDescriptionAreas(e);
		this._closeQuestionDialog(e);
		if (this.selectedPage === 'form' && !this.currentInspForm.isDraft){
			this._changePage('revision');
		} else {
			this._changePage('list');
		}
		this.$.formEdition.clearForm();
	},
	
	_done: function (e) {
	},
	
	_savebutton : function(e){
		this.saveClicked = true;
		$M.toggleWait(true);
		this.$.formEdition.fire('done');
	},
	
	/**
	 * Trigger form save
	 */
	_saveForm : function(e){
		this.$.formEdition.fire('done');
	},
	
	_saveQuestionDescriptions: function(e){	// before doing another action, update any questions that haven't been saved
		if (e && e.detail){
			var questions = e.detail;
			var questionsToUpdate = [];
			for(var i=0; i< questions.length; i++){
				var question = questions[i];
				if (question._action){	// do update if there is an existing _action
					questionsToUpdate.push(this._cloneQuestion(question, ['_action','_id','_rowstamp','href','localref','description','description_longdescription']));
				}
			}
			if (questionsToUpdate.length > 0){
				this._updateQuestions(questionsToUpdate);
			}
		}
		this.fire('setDescriptionFocus');
	},
	
	_openPreview: function (e) {
		var record = e.detail;
		var root = window.location.href;
		var url =  root + '/preview/'+ encodeURIComponent(encodeURIComponent(record.href));
		
		window.open(url,'preview', 'noopener');
	},
	
	_updateNavButtons: function (e) {
		this._fixButtons(this.selectedPage);
		this._fixQuestionButtons(this.selectedPage);
	},
	
	_createUpdate: function (e) {
		//Grab the whole form record
		var recordData = e.detail.record;
		var properties = e.detail.props;
	
		$M.toggleWait(true);
		
		if (this.revisedFrom || recordData.revision > 1){
			recordData.isrevision = true;
		}
		
		this.$.inspformstacked.createRecord(recordData, properties);
		
		//Wait layer needs to be re-enabled since createRecord disables.
		$M.toggleWait(true);
	},
	
	_changeStatus: function (e) {

		var params = e.detail.object;
		var responseProperties = e.detail.props;
		var objHref = e.detail.href;
        var detail = e.detail;
		
		//Update var inspecFormHref
		this.inspecFormHref = objHref;
		
		var message;
		var self = this;
		//Call method to update record
		this.$.inspecFormResource.updateAction('wsmethod:changeFormStatus', params, responseProperties).then(function(result){
			self.$.inspformstacked.refreshRecords();
			var record = result.response;
			this.currentInspForm = record;
			self.requestFormRevisions(record.inspformnum, record.orgid);
			message = $M.localize('uitext','mxapiinspection','inspectionform_updated');
			$M.notify(message,$M.alerts.info);
		}, function(error) {
			$M.showResponseError(error);
			$M.toggleWait(false);
			self.$.inspformstacked.refreshRecords();
			detail.errorCallback(error, detail.target);
		});
	},
	
	_changeStatusSuccess: function (e) {
		this.revisedFrom = null;
	},
	
	_deleteForm : function(e){
		var record = e.detail;
		//var successmessage = $M.localize('uitext','mxapiinspection','inspectionform_deleted',['\''+e.detail.name+'\'']);
		var failuremessage = $M.localize('uitext','mxapiinspection','inspectionform_notdeleted',[e.detail.name]);
		var self = this;
		
		if (record.hasrevision===false || record.status_maxvalue==='PNDREV'){
			$M.toggleWait(true);
			this.$.inspecFormResource.deleteRecord(record.href);
		} else {
			$M.notify(failuremessage,$M.alerts.warn);
		}
	},
	
	_deleteQuestion : function(e){
		// delete the question then update the remaining ones
		this._saveForm(e);
		var recordArray = e.detail;
		var deletedHrefMap = recordArray.map(rec => rec.href);
		var deletedGroupMap = recordArray
			.filter(rec => rec.groupid > 0 && rec.sequence === 0)
			.map(rec => rec.groupid);

		var questions = this.$.formEdition.questions;
		// find array object and delete it, if a group header is deleted then delete children too
  		for(var i=0; i< questions.length; i++){
  			if(deletedHrefMap.includes(questions[i].href) || deletedGroupMap.includes(questions[i].groupid)){
  				questions[i]._action = 'Delete';
  			}
  		}
  		this._resequenceQuestions(questions);
	},
	
	_getRevision: function(e){
		
		var record = e.detail;
		var objHref = record.href;
		
		//Update var inspecFormHref
		this.inspecFormHref = objHref;
		//var self = this;
		
		var qParams = {'responseos':'MXAPIINSPFORM'};
		
		$M.toggleWait(true);
		this.$.inspecFormResource.getAction('wsmethod:getRevision', qParams);
//			.then(function(e){
//				var newRecord = e.response;
//				newRecord.isrevision=true;
//				
//				if(newRecord.inspquestion) { 
//					for (var i=0; i<newRecord.inspquestion.length; i++) {
//						self.removeDoctypesReference(newRecord.inspquestion[i]);
//					}
//				}
//				 
//				self.mbo = newRecord;
//				self.revisedFrom = record;
//				self._setRevisionForm();
//				self._changePage('form');
//				self.isNewRecord = false;
//				self.$.formEdition.fire('done');
//				$M.toggleWait(false);
//					
//			}, function(error){
//				console.log('revision failed');
//				$M.toggleWait(false);
//			}
//		);
		
		//this.$.formEdition.createRevision(e);
	},
	
	_collapseInstructionInputArea : function(e) {
		//collapse instruction section
		if ($j('#inspectorsupmain_formEdition_inspectionformcollapse').find('iron-collapse')[0].opened == true){
			$j('#inspectorsupmain_formEdition_inspectionformcollapse')[0].toggleCollapse();
		}
	},
	
	_duplicateRecord: function (e) {
		var originalRecordData = e.detail;
		this.$.inspecFormResource.duplicateRecord(originalRecordData);
		this.$.formEdition.fire('setFormNameSelectAll');
	},
	
	_addUpdateQuestion : function(e) {
		//Grab the whole form record
		var recordData = e.detail;
		var deletedFields = [];
		/**
		 * APAR IJ35719
		 * Remove rowstamp if there's newer version at currentInspForm
		 * Considering the changes only come from question builder
		 * The newer version represents question title/description update,
		 * which is copied to question builder and might be updated as well
		 */
		 if (this.hasNewerQuestion(recordData)) {
			delete recordData._rowstamp;
		}
		$M.toggleWait(true);
		if (this.revisedFrom){
			recordData.isrevision = true;
		}

		var properties = {'href':this.$.formEdition.record.href,'object':'INSPQUESTION'};
		if (recordData.href){  // update existing question
			if(recordData.inspfield){
				for (var i = 0; i <recordData.inspfield.length; i++) {
					if(recordData.inspfield[i].isCopy){
						recordData.inspfield[i]._action='Add';
						if(recordData.inspfield[i].inspfieldoption){
							for (var n = 0; n < recordData.inspfield[i].inspfieldoption.length; n++) {
								if(recordData.inspfield[i].inspfieldoption[n].isDeleted){
									recordData.inspfield[i].inspfieldoption.splice(n,1);
								}
							}
						}
					}else if(recordData.inspfield[i].isDeleted && recordData.inspfield[i].inspfieldnum){
							recordData.inspfield[i]._action='Delete';
					}else{
						if(recordData.inspfield[i].inspfieldnum){
							recordData.inspfield[i]._action='Update';
							if(recordData.inspfield[i].inspfieldoption){
								for (var j =0 ; j <recordData.inspfield[i].inspfieldoption.length; j++) {
									if(recordData.inspfield[i].inspfieldoption[j].href){
										if(recordData.inspfield[i].inspfieldoption[j].isDeleted){
											recordData.inspfield[i].inspfieldoption[j]._action='Delete';}
										else if(recordData.inspfield[i].inspfieldoption[j].inspfieldoptionid){
											recordData.inspfield[i].inspfieldoption[j]._action='Update';}
										else{
											recordData.inspfield[i].inspfieldoption[j]._action='Add';
											}
									}
									else{
										if( (!recordData.inspfield[i].inspfieldoption[j].isDeleted) &&
												(!recordData.inspfield[i].inspfieldoption[j].description == '')){
											recordData.inspfield[i].inspfieldoption[j]._action='Add';
										} else {
											//Record was added in the list, but removed after, and it´s null. 
											//Should be removed from the updates
											recordData.inspfield[i].inspfieldoption.splice(j,1);
										}
									}
							}
						}
						}else if(!recordData.inspfield[i].isDeleted){
							//validate the options
							if(recordData.inspfield[i].inspfieldoption){
								for (var j =0 ; j <recordData.inspfield[i].inspfieldoption.length; j++) {
									if( (!recordData.inspfield[i].inspfieldoption[j].isDeleted) &&
									    (!recordData.inspfield[i].inspfieldoption[j].description == '')){
								        recordData.inspfield[i].inspfieldoption[j]._action='Add';
							        } else {
										//Option was added in the list, but removed after, and it´s null. 
										//Should be removed from the updates
										recordData.inspfield[i].inspfieldoption.splice(j,1);
									}
								}
							}
							recordData.inspfield[i]._action='Add';

						}
						else{
							// Creating an array of fields that need to be removed from recordData
							deletedFields.push(i);
						}
					}
				}
				// Removing fields from record Data that have been deleted by user while creating an inspection form
				if(deletedFields.length > 0) {
					recordData.inspfield = recordData.inspfield.reduce((acc, value, index) => {
						if (!deletedFields.includes(index)) {
							acc.push(value);
						}
						return acc;
					}, []);
				}
				this.$.inspformstacked.updateChildRecord(recordData, properties);
			}
		}else{	// create a new question
			if(recordData.inspfield){
				var i = recordData.inspfield.length;
				while(i--){
					if(!recordData.inspfield[i].isDeleted){
						recordData.inspfield[i]._action='Add';
						if(recordData.inspfield[i].inspfieldoption){
							var j = recordData.inspfield[i].inspfieldoption.length;
							while(j--){
								if(!recordData.inspfield[i].inspfieldoption[j].isDeleted){
									recordData.inspfield[i].inspfieldoption[j]._action='Add';}
								else{
									recordData.inspfield[i].inspfieldoption.splice(j,1);
								}
							}
						}
					}else{
						recordData.inspfield.splice(i,1);
					}
				}
				
			}
			if(recordData.sequence == null){ //sequence will already be set for groups
				recordData.sequence = this._getNewQuestionSequence();
				recordData.groupseq = this._generateGroupSeq(recordData);
			}
			this.$.inspformstacked.createChildRecord(recordData, properties);
			
		}
	},
	
	/**
	 * Saves updates to the Inspection Form Record.
	 */
	_updateInspectionForm : function (e) {
		var dataToUpdate = {};
		var self = this;
		dataToUpdate.name = e.detail.record.name;
		dataToUpdate.description_longdescription = tinymce && tinymce.get('inspectorsupmain_formEdition_instructionsRichtext_textarea').getContent();
		dataToUpdate.status = e.detail.record.status;
		dataToUpdate.type = e.detail.record.type;
		
		if(e.detail.record.inspquestion){
			dataToUpdate.inspquestion = [];
			for (var i = 0; i<e.detail.record.inspquestion.length; i++){
				if(e.detail.record.inspquestion[i]._action === 'Update'){
					dataToUpdate.inspquestion.push(e.detail.record.inspquestion[i]);
				}
			}
		}

		var responseProperties = 'inspectionformid,name,inspformnum,orgid';
		this.$.inspecFormResource.updateRecord(dataToUpdate, responseProperties, true).then(function(result) {
			// notify record has been updated
			var message = $M.localize('uitext','mxapiinspection','inspectionform_updated');
			$M.notify(message,$M.alerts.info);
			self.$.inspformstacked.refreshRecords();
			if(this.saveClicked){
				this.saveClicked = false;
				self._goBack();
			}
		}.bind(this), function(error) {
			$M.showResponseError(error);
		});
	},
	
	_resequenceQuestionsHandler: function(e){
		if (e && e.detail){
	  		this._resequenceQuestions(e.detail);
		}
	},
	
	// used to store a next event to be called after some other process (like a save or update)
	_setNextEvent: function(e){
		if (e.detail){
			this.nextEvent = e.detail;
		} else {
			this.nextEvent = null;
		}
	},
	
	_processNextEvent: function(e){
		// a record has been saved, are we supposed to do something else now (like go to question builder)?
		if (this.nextEvent !== null){
			var target = this;
			if(this.nextEvent.target){
				target = this.nextEvent.target;
			}
			if (this.nextEvent.event){
				if (this.nextEvent.param){
					target.fire(this.nextEvent.event, this.nextEvent.param);
				}else {
					target.fire(this.nextEvent.event);
				}
			}
			this.nextEvent = null;
		}
	},
	
	/**
	 * Listen to open-revision event
	 */
	_openRevision : function(e){
		$M.toggleWait(true);
		
		var form = e.detail;
//		if (form.revision > 1) {
//			this.requestFormRevisions(form.inspformnum, form.orgid);
//			this._changePage('revision');
//		}else {
//			this.formRevisionSet = [form];
//			this._changePage('revision');
//			$M.toggleWait(false);
//		}
		
		this.currentInspForm = form;
		this._changePage('revision');
		//this.$.revisionList.formRepresentative = form;
	},
	
	/**
	 * saveReson event from revision list 
	 */
	_saveReason: function(e) {
		var form = e.detail;
		var responseProperties = 'name, inspformnum, orgid';
		this.inspecFormHref = form.href;
		this.$.inspecFormResource.updateRecord(form, responseProperties, true);
	},
	
	/**********  LISTENERS End  *********/
	
	/**
	 * Find newer version of same question record
	 * @param {Object} question 
	 * @returns {Object} latest question
	 */
	hasNewerQuestion: function(questionRef) {
		// Get current list of questions
		const questions = this?.currentInspForm?.inspquestion || [];
		// Find matching question
		const matchingQuestion = questions.find(q => q.inspquestionid === questionRef.inspquestionid);
		// Find corresponding question and compare rowstamps
		if (matchingQuestion?._rowstamp && questionRef?._rowstamp) {
			return matchingQuestion._rowstamp > questionRef._rowstamp;
		}
		return false;
	},

  	_statusSetChange: function (newV) {
  	},
	
  	_refreshCurrentRecord : function(e){
  		if(!this.inspecFormHref){
  			return;
  		}

  		$M.toggleWait(true);
  		this.$.inspecFormResource.loadRecord();
  	},
  	
	_collapseGroupDescriptionAreas : function(e) {
		//collapse group description sections
		var myarray = $j('[id^=groupSection]');
  		for(var i=0;i<myarray.length;i++){
  			myarray[i].parentElement.parentElement.parentElement.remove();
  		}
	},
	
	_closeQuestionDialog : function(e) {
		//close question dialog
		$j('#questionBuilder_cancel_button').click();
	},
	
	_cloneQuestion: function(question, attributes){
		var clone = {};
		for(var i=0; i<attributes.length; i++){
			clone[attributes[i]] = question[attributes[i]];
		}
		return clone;
	},

	_resequenceQuestions: function(questions){	// only sends update to server if are changes!
		if (questions){
			this.$.formEdition.fire('closeAllGroupInfos', true);
			var curSeq = 0;
			var curGroup = 0;
			var curChildSeq = 0;
			var lastGroup = 0;
			var hasUpdates = false;
			for(var i=0; i< questions.length; i++){
				var question = questions[i];
				if (question._action){	// do update if there is an existing _action
					hasUpdates = true;
					if (question._action === 'Delete'){	// don't process a deleted record
						continue;
					}
				}
	  			if (question.groupid > 0){	// in a group
	  				if (question.sequence === 0){  // if seq = 0, we're on a group header, set groupseq = curGroup
						curSeq ++;	// increment curSeq for each header (once per group)
						curGroup = curSeq;
						curChildSeq = 0;
	  					if (question.groupseq !== curGroup){	// update groupseq if it doesn't match the group
							question.groupseq = curGroup;
							hasUpdates = true;
	  					}
	  				}
	  				if (question.groupid !== curGroup){	// update groups if they don't match (for headers and child questions)
		  				question.groupid = curGroup;	// only update only if necessary
		  				hasUpdates = true;
	  				}
	  				if (question.sequence !== 0){	// if sequence > 0 we're on a group question, so update groupseq and sequence
	  					curChildSeq ++;
	  					if (hasUpdates || question.sequence !== curChildSeq || question.groupseq !== this._generateGroupSeq(question)){	// if hasUpdates==true the group has changed - redo groupseq and sequence
		  					question.sequence = curChildSeq;	// set this before generating groupseq
		  					question.groupseq = this._generateGroupSeq(question);	// generate groupseq ('group' . 'sequence');
		  					hasUpdates = true;
	  					}
	  				}
	  				lastGroup = curGroup;
	  			} else {	// not in a group
	  				curSeq ++;
  					if (question.sequence !== curSeq || question.groupseq !== curSeq){	// only update only if necessary
	  					question.sequence = curSeq;
	  					question.groupseq = curSeq;		// not in a group, so groupseq = sequence
	  					hasUpdates = true;
					}
	  			}
	  			if (hasUpdates && !question._action){	// if there are updates and there's not already an action, set it to Update
  					question._action = 'Update';
	  			}
	  		}
			if (hasUpdates){	// update record if there are updates
				this._updateQuestions(questions);
			}
		}
	},
	
	_findQuestionIndex: function(questions, href){
		for(var i=0;i<questions.length;i++){
  			if(questions[i].href === href){
  				return i;
  			}
  		}
		return -1;
	},
	
	_updateQuestions: function(questions){
		var dataToUpdate = {};
		dataToUpdate.inspquestion = questions;
		var responseProperties = 'name';
		var self = this;
    	$M.toggleWait(true);
    	//delete id field when duplicating to prevent out of sync records with Long description
    	if(dataToUpdate){
	    	for(var y=0; y< dataToUpdate.inspquestion.length; y++){
	    		if(dataToUpdate.inspquestion[y]._action === 'Add'){
	    			if(dataToUpdate.inspquestion[y].inspquestionid){
	    				delete dataToUpdate.inspquestion[y].inspquestionid;	
	    			}
	    		}
	    	}	
    	} 
		this.$.inspecFormResource.updateRecord(dataToUpdate, responseProperties, true).then(function(result) {
			//self.$.inspformstacked.refreshRecords();

			self.$.inspecFormResource.loadRecord();
		}.bind(this), function(error) {
			$M.showResponseError(error);
			$M.toggleWait(false);
		});
	},
	
	_setRevisionForm: function() {
		this.$.formEdition.setRevision();
	},
	
	_getDuplicatedFormName: function(forms, name){
		var copySuffix = $M.localize('uitext','mxapiinspection','copy');
		var copyIndex = name.indexOf('('+copySuffix);
		if(copyIndex>=0){ //this record has copy in the name, strip it to get base
			name = name.substring(0,copyIndex).trim();
		}
		var nextNbr = 1;
		//search all question to see if this exists already
		for(var i=0; i<forms.length; i++){
			var cfOrigName = forms[i].name;
			var cfName = cfOrigName;
			copyIndex = cfName.indexOf('('+copySuffix);			
			//this record has copy in the name, strip it to get to base
			cfName = cfName.substring(0,copyIndex).trim();
			if(cfName === name){
				if(copyIndex>=0){ 
					//now find if the string already contained a copy number, if so, store its number
					var copySuffixLth = copySuffix.length;
					var copyNbrIdx = copyIndex+1+copySuffixLth;
					var closingIdx = cfOrigName.indexOf(')',copyNbrIdx);
					var newNbr = cfOrigName.slice(copyNbrIdx,closingIdx).trim();
					if(!newNbr){
						newNbr = 1;
					}
					var tempNbr = parseInt(newNbr)+1;
					if(tempNbr>nextNbr){
						nextNbr = tempNbr;
					}
				}
			}
		}
		var displayNbr = '';
		if(nextNbr>1){
			displayNbr = ' '+nextNbr;
		}
		name = name+' ('+$M.localize('uitext','mxapiinspection','copy')+displayNbr+')';
		return name;
	},
	
	_changePage: function (/*index*/ pageName) {
		if (pageName === 'revision') {
			this.requestFormRevisions(this.currentInspForm.inspformnum, this.currentInspForm.orgid);
		}
		this.$.formEdition.formFlag=false;
		this.selectedPage = pageName;
		document.body.scrollTop = 0;
		this._fixButtons(pageName);
		this._fixQuestionButtons(pageName);
	},
		
	_getFootberNatButtons : function(e){
   	 //Screen navigation buttons 
			return [ {'id':'goback','action':'true','label':this.localize('uitext','mxapibase','Back'),'event':'goback','icon':'chevron-left'},
        // 	{'id':'preview','action':'true','label':this.localize('uitext','mxapibase','Preview'),'event':'preview','icon':'Maximo:View'},
           	{'id':'done','default':true,'label':this.localize('uitext','mxapibase','save'),'event':'formSaveButton','icon':'Maximo:Save'}];
	},
 	
	_getQuestionButtons : function(e){
   	 //Question buttons 
			return [ {'id':'inspectorsupmain_formEdition_newQuestion','action':'true','label':this.localize('uitext','mxapibase','add_question'),'event':'addNewQuestion','icon':'Add-New'}];
	},
	
	_getRevisionListNavButtons: function() {
		var formLibButton = {
			'id':'goback',
			'action':'true',
			'label':this.localize('uitext','mxapiinspection','form_library'),
			'event':'goback',
			'icon':'chevron-left'	
		};
		return [formLibButton];
	},
 	
	_fixQuestionButtons: function(tab){
		var buttons = this._getQuestionButtons();
		var context = this;
		switch(tab){
			case 'list':
			case undefined:
				buttons = null;
				this.revisedFrom = null;
				break;
			case 'form':
				context = this.$.formEdition;
				buttons[0].disabled = !this.$.formEdition._saveAllowed();
				break;
			default:
				break;
		}
		if(buttons){
			var questButton =$j(this.$.formEdition).find('#inspectorsupmain_formEdition_newQuestion');
			if(questButton[0] != null){
				questButton[0].set('disabled', buttons[0].disabled);
			}
		}
	},
  	
	/**
	 * Set footer buttons according to component/page 
	 * hosted by this.$.pages
	 */
	_fixButtons: function(tab){
		var buttons = this._getFootberNatButtons();
		var context = this;
		switch(tab){
			case 'list':
			case undefined:
				buttons = null;
				this.revisedFrom = null;
				break;
			case 'form':
				context = this.$.formEdition;
				//This will be 2 when Preview is added back in
				//buttons[2].disabled = !this.$.formEdition._saveAllowed();
				buttons[1].disabled = !this.$.formEdition._saveAllowed();
				break;
//			case 2:
//				context = this.$.deffilter;
//				buttons[2].disabled = !this.$.deffilter._nextAllowed();
//				break;
//			case 3:
//				context = this.$.saveit;
//				buttons.pop();
//				buttons.push({'id':'savedataset','disabled':!this.$.saveit._saveAllowed(),'default':true,'label':this.localize('uitext','mxapibase','save'),'event':'_save','icon':'formatting:save'});
//				break;
			case 'revision':
				buttons = this._getRevisionListNavButtons();
				break;
			default:
				break;
		}
		$M.workScape.updateFooterToolbar(buttons, context);
	},
	
	_getNewQuestionSequence: function(){
		var seq = 1;
		var form = this.$.formEdition.record;
		if (!form.inspquestion){ // no other questions, sequence = 1
			return seq;
		}else{	// has other questions, calculate sequence
			for(var i=0; i< form.inspquestion.length; i++){
				var q = form.inspquestion[i];
				if (q.sequence === 0 || !q.groupid){	// group header or no group
					seq ++;
				}
			}
		}
		return seq;
	},
	
	/**
	 * Fetch revisions of the same form
	 * 
	 * @param inspformnum
	 * @param orgid
	 */
	requestFormRevisions: function(inspformnum, orgid) {
		
		var maximoQbe = new MaximoQbe();
		maximoQbe.setEquals('inspformnum', inspformnum).setEquals('orgid', orgid);
		var qbeObject = maximoQbe.toJSON();
		this.$.inspformrevisioncollection.queryWhere = qbeObject;
		
		this.$.inspformrevisioncollection.refreshRecords().then(function() {
			$M.toggleWait(false);
		});
	},
	
	/**
	 * Find previous revision in the current 
	 * Return the form of the previous revision
	 */
	fetchPreviousRevision: function() {
	    var revisionSet = this.formRevisionSet;
	    //var currentHref = this.currentInspForm.href;
	    var currentRev = this.currentInspForm.revision;
	    var previousRev = currentRev-1;
	    revisionSet = revisionSet.filter( function(form) { return form.revision === previousRev;} );
	    if (revisionSet.length) {
	        return revisionSet[0];
	    }
	    return null;
	},

	/**
	 * reload revisions 
	 * 
	 */
	 _refreshRevision: function() {
		this.$.inspformrevisioncollection.refreshRecords().then(function() {
			$M.toggleWait(false);
		}, function() {
			$M.toggleWait(false);
		});
	},
		
	
	/***********  COLLECTION HANDLERS Begin  **********/
	
	/**
  	 * mx-collection refreshRecords success handler
  	 */
	_handleRecordDataRefreshed: function (objs) {
		this.$.formList.dataSetRefresh();
		this.$.formList.selectRecord(this.highlightedId);
		this.highlightedId = null;
		this._refreshCurrentRecord();
		this.$.formEdition.formFlag=true;
		$M.toggleWait(false);
	},
	
	/**
	 * mx-collection generic error handler
	 */
	_handleError: function (e) {
		var error = e?.detail?.request?.response?.Error?.message ?? e?.detail?.Error?.message;
		if (error) {
			$M.notify(error, $M.alerts.error);
		}
		console.error(`Error. ${error ?? ''}`);
		
		//reset button click in case of error
		this.saveClicked = false;
		
		this.$.formList.dataSetRefresh();
		$M.toggleWait(false);
	},
	
	/**
	 * getNewRecordData success handler
	 * Creating brand new inspection form
	 */
	_handleNewRecordCreated : function(e){
		var recordData = e.detail;
		//Indicator to help in goback navigation
		recordData.isDraft = true;
		this.currentInspForm = recordData;
		this._changePage('form');
		$M.toggleWait(false);
	},
	
	/**
	 * mx-collection createRecord success handler 
	 */
	_handleRecordCreated: function (e) {
		var recordData = e.detail;
		//this.inspectionFormUrl = recordData.href;
		delete this.currentInspForm.isDraft;
		this.inspecFormHref = recordData.href;
		this.$.formEdition.isNewRecord = false;
		this._collapseInstructionInputArea(e);
		
		this.$.inspformstacked.refreshRecords();

		this.currentInspForm = recordData;
		this.requestFormRevisions(recordData.inspformnum, recordData.orgid);
		//var message = $M.localize('uitext','mxapiinspection','inspectionform_created',[recordData.name]);
		//$M.notify(message, $M.alerts.info);
		//$M.toggleWait(false);
			
		if (this.selectedPage !== 'list'){
			if(this.saveClicked && this.saveClicked===true){
				this.saveClicked = false;
				this._goBack();	
			}
		}
		this.highlightedId = recordData.inspectionformid;
//		this.$.formEdition.clearForm();
		this._processNextEvent(e);	// do we have to open the question builder?
	},
	
  	
	/**
	 * mx-collection createRecord error handler 
	 */
	_handleRecordCreatedError: function() {
		//TODO
	},
	
	/**
	 * mx-collection saveNewRecord success handler
	 */
	_handleRecordSaved: function (e){
		this.isNewRecord = false;
		var recordData = e.detail;
		
		this.$.inspformstacked.refreshRecords();
		//var message = $M.localize('uitext','mxapiinspection','inspectionform_created',[recordData.name]);
		//$M.notify(message, $M.alerts.info);
		$M.toggleWait(false);
		
		if (this.selectedPage != 'list'){
			this._goBack();
		}
		this.$.formEdition.clearForm();
	},
	
	/**
	 * createChildRecord success handler
	 */
	_handleChildRecordCreated : function(e){
		//this.$.inspformstacked.refreshRecords();
		if (this.inspecFormHref) {
			this.$.inspecFormResource.loadRecord();
		}		
	},
	
	/**
	 * Handle success response for
	 * inspformrevisioncollection collection
	 */
	_formRevisionFetchSuccessHandler: function() {
		$M.toggleWait(false);
		if (!this.formRevisionSet.length && this.selectedPage === 'revision') {
			this._goBack();
		}
	},
	
	/**
	 * Handle fail response for
	 * inspformrevisioncollection collection
	 */
	_formRevisionFetchFailHandler: function() {
		$M.toggleWait(false);
	},
	
	/**
	 * mx-resource loadRecord success handler
	 */
	_loadSuccessHandler: function (e) {
		var resource = e.detail;
		if (resource) {
			this.currentInspForm = resource;
		}
		$M.toggleWait(false);
	},
	
	/**
	 * mx-resource loadRecord error handler
	 */
	_loadErrorHandler: function (e) {
		this.currentInspForm = null;
		$M.toggleWait(false);
	},
	
	/**
	 * mx-resource duplicateRecord success handler
	 */
	_handleRecordDuplicatedSuccess: function (e) {
		var duplicatedRecordData = e.detail;
		var self = this;
		
		try {
			$M.toggleWait(true);
			
			this.currentInspForm = duplicatedRecordData;
			
			self.$.formEdition.formName = this._getDuplicatedFormName(this.groupedData, duplicatedRecordData.name);
		    if (duplicatedRecordData.inspquestion){	// duplicated form comes back with questions in no particular order
				duplicatedRecordData.inspquestion.sort(function(a, b){return a.groupseq - b.groupseq;});	// sort by groupseq
		    }

		    if(duplicatedRecordData.inspquestion) { 
			    for (var i=0; i<duplicatedRecordData.inspquestion.length; i++) {
			    	this.removeDoctypesReference(duplicatedRecordData.inspquestion[i]);
			    }
		    }
		    
			self.$.formEdition._questionsChanged(duplicatedRecordData.inspquestion);
			self.$.formEdition.fire('done');
			self._changePage('form');		
		}catch(e){
			$M.toggleWait(false);
		}

	},
	
	/**
	 * mx-resource duplicateRecord error handler
	 */
	_handleRecordDuplicatedError: function (e) {
		var response = e?.detail?.request?.response?.Error?.message ?? e?.detail?.Error?.message;

		if (response) {
			$M.notify(response, $M.alerts.error);
		}
		console.error(`Error duplicating form. ${response ?? ''}`);
		$M.toggleWait(false);
	},
	
	/**
	 * mx-resource updateRecord success handler
	 */
	_handleRecordUpdated: function(e) {
		//this.$.inspecFormResource.loadRecord();
		var recordData = this.$.inspecFormResource.resourceData;
		this.requestFormRevisions(recordData.inspformnum, recordData.orgid);
		this._processNextEvent(e);	// do we have to open the question builder?
	},
	
	/**
	 * mx-resource updateRecord error handler
	 */
	_handleRecordUpdateError: function (e) {
		//TODO
	},
	
	/**
	 * mx-resource getAction success handler for getRevision
	 */
	createRevisionSuccessHandler: function(e) {
		var newRecord = e.detail;
		newRecord.isrevision=true;
			
		if(newRecord.inspquestion) { 
			for (var i=0; i<newRecord.inspquestion.length; i++) {
				this.removeDoctypesReference(newRecord.inspquestion[i]);
			}
		}
			 
		this.revisedFrom = JSON.parse(JSON.stringify(this.currentInspForm));
		this.currentInspForm = newRecord;
		this._setRevisionForm();
		//this._changePage('form');
		//this.isNewRecord = false;
		this.$.formEdition.isNewRecord = true;
		// Fire and listen to from inspection-form component 
		this.$.formEdition.fire('done');
		// Defect #285075
		//$M.toggleWait(false);

	},
	
	/**
	 * mx-resource getAction error handler for getRevision
	 */
	createRevisionErrorHandler: function(e) {
		var response = e?.detail?.request?.response?.Error?.message;
		if (response) {
			$M.notify(response, $M.alerts.error);
		}
		console.error(`Error creating revision. ${response ?? ''}`);
		$M.toggleWait(false);
	},
	
	/**
	 * mx-resource delete success handler for deleteRecord
	 */
	_deleteSuccessHandler: function(e) {
	    $M.toggleWait(false);
	    var previousRevision = this.fetchPreviousRevision();
	    //this.$.inspformrevisioncollection.refreshRecords();
	    this.requestFormRevisions(this.currentInspForm.inspformnum, this.currentInspForm.orgid);
	    //set currentInspForm and inspecFormHref with the previous revision
	    if (previousRevision){
        this.currentInspForm = previousRevision;
        this.inspecFormHref = previousRevision.href;
	    } else {
	      this.currentInspForm = null;
        this.inspecFormHref = null;
	    }
	    this.$.inspformstacked.refreshRecords();
	},
	
	/**
	 * mx-resource delete error handler for deleteRecord
	 */
	_deleteErrorHandler: function(e) {
	  $M.toggleWait(false);
	  $M.notify(e.detail,$M.alerts.error);
    console.log(e.detail);
	},
	
	/**
	 * mx-collection formstatusdomain refreshrecords success handler
	 */
	_handleStatusDomainSet: function(e) {
		$M.toggleWait(true);
		if (this.domainStatusCollection.length > 0) {
			this.formStatusSet = this.domainStatusCollection[0].synonymdomain;
		}else {
			this.formStatusSet = [];
		}
		this.questionFlag = false;
		
		this.$.inspformstacked.refreshRecords().then(function() {
			$M.toggleWait(false);
			this.questionFlag = true;
		});;
	},
  	
  	/**
  	 * mx-collection formstatusdomain refreshrecords error handler
  	 */
  	_handleStatusDomainSetError: function(e) {
  		$M.toggleWait(true);
  		console.log('Problem loading data');
    	this.formStatusSet = [];
    	this.$.inspformstacked.refreshRecords();
  	},
  	
  	/**
  	 * mx-collection formTypeList Success domain response about inspfield types and groups 
  	 */
  	_typeDomainSetSuccessHandler: function(e) {
  		if (this.domainFormTypeSet.length) {
			this.formTypeSet = this.domainFormTypeSet[0].synonymdomain;
		}else {
			this.formTypeSet = [];
		}
  	},
  	
  	/**
  	 * mx-collection formTypeList Error domain response about inspfield types and groups 
  	 */
  	_typeDomainSetErrorHandler: function(e) {
  		this.formTypeSet = [];
  	},

    /**
     * Enable wait layer, initiated on refresh of collection
     */
  	_loading : function() {
  		$M.toggleWait(true);
  		//To load spinner making count as -1 , this means collections is being refreshed
  		this.$.formList._resultCounter=-1; 
  	},
	/***********  COLLECTION HANDLERS End  **********/
	/**
    *  Handle mobilefeatures according to the 
    * "mxe.mobile.inspection.mobilefeatures" System Property 
    */
	 _handleMobileFeatures: function(e) {
		if(e.detail){
			this.mobileFeatures = e.detail == true;
		};
	},

	/**
	 *  Handle an error in the fetch for system properties
	 */
	 _handleFeaturesError: function(e) {
		console.log(e.detail);
	},

	_handleMVIEnableSuccess: function(e) {
		if(e.detail){
			this.mviEnabled = e.detail == true;
		};
	},

	/**
	 *  Handle an error in the fetch for system properties
	 */
	_handleMVIEnableError: function(e) {
		console.log(e.detail);
	},

	/**
     *  Handle mobilefeatures according to the 
     * "mxe.mobile.inspection.features.signature" System Property 
     */
 	_handleSignatureFeature: function(e) {
		if(e.detail){
			this.signatureFeature = e.detail == true;
		}
	},

	/**
     *  Handle mobilefeatures according to the 
     * "mxe.mobile.inspection.features.multiselect" System Property 
     */
 	_handleMultiSelectFeature: function(e) {
		if(e.detail){
			this.multiSelectFeature = e.detail == true;
		}
	},

	/**
	 * Success handler for inputValidation prop fetch
	 */
	_handleInputValidationSuccess: function(e) {
		let featureEnabled = false;
		if(e.detail){
			featureEnabled = e.detail == true;
		}
		this.inputValidationFeature = featureEnabled;
	},

	/**
	 * Error handler for inputValidation prop fetch
	 */
	_handleInputValidationError: function(event) {
		console.error(event);
		this.inputValidationFeature = false;
	},

	/**
	 * Success handler for inputValidation prop fetch
	 */
	 _handleLookupResponseSuccess: function(e) {
		let featureEnabled = false;
		if(e.detail){
			featureEnabled = e.detail == true;
		}
		this.lookupResponseFeature = featureEnabled;
	},

	/**
	 * Error handler for inputValidation prop fetch
	 */
	 _handleLookupResponseError: function(event) {
		console.error(event);
		this.lookupResponseFeature = false;
	},

	/**
     *  Handle mobilefeatures according to the 
     * "mxe.mobile.inspection.features.enablereview" System Property 
     */
	 _handleFormReviewFeature: function(e) {
		let formReview = false;
		if(e.detail){
			formReview = e.detail == true;
		}
		this.formReviewFeature = formReview;
	},


});
