/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18, 5737-M66
 * (C) Copyright IBM Corp. 2018,2021 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication, or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
Polymer({
	is: 'maximo-inventory-physicalcount',
  	behaviors: [BaseComponent],
  	listeners: {
        'maximo-table-collection-sort' : 'collectionOrderByChanged',
        'custom-conditional-formatting':'_handleConditionalFormatting',
        'maximo-table-filter-data':'_filterData',
        'loadInvBalTableData':'_loadInvBalTableData',
        'recordSaved':'_recordSaved',
        'spinner-value-changed': '_physCountChanged'
	},
  	properties : {
  		record: {
    		type:Object,
  			value:{}
		},
		
		hierarchicalcolumns: {
			type: Array,
			  value: []
		},
		
		additionalParams: {
			type: Array,
			value: []
		},
		
		collectionOrderBy: {
			type: String,
			value: '%2Bnextphycntdate'
		},
		/**
    	 * Flag for table initialization
    	 */
    	isTableInitialized : {
    		type: Boolean,
    		value: false
    	},
    	invBalData: {
			type: Array,
			value: []
		},
		invBalDataArray: {
    		type: Array,
    		value: []
    	}, 
    	filterData : {
    		type: Boolean,
    		value: false
    	},
    	/**
    	 * Flag for table initialization
    	 */
    	isTableInitialized : {
    		type: Boolean,
    		value: false
    	},

		maxSynonymDomainValue: {
			type: String,
			value: 'ACTIVE'
		}
  	},
  
	_handleGetValueFromDomain: function(e) {
		this.maxSynonymDomainValue = e.detail;
	},

	_handleGetValueFromDomainError: function(e) {
		console.log(e);
	},
  	
  	/**
  	 *  Set Additional Param for Invbalance Data
  	 */
  	setAdditionalParam: function(invBal, maxSynonymDomainValue){
		if(invBal.record){
    	    var siteid = invBal.record.siteid;
			var loc = invBal.record ? invBal.record.location:'';
			var additionalparams=[];
			additionalparams.push('oslc.where=item.status%20in%5B%22'+maxSynonymDomainValue+'%22,%22PENDOBS%22%5Dand%20stagingbin=%220%22and%20location=%22'+loc+'%22and%20siteid=%22'+siteid+'%22and%20item.rotating=%220%22');
			this.$.physicalcountcollection.additionalParams=additionalparams;
		}
  	},
  	
  	/**
	 * Loading the InvBalance table
	 */  	
  	_loadInvBalTable: function(invBal,pageName){
	      var self = this;  
          var domainPromise = this.$.maxsynonymdomain.fetchValue();
			domainPromise.then(
				function() {
					 self.setAdditionalParam(invBal, self.maxSynonymDomainValue);
				},
			);
          if(invBal.$.physicalCount && pageName !=='invItemList'){
            	 this.async(function() {
        	     	self.$.physicalCountTable._clearFilter();
           	 },100);
          }
          self._initLoadTable();
          this.$.physicalcountcollection.refreshRecords().then(function(result){
       		 self._tableStyle(self);
          }); 
          
          this.reset();
       },
       
       /**
     	 * Initialization of table with columns
     	 */
       _initLoadTable: function(){

    		if(this.isTableInitialized) {
      			return;
      		}
     		 var hierarchicalcolumns = []; 		 
     		 hierarchicalcolumns =
         	 [{'attribute':'physicalcountitemnum','filterattribute':'itemnum','sortable':true, 'sortattribute': 'itemnum','title':$M.localize('uitext', 'mxapistoreloc', 'id'),'hyperlink' : true,
        				'listener' : 'inventory-detail','flexGrow': '1','enablescan':true},
  		  		{'attribute':'physicalcountdescription', 'filterattribute':'item.description', 'title':$M.localize('uitext', 'mxapistoreloc', 'description'),'flexGrow': '2'},
  		  		{'attribute':'physicalcountbinnum','filterattribute':'binnum','sortable':true, 'sortattribute': 'binnum', 'title':$M.localize('uitext', 'mxapistoreloc', 'bin'),'flexGrow': '0.7'},
  		  		{'attribute':'physicalcountlotnum','filterattribute':'lotnum','sortable':true, 'sortattribute': 'lotnum', 'title':$M.localize('uitext', 'mxapistoreloc', 'lot'),'flexGrow': '0.7'},
  		  		{
  		  			'attribute':'nextphycntdate',
  		  			'sortable':true,
  		  			'filterattribute':'nextphycntdate',
  		  			'title':$M.localize('uitext', 'mxapiinvbal', 'countdue'),
  		  			'flexGrow': '1',
  		  			'type':'date',
  		  			'conditionalformatting':'{"customcondition": "true","attribute":"nextphycntdate"}',
  		  	     	'format':$M.maxauth.whoami.dateformat.toUpperCase(),
				    'sortattribute': 'nextphycntdate',
  		  		},
  		  		{'attribute':'','filterattribute':'physcnt','title':$M.localize('uitext', 'mxapiinvbal', 'newcount'),'flexGrow': '1.5', 'numericspinnerattribute':'physcnt'},
  		  		{'attribute':'physicalcountphyscnt','filterattribute':'physcnt','title':$M.localize('uitext', 'mxapiinvbal', 'lastcount'),'flexGrow': '0.5'},
  		  		{
  		  			'attribute':'physcntdate_localized',
  		  			'filterattribute':'physcntdate',
  		  			'title':$M.localize('uitext', 'mxapiinvbal', 'countlastdate'),
  		  			'flexGrow': '1',
  		  			'type':'date',
  		  			'format':$M.maxauth.whoami.dateformat.toUpperCase() + ' ' + $M.maxauth.whoami.timeformat
  		  		}
  		  	 ];
     		 if(!this.dataHost._isSigOptionGranted('MXAPIINVBAL', 'ENABLECOLUMN')){
     			hierarchicalcolumns.splice(6,1);
     		}
     		this.set('hierarchicalcolumns', hierarchicalcolumns);
     		this.isTableInitialized = true;
     	 },
     	 
     	_tableStyle: function(self){
 	    	var tbody = this.$.physicalCountTable.$.tbody;
 	    	if(tbody) {
 	    		var topPageWidth = 255;
 	 	    	if(navigator.userAgent.toLowerCase().indexOf('firefox') > -1){
 	   				topPageWidth = 260;
 	   			}
 	    		tbody.style.maxHeight = $j(window).height() - topPageWidth + 'px';	
   			}
 	    },
	
	/**
	 * Function call for Sorting 
	 */
	collectionOrderByChanged: function(e) {
		var self=this;
		e.target.collection.refreshRecords();
	},
	
	/**
	 * Callback function for maximo-table
	 */
	 _actionCallback: function(record){
		 var self=this;
			 if (record.reconciled){
				 return;
			 } 
			 else{
				 return record.physicalcountphyscnt;
			 }
	 },
	 
	 /**
	  * Custom Conditional formatting
	  */
	 _handleConditionalFormatting: function(e) {
	        var record = e.detail.record;
	        var datacolumn = e.detail.datacolumn;
	        var dateformat = 'YYYY-MM-DD';
	        var currentDate = new Date(moment().format(dateformat));
	        var styleObject = JSON.parse(datacolumn.conditionalformatting);
	        var columnData;
	        var attribute = styleObject.attribute? styleObject.attribute : datacolumn.attribute;
	        if(attribute){
	            columnData = $M.getRelatedData(record, attribute);
	        }           
	        if(datacolumn.type && datacolumn.type==='date' && columnData){
	            columnData = new Date(moment(columnData).format(dateformat));
	            if(columnData < currentDate) {
	                this.async(function(){
	                	$j('#tabledata_'+attribute+record._id.replace(/-/g, "_").replace(/~/g, "_")).css({'color':this.getComputedStyleValue('--Primary-orange50'), 'font-weight':'bold'});   
	                },100);
	            }
	        }
	    },
	    
	    _physCountChanged:function(e){
			var self = this;
			if(document && document.activeElement) {
				document.activeElement.blur();
			}
			setTimeout(function(){
				if(e.detail.record.selectedIndex.length) {
					var record=e.detail.rowrecord;
					if(record){
						var physcnt = e.detail.target.value;
						if(physcnt !== undefined) {
							var params = {
								itemnum : record.physicalcountitemnum,
								location : record.location,
								binnum : record.physicalcountbinnum,
								lotnum : record.physicalcountlotnum,
								href : HTMLEncoder.encode(record.href),
								itemsetid:record.itemsetid,
								siteid:record.siteid,
								physcnt: parseFloat(physcnt)
							};
							record.physcnt = physcnt;
							self.invBalData[record.invbalancesid] = params;
							e.detail.isrefreshCollection = false;
							self.fire('disable-sidemenu',true);
							$j(self.$.save).removeAttr("disabled");
							$j(self.$.reset).removeAttr("disabled");
						}
					}
				}
			}, 200);
	    },
	    
	    /**
	     *  Bulk Update for count
	     */
	    _handleRecordSaved :function(e){
	    	var self=this;
	    	this.invBalData.forEach(function(item){
	    		if(item.physcnt!==undefined && !(isNaN(item.physcnt))){
	    			self.invBalDataArray.push(item);
	    		}
	    	});
	    	if(this.invBalDataArray) {
	    		this.$.physicalcountcollection.createRecordBulk(this.invBalDataArray,'wsmethod:adjustPhysicalCount',true)
		    	.then(function(){
		    		self.$.physicalcountcollection.refreshRecords();
		    		self.reset();
		    		$M.notify($M.localize('uitext','mxapiinvbal','countsupdated'));
		    	});
	    	}
	    	self.fire('disable-sidemenu',false);
	    },
	    
	    /**
	     *  For previewReconciliation data save
	     */
	    _previewReconciliation :function(e){
	    	this._handleRecordSaved(e);
	    	$M.setSubRoute('storeroomdetail/reconcilition');
	    	$j(this.dataHost.$.reconcile).show();
	    	if(this.isAuthorized('MXAPIINVBAL:READ')){
	    		$j(this.dataHost.$.physicalCount_section).hide();
	        	$j(this.dataHost.$.addInventoryLink).hide();
	        	$j(this.dataHost.$.takeActionLink).hide();
	        	$j(this.dataHost.$.balance_section).hide();
			}
        	this.dataHost.$.reconciliation._loadReconcileTable(this.dataHost);
	    },
	    
	    /**
	     * Reset record
	     */
	    _handleRecordReset : function(){
	    	$M.showDialog(this, 'physicalCountId', null, 'maximo-physicalcount-resetdialog', false, null, {'invBalData':self.invBalDataArray ,'tab':'reset' });
	    },
	    
	    _filterData:function(){
	    	var self=this;	    	
	    	if(this.invBalData.length>0){
	    		this.invBalData.forEach(function(item,index){
		    		self.invBalDataArray.push(self.invBalData[index]);
		    	});
	    					
	    	$M.showDialog(this, 'physicalCountId', null, 'maximo-physicalcount-savedialog', false, null, {'invBalData':self.invBalDataArray , 'MsgText':$M.localize('uitext','mxapiinvbal','physicalcountconfirmsg'),'tab':'count' });
	    	}
	    },

	    
	    /**
	     * Function call when click on ConfirmBox cancel 
	     */
	    _loadInvBalTableData:function(e){
	    	var invBal=$M.workScape.querySelector('#storeroomDetail_physicalcount');
	    	invBal.record=e.detail[0];
	    	this._loadInvBalTable($M.workScape.querySelector('#storeroomDetail_physicalcount'));
	    	this.reset();
	    },
	    
	    /**
	     * Function call when click on ConfirmBox save 
	     */
	    _recordSaved:function(e){
	    	var invBal=$M.workScape.querySelector('#storeroomDetail_physicalcount');
	    	invBal.invBalData=e.detail;
	    	this._handleRecordSaved(e);
	    	this.reset();
	    },
	    
	    /**
	     * Reset Table
	     */
	    reset:function(){
	    	this.$.physicalCountTable.isrefreshCollection=true;
	    	this.invBalDataArray=[];
	    	this.invBalData=[];
	    	this.$.physicalCountTable.selectedIndex=[];
	    	this.$.save.setAttribute("disabled",true);
	    	this.$.reset.setAttribute("disabled",true);
	    	
	    },
	   
	    _collectionRefreshErrorHandler: function (ev){
	    	// Notify error
	        var error = ev.detail.Error;
	      if(error && error.message) {
	        	$M.notify(error.message, $M.alerts.error);
	       }
	          $M.toggleWait(false);
	    } 
	
	
});

