/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18, 5737-M66
 * (C) Copyright IBM Corp. 2016,2021 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication, or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
Polymer({
	is: 'maximo-submitsr-review',
  	behaviors: [BaseComponent,ServiceRequestBehavior],
    properties: {
		label: {
			type: String,
			value: 'Panel'
		},
		icon: {
			type: String,
			value: 'title'
		},		
		title: {
			type: String,
			value: 'title'
		},
		
		priorityDefault:{
			type: Number
		},
		
		priorityHigh:{
			type: Number
		},
		
		requestorName: {
			type: String
		}, 
		dataList :{
			type: Object
		},
		/**
    	 * flag for checking double click
    	 */
    	isClickedOnce : {
    		type: Boolean,
    		value: false
    	},
    	
    	/**
         * store the  long description component 
         */
    	LongDescRTE : {
            type: Object 
        },
        
    	
	},
	
	_handleDefaultPriority: function(e) 
	{
		this.priorityDefault = parseInt(e.detail);
	},
	
	_handleHighPriority: function(e)
	{
		this.priorityHigh = parseInt(e.detail);
	},
	
	ready: function()
	{
		try {
			this.$.defaultPrior.fetchProperty();
			this.$.highPrior.fetchProperty();
			if($M.screenInfo.device === 'desktop'){
				$j(this.querySelector('.reviewMain')).addClass('flex-horizontal-with-ratios');
				$j(this.querySelector('.titlePanel')).addClass('flexchild leftPanel');
				$j(this.querySelector('.dataPanel')).addClass('flexchild rightPanel');
				this.querySelector('.submitDivPhone').style.display='none';
			}
			else{
				$j(this.querySelector('.templateHeader')).addClass('templateHeaderPhone');
				$j(this.querySelector('.reviewMain')).addClass('reviewMainPhone');
				this.querySelector('.submitDiv').style.display = 'none';
				this.querySelector('.triangleLeft').style.display ='none';
			} 
		} catch(err) {
			this._displayUnexpectedError(err);
		}

		
	},
	attached : function(){
		try {
			if($M.screenInfo.device === 'desktop'){
				$j(this.querySelector('.dataPanel')).addClass('dataPanelDesktop');
			}
		} catch(err) {
			this._displayUnexpectedError(err);
		}

		
	
	},
	 
	goBack: function() {
		return 1;
	},
	initPage: function() {
		try {
			this.$.checkUrgentDesktop.checked = false;
			this.$.checkUrgent.checked = false;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	renderPage: function(submitInfo) {
		try {
			var stackData = [];
			for (var prop in submitInfo) {
				if (submitInfo.hasOwnProperty(prop)) {
					if(submitInfo[prop] && submitInfo[prop].contents){
						
						if(prop === 'affectedPerson') {
							stackData.push({'step':prop, 'topic': $M.localize('uitext','mxapisr',prop), 'data':submitInfo[prop].contents.personName,  
								person:{
									personId: submitInfo[prop].contents.personId , 
									personName: submitInfo[prop].contents.personName, 
									personEmail: submitInfo[prop].contents.personEmail,
									personPhone: submitInfo[prop].contents.personPhone
								}    
						});
							
						}else if (submitInfo[prop].contents.description) {
							if(prop === 'Location') {
								stackData.push({'step':prop,'topic': $M.localize('uitext','mxapisr',prop), 'data':submitInfo[prop].contents.description, 'breadcrumb':submitInfo[prop].contents.breadcrumb});
							}
							else if (prop === 'Attachment') {
								stackData.push({'step':prop, 'topic': $M.localize('uitext','mxapisr',prop), 'data':submitInfo[prop].contents.description, 'files':submitInfo[prop].contents.files});
							}
							else{
								stackData.push({'step':prop, 'topic': $M.localize('uitext','mxapisr',prop), 'data':submitInfo[prop].contents.description});
							}
						}else{
							stackData.push({'step':prop, 'topic': $M.localize('uitext','mxapisr',prop), 'data':'--'});
						}
					}
				}
			}
			stackData.reverse();
			this.dataList = stackData;
			this.requestorName = submitInfo['affectedPerson'].contents.personName
			//this.$.stackDatalist.notifyResize(); 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	_onSubmit: function() {
		try {
			$j('.submitButtonDiv').find('.submitButton').attr('disabled', 'disabled');
			var priority = this.priorityDefault;
			
			if ($M.screenInfo.device === 'desktop') {
				if (this.$.checkUrgentDesktop.checked) {
					priority = this.priorityHigh;
				}
			} else {
				if (this.$.checkUrgent.checked) {
					priority = this.priorityHigh;
				}
			}
			
			this.fire('sr-submit', priority);
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	enableButtons: function() {
		try {
			$j('.submitButtonDiv').find('.submitButton').removeAttr("disabled");
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	_showLocationLabel : function(step){
		if(step === 'Location'){
			return true;
		}
		else{
			return false;
		}
	},
	_showFileLabel : function(step) {
		if(step === 'Attachment'){
			return true;
		}
		else{
			return false;
		}
	},
	_getFileIndex : function(index) {
		return index + 1;
	},
	
	_hideAdditionalInformation: function(item){
	   return 	(item.step=='AdditionalInformation' && item.data != '--' ) ||(item.step=='affectedPerson');
	},
	
	_hideAffectedPerson(item){
		return 	 item== undefined ||( item.step=='affectedPerson' && item.data != '--')  ; 
	},
	
	/**
	 * Flyout for Instructions Long description with rich text editor 
	 */	
	_showInstructionsLD: function(e){
		try {
			e.stopPropagation();    
			var _value='';
			if(this.dataList){
				this.dataList.forEach(function(item){
					if(item.step=="AdditionalInformation"){
						_value = item.data;
					}
				});
			}
		
			var flyout = $M.showFlyout('instractions_flyout', '<div></div>',  e.currentTarget, { 'constrainWidth':false});
			flyout.addButtons([{'label':this.localize('uitext','mxapibase','Close'),'action':true,'event':'close-long-desc','eventTarget':this}]);
			 var maximoRTE = $M.createElement('maximo-richtexteditor');
			 maximoRTE.readonly=true;
			 maximoRTE.props = {
				 'id' : 'description_longdesc',
				 'placeholder' : this.localize('uitext', 'mxapisr', 'AdditionalCommentsStep'),
				 'value' : _value,
				 'background-color': '#1d3649',
				 'color' : '#ffffff'
			 };
			this.async(function(){
				var div = $j(flyout.$.internal).find('div');
				div.css({'width':'300px'});
				div.append(maximoRTE);
				$j('.flyout').css({'width': '100%', 'z-index': '20002'});
			},100);
			 
		  
			 this.LongDescRTE = maximoRTE;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
      
	},
	
	
	/**
	 * Suppress flyout without updating the data when the cancel button is clicked 
	 */
	_closeLongDescription : function(e){
		try {
			this.LongDescRTE.remove();
			this.LongDescRTE = null;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
	
	},
	
});
