/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18, 5737-M66
 * (C) Copyright IBM Corp. 2016,2021 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication, or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

Polymer({
	is: 'maximo-step-template',
	behaviors: [BaseComponent,ServiceRequestBehavior],
	properties: {
		title: {
			type: String,
			value: 'Panel'
		},
		type:{
			type: String
		},
		enableButtons:{
			type: String
		},
		stackData :{
			type: Object
		},
		next : {
			type: Function,
			value: function() {}
		},
		
		disableNext : {
			type: Boolean,
			value: false
		},
		requestorName: {
			type: String
		},
		hideAffectedPerson: {
			type: Boolean,
			value: false
		},
		/**
         * store the  long description component 
         */
    	LongDescRTE : {
            type: Object 
        },
        
        /**
      	 * Indicates if the screen is on desktop mode
      	 */
		isDesktop: {
			type: Boolean,
			value: true,
			observer: '_updateCSS'
		},
		
		skipSigoption:{
			type: String,
			value:'MXAPISR:INSERT'
		},
        
	},
	created: function(){
	},
	ready: function(){
		var self = this;
  		$j(window).on('resize', function(){
			 self._windowResize();
	    });
	},
	attached: function(){
		try {
			this.settingTemplate();
			var self = this;
			//this.async(function(){
			this._windowResize();
			//}, 10);   
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	settingTemplate : function(){
		try {
			if(!this.enableButtons.includes('next')){
				this.querySelector('.nextButtonDiv').style.display = 'none';
			}
			if(!this.enableButtons.includes('skip')){
				this.querySelector('.skipButtonDiv').style.display = 'none';
			}
			if(this.enableButtons.includes('none')){
				this.querySelector('.nextButtonDiv').style.display = 'none';
				this.querySelector('.skipButtonDiv').style.display = 'none';
			}
		} catch(err) {
			this._displayUnexpectedError(err);
		} 
	},
	
	/**
	 * set the scree to desktop / mobile mode
   	 */
	_windowResize : function(){
		this.isDesktop = !$M.isMobile();
	},
	
	/**
	 * set the scree to desktop / mobile mode
   	 */
	_updateCSS : function(){
		try {
			$j(this.$.mainWrapper).toggleClass('mobile',!this.isDesktop);
			$j(this.$.contentWrapper).toggleClass('mobile',!this.isDesktop);
			$j(this.$.stackList).toggleClass('mobile',!this.isDesktop);
			
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	
	setStackList: function(stackData) {
		try {
			this.stackData = stackData;
			this.requestorName = this.getAffectedPerson();;
			this.hideAffectedPerson = this._hideAffectedPersonHeader();
		} catch(err) {
			this._displayUnexpectedError(err);
		}
	
		 
	},
	_nextStep: function() {
		this.fire('next-step');
	},
	_skipStep: function() {
		this.fire('skip-step');
	},
	_hideAdditionalInformation: function(item){
	   return 	item.step=='AdditionalInformation' && item.data != '--' ;
	},
	
	_hideAffectedPersonStep(item){
		var result = false;
		if(item.step=="affectedPerson"){
				result = true;
			}
		return 	 result ; 
	},
	
	_hideAffectedPersonHeader(){
		try {
			var result = true;
			if(this.stackData){
				  this.stackData.forEach(function(item){
					  if(item.step=="affectedPerson"){
						  result = false;
					  }
				  });
			  } 
			return 	 result ; 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	getAffectedPerson(){ 
		try {
			var result =  '--';
			if(this.stackData){
				  this.stackData.forEach(function(item){
					  if(item.step=="affectedPerson"){
						  result = item.data;
					  }
				  });
			  } 
			return 	 result ; 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	/**
	 * Flyout for Instructions Long description with rich text editor 
	 */	
	_showInstructionsLD: function(e){
		try {
			e.stopPropagation();    
			var _value='';
			if(this.stackData){
				this.stackData.forEach(function(item){
					if(item.step=="AdditionalInformation"){
						_value = item.data;
					}
				});
			}
		
			var flyout = $M.showFlyout('instractions_flyout', '<div></div>',  e.currentTarget, { 'constrainWidth':false});
			flyout.addButtons([{'label':this.localize('uitext','mxapibase','Close'),'action':true,'event':'close-long-desc','eventTarget':this}]);
			 var maximoRTE = $M.createElement('maximo-richtexteditor');
			 maximoRTE.readonly=true;
			 maximoRTE.props = {
				 'id' : 'description_longdesc',
				 'placeholder' : this.localize('uitext', 'mxapisr', 'AdditionalCommentsStep'),
				 'value' : _value,
				 'background-color': '#1d3649',
				 'color' : '#ffffff'
			 };
			this.async(function(){
				var div = $j(flyout.$.internal).find('div');
				div.css({'width':'300px'});
				div.append(maximoRTE);
				$j('.flyout').css({'width': '100%', 'z-index': '20002'});
			},100);
			 
		  
			 this.LongDescRTE = maximoRTE;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
      
	},
	
	
	/**
	 * Suppress flyout without updating the data when the cancel button is clicked 
	 */
	_closeLongDescription : function(e){
		try {
			this.LongDescRTE.remove();
			this.LongDescRTE = null;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
});
