/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18, 5737-M66
 * (C) Copyright IBM Corp. 2017,2021 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication, or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
Polymer({
	is: 'maximo-technician-reportwork',
  	behaviors: [BaseComponent, Assignment, HTMLEncoder],
  	listeners: {
  	    'disableAddTime' : '_disableHeader',
  	    'deleteTimeCard': '_deleteWPTimeCard',
  	    'save-time-card': '_saveWPTimeCard',
  	    'save-item-card': '_saveReportedItemCard',
  	    'addUnplannedItemCard' : '_addUnplannedItemCard',
  	    'maximo-lookup-selected' : '_setValueFromReservedLookup',
  	    'deleteItemCard' : '_deleteItemCard',
  	    'maximo-report-time-card-dirty' : '_enableAddTime',
  	    'maximo-reset-element' : '_resetCard',
  	    'disable-item-list': '_disableItemList',
  	    'disableItemMaterialHeader': '_disableItemMaterialHeader',
  	    'addUnplannedToolCard' : '_addUnplannedToolCard',
  	    'save-tool-card': '_saveToolCard',
  	    'disable-tool-list': '_disableToolList',
  	    'disableToolHeader': '_disableToolHeader',
  	    'deleteToolCard' : '_deleteToolCard',
  	    'maximo-qrcode-result' : '_returnQRcode',
  	    'maximo-report-tool-card-dirty' : '_enableAddTool',
  	    'maximo-page-header-fixed':'_toggleWSFixed',
  	    'get-external-value' : 'getExternalValueForDomain'
  	},
	properties : {
		totalTimeCount: {
			type: Number,
			value: -1,
			notify: true
		},
		totalItemCount: {
			type: Number,
			value: -1,
			notify: true
		},
		timeLoaded : {
			type: Object,
			value: false,
			notify: true
		},
		itemsLoaded : {
			type: Object,
			value: false,
			notify: true
		},
        /* set to editable */
        editable: {
            type: Boolean,
            value: false
        },
        /**
    	 * LT Type domain filter
    	 */
    	ltTypeSet: {
    		type: Array,
    		value: function () {
    			return [{'filtertype': 'SIMPLE', 'field': 'domainid', 'availablevalues': [ {'value': 'LTTYPE', 'selected': true} ]}];
    		} 	
    	},
    	
 		/**
 		 *  To show localized menu items*
 		 */
        menuItems : {
        	type: Array,
    		value: [],
    		notify:true    		
        },
        /**
		 * To show localized menu tools for add unplanned *
		 */
		menuToolsUnplanned : {
			type : Array,
			value : [],
			notify : true
		},
		 /**
		 * To show localized menu items for add unplanned *
		 */
		menuItemsUnplanned : {
			type : Array,
			value : [],
			notify : true
		},
		
        /**
         * To get the plannedItem count
         */
        totalPlannedItemCount : {
        	type: Number,
			value: -1,
			notify: true
        },
        /**
         * To get the assignment count
         */
        totalAssignmentCount : {
        	type: Number,
			value: -1,
			notify: true
        },
        /**
         * To get selected reserved Items count
         */
        itemLength : {
        	type: Number,
			value: 0
        },
        
        /* set to enable back button  */
        cameFromCard: {
            type: Boolean,
            value: false
        },
        
        showCardAsNonEditable: {
            type: Boolean,
            value: false
        },
        
        /**
		 * To check reserved item quantity
		 */
		reservedItemsLoaded : {
			type: Boolean,
			value : false,
			notify : true
		},
		
		rotating : {
			type: Boolean,
            value: false
		},
		hasAnyError : {
			type: Boolean,
            value: false
		},
		
		/**
		 * count for Tool records
		 */		
		totalToolCount: {
			type: Number,
			value: -1,
			notify: true
		},
		
		/**
		 * Flag for tools loaded
		 */
		toolsLoaded : {
			type: Object,
			value: false,
			notify: true
		},
		
    	/**
    	 * flag for checking double click
    	 */
    	isClickedOnce : {
    		type: Boolean,
    		value: false
    	},
    	 /**
		 * To check planned tool flag
		 */
    	plannedToolsLoaded : {
			type: Boolean,
			value : false,
			notify : true
		},
		/**
 		 *  To show localized menu tools*
 		 */
        menuTools : {
        	type: Array,
    		value: [],
    		notify:true    		
        },
    	/**
		 * current tab name.
		 */
		currentTab: {
			type: String,
			notify: true
		},
    	/**
         * Property created to track the link from where navigation is performed.
         */
        selectedTab: {
            type: String,
            value: ''
        },
		
		/**
		 * For fail type
         */
		failType : {
			type: Array
		},
        
		/**
		 * To check Add button
		 */
		noShowAddButton : {
			type : Boolean,
			value : false
		},
		
		sendNotification :{
			type : Boolean,
			value : false
		},
		
		/**
		 * For toolDescription
         */
		toolDescription : {
			type: String
		},
		
		/**
		 * To check for failureTab
		 */
		failureTab :{
			type : Boolean,
			value : false
		},
		
		readonly :{
			type : Boolean,
			value : false
		},
		
		/**
		 * current user labor code.
		 */
		userLaborCode: {
			type: String,
			value: ''
		},
		
		/**
    	 * Issue Type filter
    	 */
		issueTypeDomainFilter: {
    		type: Array,
    		value: function () {
    			return [{'filtertype': 'SIMPLE', 'field': 'domainid', 'availablevalues': [ {'value': 'ISSUETYP', 'selected': true} ]}];
    		} 	
    	},
    	
    	/**
    	 * Issue Type result
    	 */
		issueTypeDomainSet: {
    		type: Array,
    		value: []
    	},
    	
    	/**
    	 * Line Type result
    	 */
		lineType: {
    		type: String
    	}
	},
  	/**
  	 * Triggered when Back button is clicked.
  	 * Clear buttons and trigger goback listener.
  	 */
	_goBack: function(e){
		this.dataHost._rootDataHost._loadButtons();
		this._disableHeader(false);
		this.fire('goback',this);
	},
	
    /**
     * reset cards.
     */
    _resetCard: function (e) { 
   		this.fire('reset-dirty-element');
    	 e.stopPropagation();
    },
	
	/*
	 * Load ReportedWork data
	 * 
	 */
	loadReportedWorkData : function(){
		var self = this;
		
		//enable wait layer
		$M.toggleWait(true);
		this.timeLoaded = false;
		this.itemsLoaded = false;
		this.toolsLoaded = false;
		
		var that = self;
		if(this._firstLoad){
			this.checkAssignments();
		}
		
		if(this.record.status_maxvalue === 'WAPPR' ||this.record.status_maxvalue === 'CLOSE' || this.record.status_maxvalue === 'CAN'){
			this.noShowAddButton = true;
		} else {
			this.noShowAddButton = false;
		}
		
		//Time
		if(this.currentTab==='section_time'){
			if($M.isMobile()){
				document.body.scrollTop = 0;
			}
			if($M.getUserInfo().labor && $M.getUserInfo().labor.laborcode){
				//Add sigoption check for labor
				var laborSigoption = this.isAuthorized('TECHNICIAN:REPORT_OTHER_LABOR');
				
				if(!laborSigoption){
					var whereClause='oslc.where=laborcode=%22'+$M.getUserInfo().labor.laborcode+'%22';
					this.$.technicianActualLabor.additionalParams=[whereClause];
					this.$.laborcraftcollection.additionalParams=[whereClause];
				}				
			}
	  		this.$.technicianActualLabor.collectionUri = this.record.href+'/uxshowactuallabor';
	  		self.$.technicianActualLabor.refreshRecords().then(function(actualLabor){
	  			self.totalTimeCount = actualLabor.response.responseInfo.totalCount;
	  			var hasDirty = false;
				var dirtyCardId = '';
	  			self.async(function(){
			   		$j(self).find('maximo-report-time-card').each(function(i,v){
			   			if(this.record){
			   				this.laborcode = '';
				   			this.craft = '';
				   			this.laborcode = this.record.laborcode;
				   			if(this.laborcode){
				   				this.readonly = true;
				   			}
				   			this.craft = this.record.craft.craft;
				   			this.craftcode = '';
				   			this.craftcode = this.craft;
				   			this.hasCraft = (this.craft)? true : false;
				   			this.vendor = this.record.vendor;
				   			this.premiumpay = this.record.premiumpaycode;
				   			this.skilllevel = this.record.skilllevel;
			   			}
			   			 
			   			if(this.record && this.record.timerstatus === 'ACTIVE'){
			   				this.isDirty = true;
			   				hasDirty = true;
			   				dirtyCardId = v.id;
			   			}else{
							this._clearDirty();
						}
					});
			   		
				}, 250);
	  			
	  			self.async(function(){ 
					var selectedElement = '#'+ dirtyCardId;
			        var addTimeDiv = this.$.addtimediv;
			        var reportWorkSelector = $M.workScape.querySelector('#techReportWork_technicianReportWorkSelector');
			        var mainPage = $M.workScape.querySelector('#techReportWork_maximoAppHeader');
			        //Disable set of elements
			        $M.toggleDisabled([addTimeDiv, reportWorkSelector, mainPage], true && dirtyCardId);
			        
			        if(dirtyCardId){ 
			             $M.toggleDisabled($j('#techReportWork_reportedTimeList').children().not(selectedElement).toArray(), true);
			             $M.allowPageNavigation(false);
			         } else {
			             $M.toggleDisabled($j('#techReportWork_reportedTimeList').children().toArray(), false);
			             $M.allowPageNavigation(true);
			         }
		       },350);
	  			
	  			self.timeLoaded = true;
	  			self.refreshCounters();
	  			if($M.isMobile()){
	  				self.async(function(){ 
			  			$j(self.$.technicianReportWorkSelector).find('.inner').scrollLeft(0);
		  			}, 100);
	  			}
	  			$M.toggleWait(false);
	  		});
		}
  		
		if(this.record && this.record.ltTypeSet){
			this._domainList(this.record.ltTypeSet);
		}
  		
  		if(this.currentTab==='section_items'){
  			this.$.issueTypeDomain.refreshRecords();  			
	  		this.$.technicianActualMaterial.collectionUri = this.record.href+'/uxshowactualmaterial';
	  		this.$.technicianActualMaterial.refreshRecords().then(function(actualMaterial){
	  			self.totalItemCount = actualMaterial.response.responseInfo.totalCount;
	  			self.itemsLoaded = true; 
	  			self.refreshCounters();
	  			$M.toggleWait(false);
	  		});
	  		this.checkReservedItem();
  		}
  		
  		if(this.currentTab==='section_assignments'){
  			this.refreshCounters();
  			$M.toggleWait(false);
  		}
  		
  		if(this.currentTab==='section_tools'){
	  		// Load actual tools 
  			this.loadReportedWorkToolData();
  		}
  		
  		if(this.currentTab==='section_failures'){
  			self.refreshCounters();
  			self.loadReportedWorkFailureRecord();
  			$M.toggleWait(false);
  		}
  		
  		this._firstLoad = false;
  		this.checkPlannedTool();
	},
	
 	/*
	 * Load Failure ReportedWork data
	 * 
	 */
	loadReportedWorkFailureRecord : function(){
		//To load failure report data
  		var self = this;
  		self.$.maximo_failure_detail.failureFlag = false;
  		self.$.maximo_failure_detail.problemFlag = false;
  		self.$.maximo_failure_detail.flagCause = false;
  		self.$.maximo_failure_detail.flagRemedy = false;
  		if(this.record && this.record.failurecode){
  			self.$.maximo_failure_detail.failureclass = false;
  			$j('[data-scope-id="fail_long_desc"]').prop('disabled',false);
  		} else {
  			self.$.maximo_failure_detail.failureclass = true;
  			$j('[data-scope-id="fail_long_desc"]').prop('disabled',true);
  		}
  		
  		if(this.record && !this.record.problemcode && this.record.problem && this.record.problem.description === undefined){
  			this.$.maximo_failure_detail.$.cause.value = '';
			this.$.maximo_failure_detail.$.remedy.value = '';
			
			this.$.maximo_failure_detail.flagRemedy = false;
			this.$.maximo_failure_detail.flagCause = false;
			this.$.maximo_failure_detail.causeObj = {};
			this.$.maximo_failure_detail.remedyObj = {};
			this.$.maximo_failure_detail.$.collapseIconCauseDebug.classList.remove('blueIcon');
			this.$.maximo_failure_detail.$.collapseIconRemedyRecommendation.classList.remove('blueIcon');
  		}
  		
  		this.$.failureResource.resourceUri = this.record.href+'/uxshowfailurereport';
  		this.$.failureResource.loadRecord().then(function(results){  			 
  			var failureReportObj = results.member;
  			self.$.maximo_failure_detail.$.collapseIconCauseConfirm.hidden = true;
  			self.$.maximo_failure_detail.$.collapseIconCauseDebug.hidden =  false;
  			self.$.maximo_failure_detail.$.collapseIconRemedyConfirm.hidden = true;
  			self.$.maximo_failure_detail.$.collapseIconRemedyRecommendation.hidden =  false; 
  			self.$.maximo_failure_detail.$.collapseIconProblemConfirm.hidden = true;
  			self.$.maximo_failure_detail.$.collapseIconProblemWarning.hidden =  false;
  			self.$.maximo_failure_detail.$.failureclass.readonly =self.$.maximo_failure_detail._getReadonlyFlagForFailure(self.record.status_maxvalue);
  			if(failureReportObj && failureReportObj.length === 1){
  				self.$.maximo_failure_detail.$.cause.value = '';
				self.$.maximo_failure_detail.$.remedy.value = '';
  			}
  			if(self.failureReportObj && self.failureReportObj.length === 2){
				self.$.maximo_failure_detail.$.remedy.value = ''; 
			}

  			failureReportObj.forEach(function(failReport){
  				if(self.domHost && self.domHost.getInternalDomainValue(self.failType,failReport.type) === 'PROBLEM') {
					self.$.maximo_failure_detail.$.collapseIconProblemConfirm.hidden = false;
					self.$.maximo_failure_detail.$.collapseIconProblemWarning.hidden =  true;
					self.$.maximo_failure_detail.causeObj={};
					self.$.maximo_failure_detail.problemObj = failReport;
  					self.$.maximo_failure_detail.problemhref = failReport.localref+'/failurelist';
  					self.$.maximo_failure_detail.failureFlag = true;
  					self.$.maximo_failure_detail.problemFlag = true;
  					self.$.maximo_failure_detail.$.problem.value = failReport.failurecode.description;
  					self.$.maximo_failure_detail.problem = failReport.failurecode.failurecode;
  					self.$.maximo_failure_detail.$.problem.readonly =self.$.maximo_failure_detail._getReadonlyFlagForProblem(self.record.status_maxvalue);
				}
  				if(self.domHost && self.domHost.getInternalDomainValue(self.failType,failReport.type) === 'CAUSE') {
  					self.$.maximo_failure_detail.causeDesc = failReport.failurecode.description;
  					self.$.maximo_failure_detail.$.collapseIconCauseConfirm.hidden = false;
					self.$.maximo_failure_detail.$.collapseIconCauseDebug.hidden =  true;
					self.$.maximo_failure_detail.causeObj = failReport;
  					self.$.maximo_failure_detail.causehref = failReport.localref+'/failurelist';
  					self.$.maximo_failure_detail.flagCause = true;
  					self.$.maximo_failure_detail.$.cause.value = failReport.failurecode.description;
  					self.$.maximo_failure_detail.$.cause.readonly =self.$.maximo_failure_detail._getReadonlyFlagForCause(self.record.status_maxvalue);
  				}else{
                    self.$.maximo_failure_detail.$.cause.readonly =self.$.maximo_failure_detail._getReadonlyFlagForCause(self.record.status_maxvalue);
 				}
  				if(self.domHost && self.domHost.getInternalDomainValue(self.failType,failReport.type) === 'REMEDY') {
  					self.$.maximo_failure_detail.remedyDesc = failReport.failurecode.description;
  					self.$.maximo_failure_detail.$.collapseIconRemedyConfirm.hidden = false;
					self.$.maximo_failure_detail.$.collapseIconRemedyRecommendation.hidden =  true; 
					self.$.maximo_failure_detail.remedyObj = failReport;
  					self.$.maximo_failure_detail.remedyhref = failReport.localref;
  					self.$.maximo_failure_detail.flagRemedy = true;
  					self.$.maximo_failure_detail.$.remedy.value = failReport.failurecode.description;
  					self.$.maximo_failure_detail.$.remedy.readonly =self.$.maximo_failure_detail._getReadonlyFlagForRemedy(self.record.status_maxvalue);
  				}else{
  					self.$.maximo_failure_detail.$.remedy.readonly =self.$.maximo_failure_detail._getReadonlyFlagForRemedy(self.record.status_maxvalue);
  				}
  			});
  			
  			if(failureReportObj.length === 0){
  				self.$.maximo_failure_detail.$.problem.readonly =self.$.maximo_failure_detail._getReadonlyFlagForProblem(self.record.status_maxvalue);
  				self.$.maximo_failure_detail.$.cause.readonly =self.$.maximo_failure_detail._getReadonlyFlagForCause(self.record.status_maxvalue);
  				self.$.maximo_failure_detail.$.remedy.readonly =self.$.maximo_failure_detail._getReadonlyFlagForRemedy(self.record.status_maxvalue);
  			}
  			$M.toggleWait(false);
  		});
	},
	
	/**
	 * To localize the Add Unplanned label.
	 */
	attached: function (){		
		this.set('menuItems', [{label:$M.localize('uitext', 'mxapiwodetail', 'addunplanned'),event:'addUnplannedItemCard',icon:'Maximo:Add-New'}]);
		this.set('menuToolsPlanned', [{
			label:$M.localize('uitext', 'mxapiwodetail', 'addunplanned'),
			event:'addUnplannedToolCard',
			icon:'Maximo:Add-New'
		}]);
		if($M.isMobile()){
			this.menuToolsPlanned.push({
				label : $M.localize('uitext', 'mxapiwodetail', 'ScanTool'),
				event : 'scanTool2',
				icon : 'Maximo:Barcode-Scan',
				identifier : 'toolbarcode'
			});
			this.set('menuToolsUnplanned', [ {
				label : $M.localize('uitext', 'mxapiwodetail', 'ScanTool'),
				event : 'scanTool2',
				icon : 'Maximo:Barcode-Scan',
				identifier : 'toolbarcode'
			} ]);			
		}
		this.set('menuItemsPlanned', [{
			label:$M.localize('uitext', 'mxapiwodetail', 'addunplanned'),
			event:'addUnplannedItemCard',
			icon:'Maximo:Add-New'
		}]);
		if($M.isMobile()){
			this.menuItemsPlanned.push({
				label : $M.localize('uitext', 'mxapiwodetail', 'ScanItem'),
				event : 'scanTool2',
				icon : 'Maximo:Barcode-Scan',
				identifier : 'itembarcode'
			});
			this.set('menuItemsUnplanned', [ {
				label : $M.localize('uitext', 'mxapiwodetail', 'ScanItem'),
				event : 'scanTool2',
				icon : 'Maximo:Barcode-Scan',
				identifier : 'itembarcode'
			} ]);
		}
		
		if(!$M.isMobile() && navigator.userAgent.indexOf("Firefox") !== -1){
			$j(this).find('.outerdiv').css('max-height', 'calc(2em + 9px)');
		}
		//Fetch current user laborcode
		this._getLaborUserlaborCode();
	},
	
 	 /*
     * Switch tab for detail, plan, comments and attachment
     */
	switchContent: function(event) {
		var curIdArray = event.currentTarget.id.split('_');
		var sectionId = 'section_' + curIdArray[curIdArray.length-1];
		this._showHideSection(sectionId);
		$j('.cardLayout_labor .skilllevel, .cardLayout_labor .dynamiclabel, .cardLayout_labor .scheduledstarttime, .cardLayout_labor .scheduledstartdate .dynamiclabel, .cardLayout_labor .plans_labor_cardheader').find('label').css({'font-size':'18px','display':'block','color':'#000','line-height':'24px'});
	},
	
	/*
	 * Show and hide tab for detail, plan, comments and attachment
	 */
	_showHideSection: function(sectionName) {
		this.currentTab = sectionName;
		var currentList;
		$j(this.$.subcontainer).children('div').each(function(index, elem) {
			if (elem.id.indexOf(sectionName) > -1) {
				currentList = this;
			}
			$j(this).hide();
		});
		if(sectionName === 'section_comments'){
			$M.workScape.backTopHide = true;
			$M.workScape.toggleBackTopButton();
		}else{
			$M.workScape.backTopHide = false;
			if(sectionName === 'section_failures'){
				this.failureTab =true;
			}
			if(sectionName === 'section_assignments' && this.failureTab){
				this.checkAssignments();
				this.failureTab = false;
			} 
		}
		$j(currentList).show();
if(this.record){
			
			var oriRecord = this.record;
		  	var self = this;
		  	var externalValues=[];
		  	this.$.timeresource.resourceUri = this.record.href+'/uxshowactuallabor/newmbo/getlist~transtype?oslc.select=*&lean=1&querylocalized=1';
		  	 this.$.timeresource.loadRecord().then(function(result){
		  					 
		  					if(result && result.member && result.member.length > 0){
		  						
		  						result.member.forEach(function(domainrecord){
					 	   	    		externalValues.push(domainrecord);
						  	   });
		  						self.record = oriRecord;
		  						if(externalValues.length > 0){
		  							self.record.ltTypeSet= externalValues;
		  							self.ltTypeSet= externalValues;
		  						}
		  						self.loadReportedWorkData();
					 		}
		  					
		  				 });
		}
		
	},
	
	/*
	 * Check if count is zero
	 */
	_showNoData: function(count,collectionRefreshed)
	{
		return (count === 0 && collectionRefreshed) ? false : true;
	},
	/*
     * To Add a Time card 
     */
	_addTimeCard: function(e) {
		var collection = this.$.technicianActualLabor;
		var self = this;
    	collection.invokeAction('new&oslc.select=*','','GET').then(
		 function(result) {
			$M.toggleWait(false);
			var cards= $j('#newTimeCard');
			if(cards.length===0){
		        var reportTimeCard = Polymer.Base.create('maximo-report-time-card',{'editable':true, 'workorderurl':self.record.href});
			    reportTimeCard.id='newTimeCard';
			    reportTimeCard.newCard=true;
			    reportTimeCard.setAttribute('newCard', true);
			    self.$.section_time.insertBefore(reportTimeCard, self.$.section_time.childNodes[2]); //InserBeforeTheList
		        reportTimeCard.isDirty=true;
		        reportTimeCard.isDateDurationUpdated=true;
		        reportTimeCard.istask = self.record.istask;
		        
		    	if(parseInt(self.$.laborcraftcollection.totalCount) === 1 && parseInt(sessionStorage.getItem('allowanycraft')) === 0){ //hide craft look up if there is only one craft.
		        	$j(reportTimeCard).find('#report_times_craft_field').css('visibility', 'hidden');
		        } 
		    	self.laborCraft = self._getLaborCraft();
		    	var labCode = this.userLaborCode ? this.userLaborCode : $M.userInfo.labor.laborcode;
		    	self._getWorkTypeForActualLabour();
		    	self.async(function(){ 
		    		self.$.section_time.children[1].$.report_times_transtype.values= self._domainList(self.record.ltTypeSet);
		    		self.$.section_time.children[1].$$('#report_times_transtype').value = self._getDomainValue(self.record.ltTypeSet);
		    		if(self.isAuthorized('TECHNICIAN:REPORT_OTHER_LABOR')){
		    			for(var i = 0; i < self.laborCraft.length; i++)
	    				{
		    				if(self.laborCraft[i].laborcode === labCode && self.laborCraft[i].defaultcraft){
		    					self.$.section_time.children[1].$$('#reportedwork_time_craft').value = self.laborCraft[i].craft.description;
		    					reportTimeCard.skilllevel = (self.laborCraft[i].craftskill && self.laborCraft[i].craftskill.skilllevel)?self.laborCraft[i].craftskill.skilllevel : '';  
				        		self.$.section_time.children[1].$$('#reportedwork_time_craft_skilllevel').label = self.laborCraft[i].craftskill.description;
				        		self.$.section_time.children[1].$$('#reportedwork_time_craft')._craftcode = self.laborCraft[i].$alias_this_attr$craft;
				        		reportTimeCard.craft = self.laborCraft[i].$alias_this_attr$craft;
		    				}
	    				}
		    		}else{
			    			if(self.laborCraft[0].defaultcraft){ //For Default craft
				        		self.$.section_time.children[1].$$('#reportedwork_time_craft').value = self.laborCraft[0].craft.description;
				        		self.$.section_time.children[1].$$('#reportedwork_time_craft_skilllevel').label = self.laborCraft[0].craftskill.description;
				        		reportTimeCard.skilllevel = (self.laborCraft[0].craftskill && self.laborCraft[0].craftskill.skilllevel)?self.laborCraft[0].craftskill.skilllevel : '';
				        		self.$.section_time.children[1].$$('#reportedwork_time_craft')._craftcode = self.laborCraft[0].$alias_this_attr$craft;
				        	} 
		    			} 
		    	},300);  
	    		self.$.nodatasection_labtranss.hidden = true;
	    		self.reportTimeCard=reportTimeCard;
	    		if(!self.isAuthorized('TECHNICIAN:REPORT_OTHER_LABOR')){
	    			reportTimeCard.$.report_time_person.hidden = true;
	    			reportTimeCard.$.report_time_person_image.hidden = true;
	    		}
	    		else {
					reportTimeCard.$.report_time_person.value = $M.userInfo.displayName;
					reportTimeCard.laborcode = labCode;
					reportTimeCard.$.report_time_person._id = labCode;
					if ($M.userInfo._imagelibref) {
						reportTimeCard.$.person_image.image = $M.userInfo._imagelibref;
					}
				}
	    		if(!self.isAuthorized('TECHNICIAN:PREMIUMCODE')){
	    			reportTimeCard.$.report_time_premium.hidden = true;
	    		}
		        var newTimeCardLooup = $j.parseJSON(reportTimeCard.$.report_times_actuals_taskid.editableAttributes);
		        newTimeCardLooup.lookup.href = self.record.href + '/uxshowactuallabor/zombie';
		        reportTimeCard.$.report_times_actuals_taskid.editableAttributes = newTimeCardLooup;
		        var dateFormat = $M.maxauth.whoami.dateformat.toUpperCase();
		        self.async(function(){
			        $M.getServerDateTime().then(function(response){
		        		if($M.isMobile()){
	        			reportTimeCard.$.report_times_startdate.value = moment(response.date,dateFormat).format('YYYY-MM-DD');
	        		}else{
		        		reportTimeCard.$.report_times_startdate.value= moment(response.date,dateFormat)._i;
		        	}
			        });
		        },100);
		        self._disableHeader(true); //Disable add time button
			}
		},
		function(error){
			$M.toggleWait(false);
			$M.showResponseError(error);
		});
		 
    },
    /**
     * To disable/enable Add time button, SideMenu and MastHead.
     * Mask onClick of Add Time Button
     */
    _disableHeader: function(isEnable) {
    	var selectedElement;
    	if(isEnable.target){
    		selectedElement = '#'+ isEnable.target.id;
    	}
    	var actionObjList = [this.$.addtimediv, $M.workScape.querySelector('#techReportWork_maximoAppHeader'),
				$j('#techReportWork_reportedTimeList').children().not(selectedElement).toArray(),
				$M.workScape.querySelector('#techReportWork_technicianReportWorkSelector')];
    	if(isEnable === true || isEnable.detail){
			$M.allowPageNavigation(false);
			$M.toggleDisabled(actionObjList, true);
    	 }else{
			$M.allowPageNavigation(true);
			$M.toggleDisabled(actionObjList, false);
    	 }
    },
    /**
     * To Delete a Time card.
     */
	_deleteWPTimeCard: function(e) {
		var self = this;
		this.timeLoaded = false;
		//To load spinner making count as -1 , this means collections is being refreshed
		this.totalTimeCount = -1;
        if (e.detail) {
        	var dataHost = self.dataHost._rootDataHost;
        	dataHost.addContent('techWODetail');
        	var techWODetail = $j(dataHost).find('#techWODetail')[0];
        	
            this.$.timeresource.deleteRecord(e.detail).then(function(response) {
                this.$.technicianActualLabor.refreshRecords();
                self.timeLoaded = true;
                if(self.cameFromCard){
                	self.showCardAsNonEditable = true;
                }
                techWODetail.$.worklinks1.hidden = false;
                techWODetail.$.worklinks2.hidden = true;
            }.bind(this), function(error) {
                $M.showResponseError(error);
                techWODetail.$.worklinks1.hidden = true;
                techWODetail.$.worklinks2.hidden = false;
            });
        }else{
        	$j('#newTimeCard').remove();
        	self.timeLoaded = true;
        }
        this._disableHeader(false);
    },
    /**
     * This method perform Save/Update action for actual Times.
     */
    _saveWPTimeCard: function(e) {
        $M.toggleWait(true);
        var totalTime = this.totalTimeCount;
        //To load spinner making count as -1 , this means collections is being refreshed
        this.totalTimeCount = -1;
        var timeData = e.detail;
        var responseProperties = 'labtransid';
        this.$.timeresource.resourceUri = this.record.href + '/uxshowactuallabor';
        var self = this;
        if (e.detail.href) {
            timeData._action = 'Update';
        } else {
            timeData.siteid = this.record.siteid;
        }
        var timeList = [];
        timeList.push(timeData);
        
        var dataHost = self.dataHost._rootDataHost;
        dataHost.addContent('techWODetail');
        var techWODetail = $j(dataHost).find('#techWODetail')[0];
                
        this.$.timeresource.updateRecord(timeList, responseProperties, true)
            .then(function(result) {
                $M.toggleWait(false);
                if (!e.detail.href) {
                    self.reportTimeCard.remove();
                }
                self._disableHeader(false);
                dataHost._refreshCurrentCompAll();
                self.$.technicianActualLabor.refreshRecords().then(function(){ 
                	self.fetchActualLabor();
                	self.async(function(){
    			   		$j(self).find('maximo-report-time-card').each(function(i,v){
    			   			if(this.record){
    			   				this.laborcode = '';
    				   			this.craft = '';
    				   			this.laborcode = this.record.laborcode;
    				   			if(this.laborcode){
    				   				this.readonly = true;
    				   			}
    				   			this.craft = this.record.craft.craft;
    				   			this.craftcode = '';
    				   			this.craftcode = this.craft;
    				   			this.hasCraft = (this.craft)? true : false;
    				   			this.vendor = this.record.vendor;
    				   			this.premiumpay = this.record.premiumpaycode;
    				   			this.skilllevel = this.record.skilllevel;
    			   			}
    					});
    				}, 100);
                });
                if(self.cameFromCard){
                	self.showCardAsNonEditable = true;
                }
                techWODetail.$.worklinks1.hidden = false;
                techWODetail.$.worklinks2.hidden = true;
                self.hasAnyError = false;
                if(techWODetail.record.labtranstimerstatus){
                	delete techWODetail.record.labtranstimerstatus;
                }
            }, function(error) {
                $M.toggleWait(false);
                $M.showResponseError(error);
                self.showCardAsNonEditable = false;
                techWODetail.$.worklinks1.hidden = true;
                techWODetail.$.worklinks2.hidden = false;
                self.hasAnyError = true;
                self.totalTimeCount = totalTime;
                self.fire('disable-time-list',self);
            });
    },
	  /**
	   * fetch domain List values for LtType Work.
	   */
    _domainList : function(e){
    	this.fire('domainlist',this.record.ltTypeSet);
    	var rwork = this.record.ltTypeSet;
    	if(rwork){
    	    return this.record.ltTypeSet.selectedValues;
    	} else{
    		return null;
    	}
    }, 
    
    _anyNewItemCards: function(){
    	return $j('maximo-report-item-card#newItemCard').length > 0
    },
    
    _anyNewToolCards: function(){
    	return $j('maximo-report-tool-card[newcard="true"]').length > 0
    },
    
    /**
     * To Add a Unplanned card.
     */
    _addUnplannedItemCard : function(e) {
		var cards= $j('#newItemCard');
		if(cards.length === 0){
			var props = {
					       'newCard' : true,
                           'recordhref' : this.record.href,
                           'siteid' : this.record.siteid,
                           'location' : '',
                           'itemnum' : '',
                           'id' : 'newItemCard',
                           'isNew' : true,
                           'editable' : true

				};
			var itemDescription = '';
			if (e.detail.record && e.detail.record.itemnum) {
				props.itemnum = e.detail.record.itemnum;
				props.rotating = e.detail.record.rotating;
				itemDescription = e.detail.record.description;
			}
			var reportItemCard = Polymer.Base.create('maximo-report-item-card',props);
			$j(reportItemCard).find('#reportedButtonDiv').prop('hidden', true);
			$j(reportItemCard).find('.material_toggle').prop('hidden', true);
			$j(reportItemCard).find('.report_item_issued').prop('checked',true);
	        this.$.section_items.insertBefore(reportItemCard, this.$.reportItemList); //InserBeforeTheList
	        this.$.nodatasection_item.hidden = true;
	        var materialTaskAttr = $j(reportItemCard).find('.report_material_task')[0].editableAttributes;
        	var itemTasklookupAttr = $j.parseJSON(materialTaskAttr);
        	itemTasklookupAttr.lookup.href = this.record.href +'/showplanlabor/zombie'; 
        	reportItemCard.$.report_material_task.editableAttributes = itemTasklookupAttr;
        	var materialSiteAttr = $j(reportItemCard).find('.report_material_site')[0].editableAttributes
        	var itemSitelookupAttr = $j.parseJSON(materialSiteAttr);
	        itemSitelookupAttr.lookup.href = this.record.href;
	        reportItemCard.$.report_material_site.editableAttributes = itemSitelookupAttr;
	        reportItemCard.$.report_material_site.value = $M.getMaxauth().whoami.defaultSiteDescription;
	        reportItemCard.$.report_item_site.value= $M.getMaxauth().whoami.defaultSiteDescription;
	        reportItemCard.isDirty = true;
	        if(reportItemCard.rotating){
	        	this.async(function(){
	        		$j(reportItemCard.$.report_item_issued).attr('readonly',true);
	        		reportItemCard.$.report_item_rotating_asset.hidden = false;
	        		reportItemCard._getLookupCount();
	        	},250);
    	    }else if(props.itemnum){
    	    	this.async(function(){
	        		reportItemCard._getLookupCount();
	        	},250);
    	    }
    	            
	        this._disableItemMaterialHeader(true);
	        reportItemCard.$.report_item_rotating_asset1.hidden = true;
	        reportItemCard.$.report_item_quantity.value = 1;
	        if(props.itemnum){
	        	reportItemCard.$.report_item_quantity.value = 1;
	        	reportItemCard.$.report_item_itemnum.value = props.itemnum; 
	        	reportItemCard.$.report_item_description.value = (itemDescription?itemDescription:props.itemnum);
	        }else {
	        	this.rotating = false;	
	        }
	        
		}
    },
    
    /**
     * This method perform Save/Update action for actual Times.
     */
    _saveReportedItemCard: function(e) {
        $M.toggleWait(true);
        var itemCnt =this.totalItemCount;
        this.itemsLoaded = false; 
        //To load spinner making count as -1 , this means collections is being refreshed 
        this.totalItemCount = -1;
        var itemData = e.detail;
        var responseProperties = 'matusetransid';
        this.$.timeresource.resourceUri = this.record.href + '/uxshowactualmaterial';
        var self = this;
        if (itemData.linetype === this.lineType && itemData.issuetype === this.getExternalValueForIssuetype('RETURN')){
        	itemData._action = 'AddChange';
        }else {
        	itemData._action = 'Add';
        }
        var itemList = [];
        itemList.push(itemData);
        this.$.timeresource.updateRecord(itemList, responseProperties, true)
            .then(function(result) {
                $M.toggleWait(false);
                if (!e.detail.href) {
                	e.target.remove();
                }
                self.$.technicianActualMaterial.refreshRecords().then(function(actualMaterial){ 
          			self.totalItemCount = actualMaterial.response.responseInfo.totalCount;
          			self.checkReservedItem();
          			self.itemLoaded = true; 
                });
                if(self.itemLength > 0){
                	self.itemLength = self.itemLength-1;
                	self._disableItemMaterialHeader(true);
                }else {
                	self._disableItemMaterialHeader(false);
                }
                if(self.itemLength === 0){
                	self._disableItemMaterialHeader(false);
                }
               	self._disableItemMaterialHeader(self._anyNewItemCards());
    	    	self.fire('disable-item-list', {'card':null, 'disabled':true});
            }, function(error) {
                $M.toggleWait(false);
                $M.showResponseError(error);
                self.totalItemCount = itemCnt;
                self.$.technicianActualMaterial.refreshRecords();
                self.dataHost._rootDataHost._refreshCurrentCompAll();
                self._disableItemMaterialHeader(true);
                self.fire('disable-item-list', {'card':null, 'disabled':false,'failedTransaction':true});
            });
    	},
    	
    	/**
	 	 * Select reserve look will display.
	 	 */
    	_selectReservedItem : function(e) {
			e.stopPropagation(e);
			var self = this;
			var additionalParams = ['oslc.where=item.itemtype!=%22TOOL%22'];
			$M.showLookup('invreserve-custom', self,'RESERVEDITEMS', self.record.href, true, additionalParams);
		},
    	
		/**
    	 * Return the item selected and dynamic creation of tiles.
    	 */
		_setValueFromReservedLookup : function(e){
    		var self = this;
    		this.sendNotification = true;
    		if(e.detail.identifier === 'RESERVEDITEMS'){
    			if(e.detail){
        			var items = e.detail.records;
        			this.itemLength = items.length;
        			if(items != null){
        				this._checkStrAccessForReservedItems(items);
        			}
                    if(this.totalItemCount === 0){
	                	this.$.nodatasection_item.hidden = true;
	                	this.$.selectReservedDivTop.hidden=false;                  	
	                }
                }
    		}
    		
    		if(e.detail.identifier === 'PLANNEDTOOLS'){
    			if(e.detail){
        			var tools = e.detail.records;
        			this.toolLength = tools.length;
        			if(tools != null){
        				var toolCardData = [];
        				var whereClause = '';
        				tools.forEach(function(tool){
        					
                        	var props = {'recordhref':self.record.href,
                        		'itemnum': tool.itemnum,
                        		'record' : {
            					'toolhrs_localized' : tool.hours_localized,
            					'toolhrs' : tool.hours,
            					'toolqty': parseInt(tool.itemqty),
            	    			'id':'newToolCard',
            	    			'isNew':true,
            	    			'itemnum': tool.itemnum,
            	    			'editable':false,
            	    			'toolitem': {'description':tool.toolitem.description},
            	    			'taskid':tool.taskid,
            	    			'id':'newToolCard'
                        	}};
                        	toolCardData.push(props);
                        	if(!whereClause.includes(tool.itemnum)){
                        		if(whereClause!==''){
                        			whereClause+=',';
                        		}
                        	    whereClause+='"'+tool.itemnum+'"';
                        	}
                        });
        	
        				//Load rotating flag for all tools
        				this.$.toolresource.additionalParams=['oslc.where=itemnum in ['+whereClause+']'];
        	      		this.$.toolresource.resourceUri = this.record.href+'/uxshowactualtool/zombie/getlist~itemnum';
        	      		this.$.toolresource.loadRecord().then(function(results){ 
        	      			var tools = results.member;
        	      			if(tools && tools.length > 0){
        	      				toolCardData.forEach(function(toolCard){
        	      					var flag = self.getFlag(tools,toolCard.itemnum);
        	      						if(flag){
        	      							toolCard.record['rotating']=true;
        	      							toolCard['rotating']=true;
            	      					}
        	      						var reportToolCard = Polymer.Base.create('maximo-report-tool-card',toolCard);
        	      						reportToolCard.isDirty=true;
    	    	    				    self.$.section_tools.insertBefore(reportToolCard, self.$.reportToolList); //InserBeforeTheList
    	    	    				    self._configureReportToolCard(toolCard,reportToolCard);
        	      				});
	        	      		}
        	      		});
        				
        			}
                    if(this.totalToolCount === 0){
	                	this.$.nodatasection_tool.hidden = true;
	                	this.$.selectToolsDivBottom.hidden = true;
	                	this.$.selectToolsDivTop.hidden=false;                  	
	                }
                }
    		}
    	 },
    	  
    	 /*
    	  * Getting the flag for the rotating details. 
    	  */
    	 getFlag: function(tools, itemnum){
    		var flag = false;
    		tools.forEach(function(tool){
    		 	if(tool.itemnum === itemnum && tool.rotating){
    		 		flag = true;
    		 	}
    		 });
    	   return flag;
    	  },
    	 /*
    	  * Populates the Item values in reportItemcard.
    	  */
    	 _configureReportItemCard : function(item, reportItemCard, noAccess){
    		  this.async(function(){
	    		    reportItemCard.setAttribute('newCard', true);
				    $j(reportItemCard).find('#reportedButtonDiv').prop('hidden',true);
				    $j(reportItemCard).find('.material_toggle').prop('hidden',true);
				    $j(reportItemCard).find('#actionButtons').prop('hidden',false);
				    this.$.nodatasection_item.hidden = true;
				    this._disableItemMaterialHeader(true);
				    this.$.selectReservedDivTop.hidden=false;
				    reportItemCard.$.report_item_site.value = item.site.description; 
				    reportItemCard.$.plans_item_linetype_itemInfoTooltip.value = "ITEM";
				    reportItemCard.$.report_item_description_readonly.label = item.description?item.description:item.itemnum;
				    reportItemCard.$.report_item_itemnum.value = item.itemnum;
				    reportItemCard.$.report_item_storeroom.value = item.locations.description?item.locations.description:item.location;
				    this.async(function(){
					    reportItemCard.$.report_item_task.value = item.displaytaskid;
					    reportItemCard.$.report_item_quantity.value = item.reservedqty;
					    reportItemCard.$.report_item_bin.value = item.inventory.binnum;
					    reportItemCard.itemnum = item.itemnum;
					    reportItemCard.setAttribute('location', HTMLEncoder.sanitize(item.location));
					    reportItemCard.setAttribute('siteid', HTMLEncoder.sanitize(item.storelocsiteid));
					    this.async(function(){
					        reportItemCard.$.report_item_quantity.editable =true;
					    	reportItemCard.$.report_item_bin.editable =true;
					    	reportItemCard.$.report_item_issued.checked = true;
					    	reportItemCard.isDirty=true;
					    	if(item.item.rotating){
				    			reportItemCard.rotating = true;
				    			reportItemCard.$.report_item_rotating_asset.hidden = false;
				    			reportItemCard.$.report_item_rotating_asset.readonly = false;
					    		reportItemCard.$.report_item_description.readonly = true;
					    		reportItemCard.$.report_item_storeroom.editable = false;
					    		reportItemCard.$.report_item_quantity.editable =true;
						    	reportItemCard.$.report_item_bin.editable =true;
						    	reportItemCard.$.report_item_issued.checked = true;					    	
					            reportItemCard.$.report_item_task.editable = false;
					    		reportItemCard.$.report_item_issued.readonly = true;
	    				    } else{
	    				    	reportItemCard.$.report_item_issued.readonly = false;
	    				    	reportItemCard.rotating = false;
	    				    	reportItemCard.$.report_item_rotating_asset.hidden = true;
	    				    }
					    	if(noAccess){
					    		reportItemCard.$.report_item_storeroom.value = '';
					    	}
					    }, 200);
				    }, 300);
    		  }, 400);
    	  },
    		
    		/**
        	 * Delete technician actual item.
        	 */
        	_deleteItemCard : function(e){
                $j(e.target).remove();
                if(this.totalItemCount === 0){
	                if(this.itemLength > 0){
	                	this.itemLength = this.itemLength-1;
	                	this._disableItemMaterialHeader(true);
	                }
	                if(this.itemLength === 0){
	                	this.$.nodatasection_item.hidden = false;
	                	this._disableItemMaterialHeader(false);
	                }
	            } else {
	            	this._disableItemMaterialHeader(false);
	            }
                this.dataHost._rootDataHost._refreshCurrentCompAll();
                this._disableItemMaterialHeader(this._anyNewItemCards());
            },
            /** 
             * To disable/enable Add Item button, MastHead using maximo-context.
             * Mask onClick of Add Item Button
             */
            _disableItemMaterialHeader: function(evt) {
          		if(evt === true || evt.detail){
          			$M.allowPageNavigation(false);
          			$M.toggleDisabled([this.$.addUnPlanDiv, this.$.selectReserverdDiv, this.$.technicianReportWorkSelector, this.$.mainPage, $M.workScape.querySelector('#workscape_footerToolbar')], true);
                }else{
            	    $M.allowPageNavigation(true);
            	    $M.toggleDisabled([this.$.addUnPlanDiv, this.$.selectReserverdDiv, this.$.technicianReportWorkSelector, this.$.mainPage, $M.workScape.querySelector('#workscape_footerToolbar')], false);
                }
            },
            
		/** 
         * handle time collection
         * Mask onClick of Add Item Button
         */
        _handleRecordTimeRefreshed: function(e) {
        	var self = this;
        	this.async(function(){
        		var card = $j(self).find('maximo-report-time-card')[0];
        		if(this.showCardAsNonEditable){
        			$j(self).find('maximo-report-time-card').each(function(){ 
    					card.isDirty = false;
					});	
    				if(self.cameFromCard){
						this.showCardAsNonEditable = false;
						this.cameFromCard = false;
					} 
        		}else{
        			if(card){
        				if(self.cameFromCard){
        					if(card.record && card.record.finishdate_localized && card.record.finishtime_localized){
        						var dateformat = $M.getMaxauth().whoami.dateformat.toUpperCase();
            					var timeformat = $M.getMaxauth().whoami.timeformat.toLowerCase();
            					card.$.report_times_finishdate.value = moment().format(dateformat);
            					card.$.report_times_finishtime.value = card.record.finishtime_localized;
            					card.record.finishdate_localized = moment().format(dateformat);
            					if(card.record.finishTime){ 								
            						if (timeformat.includes('a')){
            							card.record.finishtime_localized = moment().format('hh:mm A'); 
	            		    		}else{
    	        		    			card.record.finishtime_localized = moment().format('HH:mm'); 
            		    			}
            		    		}
            					card.$.report_times_finishtime.value = card.record.finishtime_localized;
            					card.isDateDurationUpdated=true;
            					card._calculateDurations();
            					card._clearDirty();
        					}
        					card.isDirty = true;
        					card.editable = true;
        					self.fire('disable-time-list',card);
        				}else{
        					if(!(card.record && card.record.finishdate && card.record.finishdate_localized)){
        						card.$.report_times_finishtime.minutevalue='0';
        						card.$.report_times_finishtime.hourvalue='0';
        						card.$.report_times_finishtime.value='';
        						card._clearDirty();
        						card.editable = true; 
        					}

        					if(card.record && card.record.finishtime_localized && card.record.finishdate_localized){
        						card.newCard = false;
        						card.editable = true;  
        						$M.toggleDisabled([self.$.addtimediv], true);
        					}
        					card._clearDirty();
        					card.isDirty = false;
        				}
        				if(this.hasAnyError){
        					card.isDirty = true;
        					this.showCardAsNonEditable = !this.hasAnyError ;
        				}
        				
        				$j(self).find('maximo-report-time-card').each(function(i,v){ 
        					if(i>0){
        						this._clearDirty(); 
        					}
        					this.laborCraft = self._getLaborCraft(); 
        		        	if(parseInt(self.$.laborcraftcollection.totalCount) === 1 && parseInt(sessionStorage.getItem('allowanycraft')) === 0){ //hide craft look up if there is only one craft.
        		        		$j(this.$.report_times_craft_field).css('visibility', 'hidden');
        		            }
        				});
        			}
        		}

        		if(self.cameFromCard || self.$.assignments){
        			$M.toggleDisabled([self.$.mainPage], false);
        		    self.$.goBack.enable = true;

        		}
		   }, 250);
        },

         /** 
         * verify if addNewTime button can be enable
         */
        _enableAddTime: function(e) {
        	var card = $j(this).find('maximo-report-time-card')[0];
        	if(card && card.isDirty){
        		$M.toggleDisabled([this.$.addtimediv], true);
        	}else{
        		$M.toggleDisabled([this.$.addtimediv], false);
        	} 
        }, 
        
        _disableItemList: function(e){
        	var items = e.detail.card?$j('maximo-report-item-card').not(e.detail.card).toArray():$j('maximo-report-item-card').not('maximo-report-item-card#newItemCard').toArray();
        	if (!e.detail.revertOperation && !e.detail.failedTransaction && !e.detail.card){
        		$M.toggleDisabled(items, e.detail.disabled);
        	}
        },
        /**
         * @return:- labor default craft.
         */
        _getLaborCraft:function(){
			if(this.$.laborcraftcollection.__data__.collectionData) {
				return this.$.laborcraftcollection.__data__.collectionData
				.filter(function(craft) {
					if (craft && craft.defaultcraft && craft.orgid == $M.getUserInfo().defaultOrg && craft.laborcode == $M.getUserInfo().labor.laborcode) {
						return craft;
					}
				});
			}
        },
        _compReportWork : function(e){
    		this.record.ltTypeSet=this.ltTypeSet;
    		this.record.isWorkOrderComp=true;
    		var dataHost = this.dataHost._rootDataHost;
        	dataHost.addContent('techWODetail');
        	var techWODetail = $j(dataHost).find('#techWODetail')[0];
        	techWODetail.completeWO = true;
    		this.fire('finishAssignment',this.record);
    	},
    	
    	/**
    	 * _showInfo will show the Tooltip.
    	*/
    	_showInfo: function(e) {
    		e.stopPropagation();
    		var tooltip = this.$.reportmyworkinfo;
    		$M.showTooltip(tooltip, e.currentTarget);
    	},
    	
    	/**
    	 * Checks if mouse hovers for more than 350 milisecs
    	 */
    	_showInfoHover: function(e) {
    		var icon = e.currentTarget;
    		var card = this;
    		if(icon.hoverTimer){
    			window.clearTimeout(icon.hoverTimer);
    		}
    		
    		$j(icon).one('mouseleave', function(){
    			if(icon.hoverTimer){
    				window.clearTimeout(icon.hoverTimer);
    			}
    			$M.tooltip.hide();
    		});
    		
    		icon.hoverTimer = setTimeout(function(){
    			card._showInfo(e);
    		}, 350);
    	},
    	
    	/**
    	 * Checks if count is more than 0 so Assignments tab can be shown
    	 */
    	checkForCount: function(totalAssignmentCount) {
    		if(totalAssignmentCount>0){
    			return true;
    		}else {
    			return false;
    		}
    	},
    	
    	/**
    	 * return the logged in user id for filtering assigments
    	 */    	
    	getCurrentUser: function() {
    	 return $M.getMaxauth().whoami.personId;	
    	},
    	
    	/**
    	 * checking the reserved item quantity
    	 */
    	checkReservedItem : function(){
      	    //To check reserved item quantity
    		this.$.invresource.additionalParams=['oslc.where=item.itemtype!=%22TOOL%22'];
      		this.$.invresource.resourceUri = this.record.href+'/showinvreserve';
      		var self = this;
      		this.$.invresource.loadRecord().then(function(results){ 
      			var items = results.member;
      			if(items && items.length > 0){
      				self.reservedItemsLoaded = true;
      		} else {
      				self.reservedItemsLoaded = false;
      		}
      			$M.toggleWait(false);
      		});
    	},
    	
    	/**
    	 * Fetching actual labor details.
    	 */
    	fetchActualLabor : function(){
	    	//check collection count for time and assign to local variable
    		
    		//Add sigoption check for labor
			var laborSigoption = this.isAuthorized('TECHNICIAN:REPORT_OTHER_LABOR');
			
			if(!laborSigoption && $M.getUserInfo().labor && $M.getUserInfo().labor.laborcode){
				var whereClause='oslc.where=laborcode=%22'+$M.getUserInfo().labor.laborcode+'%22';
				this.$.technicianActualLabor.additionalParams=[whereClause];
				this.$.laborcraftcollection.additionalParams=[whereClause];
			}
			
	  		this.$.technicianActualLabor.collectionUri = this.record.href+'/uxshowactuallabor';
	  		this.$.technicianActualLabor.refreshRecords().then(function(actualLabor){
	  			self.totalTimeCount = actualLabor.response.responseInfo.totalCount;
	  			self.timeLoaded = true;
	  			$M.toggleWait(false);
	  		});
        },
        
        /**
         * To Add a Unplanned tool card.
         */
        _addUnplannedToolCard : function(e) {
      		if (this.isClickedOnce) {
    			return;
    		}
    		this.isClickedOnce = true;
    		var cards= $j('#newToolCard');
    		if(cards.length === 0){
    			var props = {'record' : {
    					'toolhrs_localized' : '1:00',
    					'toolhrs' : 1,
    					'toolqty': 1,
    	    			'id':'newToolCard',
    	    			'isNew':true,
    	    			'itemnum': ''
    			}};
    			if (e.detail && e.detail.record && e.detail.record.itemnum) {
    				props.record.itemnum = e.detail.record.itemnum;
    				props.itemnum = e.detail.record.itemnum;
    				props.record.toolitem = {
    					'description' : e.detail.record.description
    				};
    				props.rotating = e.detail.record.rotating;
    			}
    	        var reportToolCard = Polymer.Base.create('maximo-report-tool-card', props); 
    	        $j(reportToolCard).find('#reportedButtonDiv').prop('hidden',true); 
    	        reportToolCard.setAttribute('editable', true);
    	        reportToolCard.setAttribute('recordhref', HTMLEncoder.sanitize(this.record.href));
    	        reportToolCard.isDirty = true;
    	        reportToolCard.istask = this.record.istask;
    	        if(reportToolCard.rotating){
    	        	var that = this;
    	        	this.async(function(){
    	        		reportToolCard.$.report_tool_rotating_asset.hidden = false;
    	        		reportToolCard.$.report_tool_quantity.editable = false;
    	        		reportToolCard._getAssetLookupCount();
    	        	},250);
        	    }		
    	        
    	        this.$.section_tools.insertBefore(reportToolCard, this.$.reportToolList);
    	        this.$.nodatasection_tool.hidden = true;
    	        this._disableToolHeader(true);
    	        reportToolCard.$.report_tool_rotating_asset.hidden = true; 
    		}
    		var self = this;
    		this.async(function(){
    			self.isClickedOnce = false;			
    		},250);
        },
    	
        /**
         * This method perform Save/Update action for actual tool.
         */
        _saveToolCard: function(e) {
            $M.toggleWait(true);
            var toolCount =this.totalToolCount;
            //To load spinner making count as -1 , this means collections is being refreshed
            this.totalToolCount = -1;
            this.toolsLoaded = false; 
            var toolData = e.detail;
            var responseProperties = 'tooltransid';
            this.$.timeresource.resourceUri = this.record.href + '/uxshowactualtool';
            var self = this;
           	toolData._action = 'Add';
            var toolList = [];
            toolList.push(toolData);
            this.$.timeresource.updateRecord(toolList, responseProperties, true)
                .then(function(result) {
                    $M.toggleWait(false);
                    if (!e.detail.href) {
                    	e.target.remove();
                    }
                    self.$.technicianActualTool.refreshRecords().then(function(actualTool){ 
              			self.totalToolCount = actualTool.response.responseInfo.totalCount;
              			self.itemLoaded = true; 
                    });
                   	self._disableToolHeader(self._anyNewToolCards());
                }, function(error) {
                    $M.toggleWait(false);
                    self.totalToolCount = toolCount;
                    $M.showResponseError(error);
                    self._disableToolHeader(true);
                    
                });
        	},

    		/**
           	 * Delete actual tool.
           	 */
         	_deleteToolCard : function(e){
        	   $j(e.target).remove();
        	   if(this.totalToolCount === 0){
        	      if(this.toolLength > 0){
        	        	this.toolLength = this.toolLength-1;
        	        	this._disableToolHeader(true);
        	        	this.plannedToolsLoaded = true;
        	      }
        	      if(this.toolLength === 0 || this.toolLength === undefined){
        	            this.$.nodatasection_tool.hidden = false;
        	            this._disableToolHeader(false);
        	      }
               } else {
        	     this._disableToolHeader(false);
        	   }
               this._disableToolHeader(this._anyNewToolCards());
        	}, 

            
           _disableItemList: function(e){
            	var items = e.detail.card?$j('maximo-report-tool-card').not(e.detail.card).toArray():$j('maximo-report-tool-card').not('maximo-report-tool-card#newToolCard').toArray();
            	if (!e.detail.revertOperation && !e.detail.failedTransaction && !e.detail.card){
            		$M.toggleDisabled(items, e.detail.disabled);
            	}
            },
            
            /** 
             * To disable/enable Add Tool button, MastHead using maximo-context.
             */
            _disableToolHeader: function(evt) {
          		if(evt === true || evt.detail){
          			$M.allowPageNavigation(false);
          			$M.toggleDisabled([this.$.addUnPlanToolDiv, this.$.selectPlannedToolDiv,this.$.technicianReportWorkSelector, this.$.mainPage, $M.workScape.querySelector('#workscape_footerToolbar'),$M.workScape.querySelector('#techReportWork_reportToolList')], true);
                }else{
            	    $M.allowPageNavigation(true);
            	    $M.toggleDisabled([this.$.addUnPlanToolDiv, this.$.selectPlannedToolDiv,this.$.technicianReportWorkSelector, this.$.mainPage, $M.workScape.querySelector('#workscape_footerToolbar'),$M.workScape.querySelector('#techReportWork_reportToolList')], false);
                }
            },
            
            /** 
             * To check the planned tools 
             */
            checkPlannedTool : function(){
          	    //To check reserved tool
            	this.$.toolresource.additionalParams ='[]';
          		this.$.toolresource.resourceUri = this.record.href+'/showplantool';
          		var self = this;
          		this.$.toolresource.loadRecord().then(function(results){ 
          			var tools = results.member;
          			if(tools && tools.length > 0){
          				self.plannedToolsLoaded = true;
          		} else {
          				self.plannedToolsLoaded = false;
          		}
       	    	});
        	},
        	
        	/**
    	 	 * Show planned tool lookup.
    	 	 */
        	_selectPlannedTool : function(e) {
    			e.stopPropagation(e);
    			var self = this;
    			$M.showLookup('plannedtool', self,'PLANNEDTOOLS', self.record.href, true, null);
    		},
    		
    	  /**
       	  * Populates the tool values in reportToolcard.
       	  */
       	   _configureReportToolCard : function(tool, reportToolCard){
       		   $j(reportToolCard).find('#reportedButtonDiv').prop('hidden',true);
       		   this.async(function(){
       			    reportToolCard.setAttribute('newCard', true);
   				    $j(reportToolCard).find('#actionButtons').prop('hidden',false);
   				    this.$.nodatasection_tool.hidden = true;
   				    this._disableToolHeader(true);
   				    this.$.selectToolsDivTop.hidden=false;
   				    this.async(function(){
   				    	if(tool.rotating){
   				    		reportToolCard.$.report_tool_quantity.editable = false;
   				    	    //Check if rotating asset auto populated
   				    		 this.async(function(){					 	
   								reportToolCard._getAssetLookupCount();
   							 },250);
   				    	} else {
   				    		 reportToolCard.$.report_tool_quantity.editable = true;
   				    	}
   				    	reportToolCard.$.report_tool_hours.editable =true;
   				    	reportToolCard.$.report_tool_rotating_asset.editable = true;
   				    	reportToolCard.isDirty=true;
   				    }, 300);
      		  }, 300);
       	  },
            
            /**
        	 * Reloads counter for time, material and tools except current selected tab which will fetch count on refresh record.
        	 */
        	refreshCounters: function() {
        		var self = this;
        		if(this.currentTab!=='section_time'){
	        		self.$.technicianActualLabor.queryInfo=[];
	        		self.$.technicianActualLabor.queryInfo[0]={'href':this.record.href+'/uxshowactuallabor'};
	        		this.$.technicianActualLabor.$.mxajaxCollectionCount.url  = this.$.technicianActualLabor.queryInfo[0].href;
	        		this.$.technicianActualLabor.collectionUri = this.record.href+'/uxshowactuallabor';
	        		this.$.technicianActualLabor.getCollectionCount().then(function(response){
	        	    	self.totalTimeCount = response.response.totalCount;
	        	    });
        		}
        		
        		if(this.currentTab!=='section_items'){
	        		this.$.technicianActualMaterial.queryInfo=[];
	        		this.$.technicianActualMaterial.queryInfo[0]={'href':this.record.href+'/uxshowactualmaterial'};
	        		this.$.technicianActualMaterial.$.mxajaxCollectionCount.url  = this.$.technicianActualMaterial.queryInfo[0].href;
	        		this.$.technicianActualMaterial.collectionUri = this.record.href+'/uxshowactualmaterial';
	        		this.$.technicianActualMaterial.getCollectionCount().then(function(response){
	        	    	self.totalItemCount = response.response.totalCount;
	        	    });
        		}
        		
        		if(this.currentTab!=='section_tools'){
	        		this.$.technicianActualTool.queryInfo=[];
	        		this.$.technicianActualTool.queryInfo[0]={'href':this.record.href+'/uxshowactualtool'};
	        		this.$.technicianActualTool.$.mxajaxCollectionCount.url  = this.$.technicianActualTool.queryInfo[0].href;
	        		this.$.technicianActualTool.collectionUri = this.record.href+'/uxshowactualtool';
	        		this.$.technicianActualTool.getCollectionCount().then(function(response){
	        	    	self.totalToolCount = response.response.totalCount;
	        	    });
        		}
        		
        		if(this.currentTab!=='section_assignments'){
        			var additionalassetparams=[];
        			additionalassetparams.push('amcrew.amcrewlabor.where=laborcode=%22'+$M.getUserInfo().labor.laborcode+'%22 and laborcode=%22'+$M.getUserInfo().labor.laborcode+'%22&amcrew.amcrewlabor.opmodeor=1');
        			this.$.wpassignment.additionalParams = additionalassetparams;
	        		this.$.wpassignment.queryInfo=[];
	        		this.$.wpassignment.queryInfo[0]={'href':this.record.href+'/showassignment'};
	        		this.$.wpassignment.$.mxajaxCollectionCount.url  = this.$.wpassignment.queryInfo[0].href;
	        		this.$.wpassignment.collectionUri = this.record.href+'/showassignment';
	        		this.$.wpassignment.getCollectionCount().then(function(response){
	        	    	self.totalAssignmentCount = response.response.totalCount;
	        	    });
        		}
        		
        	},
        	
        	/** 
        	 * This method is to check whether to show Add button or not
        	 */
        	showAddButton : function(plannedLoaded, noShowAddButton, buttonType){ 
        		if(noShowAddButton || !this.isAuthorized('MXAPIWODETAIL:SAVE')){
        			return true;
        		} else {
        			if(buttonType==='addUnplanned'){
        				return plannedLoaded;	
        			}else if(buttonType==='addReserved'){
        				return !plannedLoaded;
        			}
        		}	
        	},

	/**
	 * To get the Tool Id lookup count when tool is selected and auto populate
	 * when the count is 1
	 */
	_returnQRcode : function(event) {
		if (event.detail.identifier == 'toolbarcode') {
			this._getToolLookupCount(event.detail.value);
		} else if (event.detail.identifier == 'itembarcode') {
			this._getItemLookupCount(event.detail.value);
		}
	},

	/**
	 * To get the Tool Id lookup count when tool is selected and auto populate
	 * when the count is 1
	 */
	_getToolLookupCount : function(toolid) {
		var lookup = $M._lookups['tool'];
		var additionalLookupParams = 'setupdata={\'tooltrans\':{\'siteid\':\''
				+ this.record.siteid + '\'}}';
		var lookupURI = this.record.href + '/uxshowactualtool/zombie/getlist~'
				+ lookup.getList + '?' + additionalLookupParams
				+ '&oslc.where=itemnum="' + encodeURIComponent(toolid) + '"';

		// Find the record count
		this.$.qrcode.collectionUri = lookupURI;
		this.$.qrcode.attributeNames = lookup.attributeNames;
		var self = this;
		this.$.qrcode
				.refreshRecords()
				.then(
						function(lookupData) {
							if (self.$.qrcode.collectionData.length > 0) {
								var fakeEvent = {};
								fakeEvent.detail = {};
								fakeEvent.detail.record = self.$.qrcode.collectionData[0];
								self._addUnplannedToolCard(fakeEvent);
								$j('body').trigger('click');
							} else {
								$M.alert($M.localize('uitext', 'mxapibase',
										'no_result'), $M.alerts.info);
							}
						});
	},

	/**
	 * To get the Tool Id lookup count when tool is selected and auto populate
	 * when the count is 1
	 */
	_getItemLookupCount : function(itemid) {
		var lookup = $M._lookups['item'];
		var additionalLookupParams = 'setupdata={\'matusetrans\':{\'siteid\':\''
				+ this.record.siteid + '\'}}';
		var lookupURI = this.record.href
				+ '/uxshowactualmaterial/zombie/getlist~' + lookup.getList
				+ '?' + additionalLookupParams + '&oslc.where=itemnum="'
				+ encodeURIComponent(itemid) + '"';

		// Find the record count
		this.$.qrcode.collectionUri = lookupURI;
		this.$.qrcode.attributeNames = lookup.attributeNames;
		var self = this;
		this.$.qrcode
				.refreshRecords()
				.then(
						function(lookupData) {
							if (self.$.qrcode.collectionData.length > 0) {
								var fakeEvent = {};
								fakeEvent.detail = {};
								fakeEvent.detail.record = self.$.qrcode.collectionData[0];
								self._addUnplannedItemCard(fakeEvent);
								$j('body').trigger('click');
							} else {
								$M.alert($M.localize('uitext', 'mxapibase',
										'no_result'), $M.alerts.info);
							}
						});
	},
	
	/** 
     * verify if addNewTool button can be enabled
     */
     _enableAddTool: function(e) {
         var card = $j(this).find('maximo-report-tool-card')[0];
         if(card && card.isDirty){
         	$M.toggleDisabled([this.$.addUnPlanToolDiv], true);
         }else{
         	$M.toggleDisabled([this.$.addUnPlanToolDiv], false);
         } 
     },

     /**
      * To display the reserved Item card
      */
     _showReservedItemCards : function(item,flag){
	    var reportItemCard = Polymer.Base.create('maximo-report-item-card',{'id':'newItemCard','isNew':true,'editable':false,'requestnum':item.requestnum,'newCard':true,'recordhref':this.record.href,'location':item.location,'itemnum':item.itemnum,'siteid':item.storelocsiteid,'itemsetid':item.itemsetid});
	    this.$.section_items.insertBefore(reportItemCard, this.$.reportItemList); //InserBeforeTheList
	    this._configureReportItemCard(item,reportItemCard,flag);
     },
     
     /**
      * To check the access for storeroom for logged in user.
      */
     _checkStrAccessForReservedItems : function(records){
		 if(records.length===0){
			 return;
		 }
		 var item = records.shift();
		 var leftItems = records;

	 	 var items = [];
		 var itemData = {};
		 itemData._action = 'Add';
		 itemData.storeloc = item.location;
		 itemData.siteid = item.siteid;
		 itemData.positivequantity = 1;
		 itemData.itemnum = this.itemnum;
		 itemData.linetype = this.lineType;
		 items.push(itemData);
 		 var responseprops = 'storeloc,itemnum,siteid';
 		 var self = this;
 		 $M.toggleWait(true);
 		 this.$.timeresource.resourceUri = self.record.href+ '/uxshowactualmaterial';
 		 this.$.timeresource.updateRecord(items, responseprops, true, false, {'setvaluemode':true}).then( function(result) {
 		    $M.toggleWait(false);
 		    self._showReservedItemCards(item,false);
 		    self._checkStrAccessForReservedItems(leftItems);
 		 }, function(error){
 			$M.toggleWait(false);
 			if(self.sendNotification && error.request.__data__.response && error.request.__data__.response.Error){
 				$M.notify(error.request.__data__.response.Error.message, $M.alerts.warn);
 				self.sendNotification = false;
 			}
 			self._showReservedItemCards(item, true);
			self._checkStrAccessForReservedItems(leftItems);
 		 });
	},
	
	/**
 	 * Show worktype if istask=0 
 	 */
 	_showWorktype: function(worktype){
 		if(this.record && !this.record.istask){
 			return worktype;
		}
 		return '';
 	},
 	
 	 /**
     * Show Unspecified if tool description is undefined
     */
     loadReportedWorkToolData: function(){
	    	var self=this;
	    	 this.$.technicianActualTool.collectionUri = this.record.href+'/uxshowactualtool';
		  		this.$.technicianActualTool.refreshRecords().then(function(actualTool){ 
		  			self.totalToolCount = actualTool.response.responseInfo.totalCount;
		  			self.toolsLoaded = true;
		  			self.refreshCounters();
		  			$M.toggleWait(false);
		  			
		  			
		  			self.async(function(){
	 	 		   		$j(self).find('maximo-report-tool-card').each(function(i,v){ 
	 	 						if(this.record){
	 	 							this.$.report_tool_description.value = (this.record.toolitem.description)?this.record.toolitem.description:$M.getNoDataString();
	 	 						}
	 	 						if(this.isDirty){
									this.isDirty = false;
									this._clearDirty();	
								} 
	 	 					});
	 				}, 200);	
		  		});
		},
 	
 	 /** 
     * checking where failure tab should display or not
     */
     _hideFailuresTab : function(){
  	   var hasAccess = this.isAuthorized('REPORT_FAILURES');
  	   var readAccessGranted = this.isAuthorized('MXAPIFAILURELIST:READ');
  	   if(hasAccess && readAccessGranted){
  		   return false;
  	   }
  	  return  true;
     },
	
 	/**
 	 * Show/Hide complete work button  
 	 */
 	_accessForCompletework: function(){
 		if(this.isAuthorized('MXAPIWODETAIL:SAVE') && this.isAuthorized('MXAPIWODETAIL:COMP')){
 			return false;
 		}
 		return true;
 	},
 	 	
 	/**
 	 * Handle scrolling issue for tabheader
 	 */
 	_toggleWSFixed: function(e){
        $j(this.$.reportworkSelectorDiv).toggleClass('fixedSelector', e.detail === true);
       if(!$M.isMobile() && this.$.maximoAppHeader){
         var pageHeaderHeight = $j(this.$.maximoAppHeader).find('.pageHeader').height();
         $j(this.$.reportworkSelectorDiv).css('top', pageHeaderHeight+'px');
     }
    },
    
    _getDomainValue : function(ltTypeSet){
        var value = '';
        if(ltTypeSet.length>0){
             ltTypeSet.forEach(function(member){
                  if(member.valueid==='LTTYPE|WORK'){
                       value = member.value; 
                       return;                      
                  }
             });
    
	    }
	    return value;
	},
 	
 	/**
 	 * Checking the count for assignments , based on that displaying the tab in page.  
 	 */
 	checkAssignments : function(){
 	
 	    //To load spinner making count as -1 , this means collections is being refreshed
 		this.totalTimeCount= -1;
		this.totalItemCount = -1;
		this.totalToolCount = -1;
		
 		//check collection count for assignments and assign to local variable
 		var additionalassetparams=[];
 		additionalassetparams.push('amcrew.amcrewlabor.where=laborcode=%22'+$M.getUserInfo().labor.laborcode+'%22 and laborcode=%22'+$M.getUserInfo().labor.laborcode+'%22&amcrew.amcrewlabor.opmodeor=1');
 		this.$.wpassignment.additionalParams = additionalassetparams;
  		this.$.wpassignment.collectionUri = this.record.href+'/showassignment';
  		var self=this;
  		this.$.wpassignment.refreshRecords().then(function(labor){
  			self.totalAssignmentCount = labor.response.responseInfo.totalCount;
			if(self.totalAssignmentCount>0){
				self._showHideSection("section_assignments");
				self.$.technicianReportWorkSelector.select(0);
				self.async(function(){
					$j('.cardLayout_labor .skilllevel, .cardLayout_labor .dynamiclabel, .cardLayout_labor .scheduledstarttime, .cardLayout_labor .scheduledstartdate .dynamiclabel, .cardLayout_labor .plans_labor_cardheader').find('label').css({'font-size':'18px','display':'block','color':'#000','line-height':'24px'});
  				},50);
	  		}else{
	  		    self._showHideSection("section_time");
	  			self.$.technicianReportWorkSelector.select(1);
	  		}
  		});
 	},
 	
 	/**
 	 * To get Crew description or value
 	 */
 	_getCrew : function(assignment){
 		if(assignment && assignment.amcrew && assignment.amcrew.length > 0){
 			var amcrew = assignment.amcrew;
 			var index = amcrew.length-1;
 			return amcrew[index].description ? amcrew[index].description : amcrew[index].amcrew;
 		} else {
 			return '';
 		}	
 	},
 	
 	/**
     * Format Date for given dateInput.
     */
 	_getFormatedDate : function (dateInput){
 		var currentDate = '';
	 	if($M.isMobile()){
			var datetimeformat = $M.getMaxauth().whoami.dateformat.toUpperCase()+' '+$M.getMaxauth().whoami.timeformat;
			currentDate = moment(dateInput,datetimeformat).format('YYYY-MM-DD HH:mm');
	  	}else {
			currentDate = moment(dateInput,$M.maxauth.whoami.datetimeformat.toUpperCase())._i;
	  	}
	 	return currentDate;
 	},
 	
 	/**
     * Set Date based on format.
     */    
 	_setRequiredByDate: function (dateInput){
 		var currentDate = this._getFormatedDate(dateInput);
	  	return currentDate.substr(0,currentDate.indexOf(' '));
	},
	
	/**
     * Set Time based on format.
     */
	_setRequiredTime : function (dateInput){
		var currentDate = this._getFormatedDate(dateInput);
		return currentDate.substr(currentDate.indexOf(' ')+1);
	},
	/**
     * @return:- labor code for current user.
     */
    _getLaborUserlaborCode:function(){
    	var additionalparams=[];
    	var currentUserLaborCode ;
 		additionalparams.push('oslc.where=personid=%22'+$M.getUserInfo().personid+'%22and%20orgid=%22'+$M.getUserInfo().defaultOrg+'%22');
		this.$.fetchUserLaborCodes.additionalParams=additionalparams;
		this.$.fetchUserLaborCodes.refreshRecords().then(function(result){
			if(result.response && result.response.member){				
				this.userLaborCode = result.response.member[0].laborcode;
			}
		});   
    },
    
    /**
	 * Handler for issue Type domain
	 */
    _handleIssueTypeDomainSet : function(){
    	var issueTypeFilteredSet = this.issueTypeDomainCollection[0].synonymdomain;
		this.issueTypeDomainSet = issueTypeFilteredSet.filter(function(o){return (o.maxvalue === 'ISSUE' || o.maxvalue === 'RETURN');} );
    },
    
    /**
	 * Error Handler for issue Type domain
	 */
    _handleIssueTypeDomainSetError : function(){
    	this.issueTypeDomainSet = [];
    },
    
    /**
  	 * Fetch Internal Value based on External Value for Issuetype
  	 */
  	 getInternalValueForIssuetype: function(externalValue){
  		 var internalValue; 
 		 if(this.issueTypeDomainSet){
 			this.issueTypeDomainSet.forEach(function(domainrecord){
 	   	    	if(domainrecord.value===externalValue){
 	   	    		internalValue = domainrecord.maxvalue;
 	   	    	}	
 	 		});
 		 }
  		 return internalValue;
  	 },
  	 
  	/**
  	 * Fetch Internal Value based on External Value for Issuetype
  	 */
  	 getExternalValueForIssuetype: function(internallValue){
  		 var externalValue; 
 		 if(this.issueTypeDomainSet){
 			this.issueTypeDomainSet.forEach(function(domainrecord){
 	   	    	if(domainrecord.maxvalue===internallValue){
 	   	    		externalValue = domainrecord.value;
 	   	    	}	
 	 		});
 		 }
 		 return externalValue;
  	 },
  	 
  	/**
   	 * Fetch External Value for Domain
   	 */
  	 getExternalValueForDomain : function(e){
  		 this.lineType = e.detail;
  	 },
  	 
  	  /**
	  * get valid list for a lttype domain values
	  */
	 _getWorkTypeForActualLabour: function(e){
	  	var oriRecord = this.record;
	  	var self = this;
	  	var externalValues=[];
	  	this.$.timeresource.resourceUri = this.record.href+'/uxshowactuallabor/newmbo/getlist~transtype?oslc.select=synonymdomainid,valueid,maxvalue,href,defaults,description,value,domainid&lean=1&querylocalized=1';
	  	 this.$.timeresource.loadRecord().then(function(result){
	  					 
	  					if(result && result.member && result.member.length > 0){
	  						
	  						result.member.forEach(function(domainrecord){
				 	   	    		externalValues.push(domainrecord);
					  	   });
	  						self.record = oriRecord;
	  						if(externalValues.length > 0){
	  							self.record.ltTypeSet= externalValues;	
	  							self.ltTypeSet= externalValues;	
	  						}
	  						
				 		}
	  					
	  				 });
	  	 
	  						
 	}
});	