/*
 *	Licensed Materials - Property of IBM
 *	"Restricted Materials of IBM"
 *	5724-U18, 5737-M66
 *	(C) COPYRIGHT IBM CORP. 2012,2022 All Rights Reserved.
 *	US Government Users Restricted Rights - Use, duplication or
 *	disclosure restricted by GSA ADP Schedule Contract with
 *	IBM Corp.
 **/

     var navContainerMinWidth = 75;
     var navIconWidth = 0; //50
/**
 * Resize the navsections. Each open section will get an equal percentage of vertical space. 
 * If a section uses less than it is given that space is shared among others. 
 * @param containerOrId
 * @param contHeight
 * @return
 */

function sizeNavSections(containerOrId, contHeight) {
	if(DESIGNMODE)
		return;
	var closedHeadingHeights = 0;
	var totalHeadingHeights = 0;
	var open = new Array();
	var containerOrId = dojo.byId(containerOrId);
	var containerState = dojo.attr(containerOrId, "state");
	if(containerState=="closed"){
		return;
	}
	var navSections = eval(containerOrId.id + "navSections");
	var sectionCount = getObjectLength(navSections);
	var open = new Array();
	for (id in navSections) {
		var secDef = navSections[id];
		var tempHeight = dojo.byId(id+"_head").offsetHeight;
		if(tempHeight == 0)
			tempHeight = 30;
		totalHeadingHeights += (secDef.headingHeight = tempHeight + 1);
		if(secDef.state == "min") {
			closedHeadingHeights += secDef.headingHeight;
		}
		else {
			open.push(id);
		}
	}
	var reclaimed = 0;
	var scrolled = new Array();
	var lastOpen;
	var distributedHeight = Math.ceil((contHeight - closedHeadingHeights) / open.length);
	for (id in navSections) {
		var secDef = navSections[id];
		var menuContainer = dojo.byId(id + "_content");
		var sec = dojo.byId(id);
		switch (secDef.state) {
			case "min":
				dojo.style(menuContainer, {"display":"none"});
				dojo.style(sec, {"height" : secDef.headingHeight + "px"});
				break;
			case "normal":
			case "max":
				var height = distributedHeight;
				lastOpen = id;
				dojo.style(sec, {"height" : height + "px"});
				var inner = dojo.byId(menuContainer.id + "_inner");
				dojo.style(sec, {"height" : height + "px"});
				dojo.style(menuContainer, {"height":"", "overflowY":"auto", "overflowX":"hidden", "display":""});
				dojo.style(inner, {"display":""});
				var adjust = 1; //deals with unexplained div offset in IE
				var extra = (height - secDef.headingHeight) - inner.offsetHeight - adjust;
				if(extra>=0) {
					if(dojo.isIE > 0) {
						adjust = 2;
					}
					reclaimed += extra + adjust;
					height -= extra - adjust;
				}
				else {
					scrolled.push(id);
				}
				dojo.style(sec, {"height" : height + "px"});
				dojo.style(menuContainer, {"height":(height - secDef.headingHeight) + "px", "overflowY":"auto", "overflowX":"hidden"});
				break;
		}
	}
	if(reclaimed > 0) {
		if(scrolled.length == 0 && lastOpen) {
			scrolled.push(lastOpen);
		}
		if(scrolled.length>0) {
			//distribute reclaimed amount across all scrolled
			var unused = 0;
			reclaimed = Math.ceil((reclaimed / scrolled.length));
			for(var i = 0;i<scrolled.length;i++) {
				reclaimed += unused;
				var sizeContent = dojo.byId(scrolled[i] + "_content");
				var unused = (sizeContent.offsetHeight + reclaimed) - sizeContent.scrollHeight;
				var secDef = navSections[scrolled[i]];
				if(unused < 0 || scrolled[i]==lastOpen) {
					unused = 0;  
				}
				var height = (sizeContent.offsetHeight+(reclaimed-unused));
				dojo.style(sizeContent,{"height":height-1+"px","overflowY":"auto","overflowX":"hidden"});
				dojo.style(sizeContent.parentNode,{"height":(height+secDef.headingHeight)+"px"});
				if(unused > 0) {
					dojo.style(sizeContent,{"overflowY":"hidden"});
				}
			}
			if(unused > 0) {
				var sizeContent = dojo.byId(lastOpen + "_content");
				var height = sizeContent.offsetHeight+unused;
				dojo.style(sizeContent,{"height":height+"px"});
				dojo.style(sizeContent.parentNode,{"height":(height+secDef.headingHeight)+"px"});
			}
		}
	}
}

/**
 * Find out how much extra space this element is using
 * @param content
 * @param headHeight
 * @return
 */
function getExtraSpace(content, headHeight) {
	var contentHeight = (content.parentNode.offsetHeight - headHeight) - 3;
	var inner = dojo.byId(content.id + "_inner");
	return (contentHeight - inner.offsetHeight);
}

/**
 * Used to resize the navbar and adjacent column
 * We allow supression of section sizing as the scrolling is reset on them and we don't really want to do that if
 * user closes and open the whole container
 * @param containerOrId
 * @return
 */
var dragScroll = false;
function sizePanes(containerOrId, sections) {
	if(DESIGNMODE)
		return;
	var navContainer = dojo.byId(containerOrId);
	var overFlow = dojo.attr(navContainer, "pagenav") == "true";
	if (overFlow) {
		var adjacentCol = dojo.byId(dojo.attr(navContainer, "adjacentColId"));
		if(!adjacentCol) {
			adjacentCol = getAdjacentCol(navContainer.id).adjacentCol; 
		}
		pageScrollArea = adjacentCol;
		
		if(pageScrollArea && !dragScroll && SKIN.indexOf("iot18") > -1 ){
			dragScroll = true;
			var curDown;
			pageScrollArea.addEventListener('mousemove', function(e){ 
				if(curDown && !e.ctrlKey){
					if(e.buttons>0){
						var pos = dojo.position(pageScrollArea);
						if(e.pageX > pos.x && e.pageX < pos.x + pos.w && e.pageY > pos.y && e.pageY < pos.y + pos.h){
							pageScrollArea.scrollTo(pageScrollArea.scrollLeft - e.movementX, pageScrollArea.scrollTop - e.movementY);
						}
					}
					else {
						curDown = false;
						pageScrollArea.removeAttribute('draggable');
						e.currentTarget.className = e.currentTarget.className.replace(/stopSelect/, '');
					}
					e.stopPropagation();
				}
			});
	
			/*
			 * Adding the e.mapPoint verification to prevent disable the "pan" behavior when using ESRI Map API. 
			 */
			pageScrollArea.addEventListener('mousedown', function(e){
				if(!e.ctrlKey && e.target.tagName !== "INPUT" && e.target.tagName !== "TEXTAREA" 
					&& e.mapPoint == null){
					curDown = true;
					pageScrollArea.setAttribute('draggable','false');
					e.currentTarget.className = e.currentTarget.className + "stopSelect";
				}
			});
	
			pageScrollArea.addEventListener('mouseup', function(e){
				curDown = false;
				pageScrollArea.removeAttribute('draggable');
				e.currentTarget.className = e.currentTarget.className.replace(/stopSelect/, '');
			});
		}
		
		// make room for breadcrumbs if there are any
		var breadcrumbsHeight = 0;
		if (typeof breadcrumbsId != 'undefined') {
			var breadCrumbs = dojo.byId(breadcrumbsId);
			if(breadCrumbs) {
				breadcrumbsHeight = dojo.marginBox(breadCrumbs).h;
			}
		}
		if (navContainer && adjacentCol) {
			dojo.style(adjacentCol, {"width":"100%"});
			window.setTimeout('forceObjsRefresh()', 100);	
			var bodyScroll = document.documentElement.scrollHeight;
			var setHeight = 0;
			var contSetHeight = 0;
			if (bodyScroll > 0) {
				setHeight = (dojo.window.getBox().h - dojo.position(navContainer.parentNode).y - breadcrumbsHeight - 1);
				var searchHeight = 0;
				var search = dojo.byId(navContainer.id+'_search');
				if(search){
		    		require(["dojo/dom-geometry"], function(domGeom){
						contSetHeight = setHeight - (domGeom.position(search).h - 2);
		    		});
				}
				if(setHeight>0) {
					var widthOffset = 0;
					if(dojo.attr(navContainer, "collapsible") == "true") { //needs to match the width of the collapse sep, but it may not be created yet 
						if(SKIN.indexOf("iot18") == -1 ){
							widthOffset = 6;
						}
					}
					var sep = dojo.byId(navContainer.id + "_navSep");
					if(sep) {
						dojo.style(sep, {
							"height" : setHeight+"px"
						});
					}
		
					var currentAppDiv = dojo.byId("pagecurrentappdiv");
					if (currentAppDiv) {
						widthOffset += currentAppDiv.offsetLeft; 
					}		

					dojo.attr(adjacentCol, {
						'tabindex' : '-1'
					});
					
					dojo.attr(adjacentCol, {
						'tabindex' : '-1'
					});
					var navEl = dojo.byId(navContainer.id);
					var navWidth = (dojo.window.getBox().w - document.querySelector('.navContainer').closest('.noPadding').offsetWidth - widthOffset -1);
					if (navEl.getAttribute('state')=="closed" && SKIN.indexOf("iot18") > -1 )
					{
						if (adjacentCol.id.indexOf("_div") > 0)
							navWidth = navWidth - 12;
					}
					dojo.style(adjacentCol, {
						"height" : setHeight + "px",
						"verticalAlign" : "top",
						"width" : navWidth + "px",
						"overflow" : "auto"
					});
					dojo.connect(adjacentCol, "scroll", adjacentCol, function() {
						stopPopWait();
						stopPopOver();
						dojohelper.closePickerPopup();
					});
					dojo.connect(adjacentCol, "mousewheel", adjacentCol, function() {
						stopPopWait();
						stopPopOver();
						dojohelper.closePickerPopup();
					});
					dojo.connect(adjacentCol, "wheel", adjacentCol, function() {
						stopPopWait();
						stopPopOver();
						dojohelper.closePickerPopup();
					});
					dojo.forEach(dojo.query("> table", adjacentCol), function(element) {
						dojo.style(element, {
							"height" : ""
						});
						dojo.attr(element, {
							"height" : ""
						});
					});
					dojo.style(navContainer, {
						"height" : contSetHeight + "px"
					});
				}
			}
			var tb = findParent(navContainer, "table", 1);
			dojo.style(tb, {
				"height" : ""
			});
			dojo.attr(tb, {
				"height" : ""
			});
			if(sections) {
				sizeNavSections(containerOrId, contSetHeight - 1);
			}
			
			dojo.publish("sizeNavHeader", navContainer.clientWidth);
			
			var navSections = eval(navContainer.id + "navSections");
			for(navsection in navSections) {
				var navSection =  dojo.byId(navsection);
				dojo.forEach(dojo.query(".navTitle", navSection), function(element) {
					var head = dojo.byId(navsection+"_head");
					var inputs = dojo.query("input", head).length;
					var navSec = navSections[navsection];
					var padding = 0;
					var iconWidth = 16; //width of internal icons
					if(inputs>0) {
						padding = iconWidth * 2;
						if(SCREENREADER){
							padding = iconWidth * 3;
						}
					}
					width = navContainer.clientWidth-padding;
					if(width<0){
						width = 0;
					}
					dojo.style(element, {
						"width":width+"px"
					});
				});
				sizeAnchors(navContainer, navContainer.clientWidth);
			}
		}
	} else if(sections){
		sizeNavSections(containerOrId, null);
	}
}

/**
 * Used to remove any visble trace of the navcontainer from the page
 * @param navContainerId
 * @return
 */ 
function killNavContainer(navContainerId) {
	var navContainer = dojo.byId(navContainerId);
	if (!navContainer)
		return;
	var sectionCol = findParent(navContainer, "td", 2);
	if(!sectionCol)
		return;
	sectionCol.style.display="none";
	var section = findParent(sectionCol, "table", 1);
	if(!section)
		return;
	dojo.attr(section, {
		"cellspacing": "0"
		});
	dojo.style(section, {
		"padding": "0px", 
		"borderSpacing" : "0px"
	});
	var adjacentCol = section.rows[0].cells[sectionCol.cellIndex+1];
	pageScrollArea = adjacentCol;
	dojo.style(adjacentCol, {
		"padding": "0px" 
	});

}

function getAdjacentCol(navContainerId) {
	var navContainer = dojo.byId(navContainerId);
	var adjacentColId = dojo.attr(navContainer, "adjacentColId");
	var sectionCol = findParent(navContainer, "td", 2);
	var sectionRow = sectionCol.parentNode;
	var adjacentCol;
	if(adjacentColId) {
		adjacentCol = dojo.byId(adjacentColId);
	}
	else {
		adjacentCol = sectionRow.cells[sectionCol.cellIndex + 1];
		if(adjacentCol && dojo.attr(adjacentCol, "className")=="navSep") {
			adjacentCol = sectionRow.cells[sectionCol.cellIndex + 2];
		}
	}
	
	var handleOffset = 1;
	var collapseDir = "L";
	if (!adjacentCol) {
		try
		{
			adjacentCol = sectionRow.cells[sectionCol.cellIndex - 1];
			if(adjacentCol) {
				collapseDir = "R";
				handleOffset = 0;
			}
		}
		catch(error)
		{} //safety for bad colindex
	}
	if(sectionCol) {
		dojo.attr(navContainer, {"navcolId" : sectionCol.id});
		dojo.style( sectionCol , {"background":"#efefef"});
	}
	pageScrollArea = adjacentCol;
	return {"adjacentCol":adjacentCol,"collapseDir":collapseDir,"handleOffset":handleOffset};
}

/**
 * Called after initial render to allow us to create the control structure
 * within the existing DOM
 * 
 * @param navContainerId
 * @return
 */
function fixNavContainer(navContainerId) {
	NAV_SECTION = true;
	dojo.require("dojox.fx");
	var navContainer = dojo.byId(navContainerId);
	if (!navContainer)
		return;
	dojo.connect(navContainer, "selectstart", navContainer, function () { return false; });
	var collapsible = dojo.attr(navContainer, "collapsible") == "true";
	var sectionCol = findParent(navContainer, "td", 2);
	dojo.attr(sectionCol, {"class":dojo.attr(sectionCol,"class")+" noPadding"});
	var navTop = getTopPosition(navContainer);
	sectionCol.style.display = "none";
	var viewport = dojo.window.getBox();
	sectionCol.style.display = "";
	navContainer.parentNode.removeAttribute("noWrap");
	var setWidth = sectionCol.offsetWidth;
	var sectionRow = sectionCol.parentNode;
	if (!sectionRow || !sectionCol)
		return;
	var table = sectionRow.parentNode.parentNode
	table.style.borderSpacing = "0";
	table.cellPadding = "0";
	sectionCol.style.padding = "0px";
	var cell = sectionRow.cells[sectionCol.cellIndex];
	dojo.style(cell, {
		"width":"0%"
	});
	var section = div = findParent(cell, "TD", 1);
	dojo.style(section, {
		"padding":"0px"
	});

	var adjacentColInfo = getAdjacentCol(navContainerId);
	var adjacentCol = adjacentColInfo.adjacentCol; 
	var handleOffset = adjacentColInfo.handleOffset;
	var collapseDir = adjacentColInfo.collapseDir;
	dojo.attr(navContainer, {
		"collapseDir" : collapseDir
	});

	if(DESIGNMODE)
		return;
	if (adjacentCol) {
		adjacentColumn = adjacentCol;
		dojo.style(adjacentCol, {
			"width" : "100%",
			"padding" : "0px",
			"verticalAlign" : "top"
		});
		dojo.style(adjacentCol, {
			"overflowY" : "auto"
		});

     if(APPID !== 'startcntr' && SCREENREADER) {
       var content = document.getElementById('SystemNavAppContent-sc_div')!=null?document.getElementById('SystemNavAppContent-sc_div'):document.getElementById('SystemNavAppContentg-sc_div');
       var newDiv= document.createElement('div');
 	   newDiv.className ='main-form';
       var mainform = document.createElement('span');
       mainform.id = 'main_content';
       mainform.tabIndex = '0';
       mainform.innerText = MAINFORM;
       newDiv.append(mainform);
       var requiredHelp = document.createElement('span');
       requiredHelp.innerHTML = '&nbsp;&nbsp;&nbsp;' + REQUIRED_INDICATOR;
       requiredHelp.className = 'help';
       newDiv.append(requiredHelp);
       content.insertBefore(newDiv, content.firstElementChild);
      }		
		
		var childTables = dojo.query("> table", adjacentCol);
		childTables.style( {
			"height" : ""
		});
		childTables.attr( {
			"height" : ""
		});
		dojo.attr(adjacentCol, {
			"height" : ""
		});
		dojo.style(dojo.byId(adjacentCol.id + "_div"), {
			"overflow":"auto"
		});
		dojo.attr(navContainer, {
			"adjacentColId" : adjacentCol.id + "_div"
		});
		sizePanes(navContainer, true);
	}
	// we had to make sure this happens after the initial render.
	// IE 7 does not like us modifying the table until it is completely created 
	if(collapsible && adjacentCol && !dojo.byId(navContainerId + "_navSep")) {
		var newTD;
		try {
			newTD = sectionRow.insertCell(sectionCol.cellIndex + handleOffset);
			newTD.id = navContainerId + "_navSep";
			newTD.className = "navSep";
	
			var img = document.createElement("button");
			img.setAttribute("type","button");
			img.id = navContainerId + "_collapseImage";
			img.className = "navCollapse" + dojo.attr(navContainer, "collapseDir");
			dojo.attr(img, {"title":navContainerStrings.collapse,"aria-label":navContainerStrings.collapse}); 
			dojo.connect(img, "mouseover", img, function(event) {
				dojo.style(img, {
					"backgroundImage" : dojo.style(img, "backgroundImage").replace('.png', '_hover.png')
				});
				stopBubble(event);
			});
			dojo.connect(img, "mouseout", img, function() {
				dojo.style(img, {
					"backgroundImage" : dojo.style(img, "backgroundImage").replace('_hover.png', '.png')
				});
			});
			appendClass(this, "navCollapseHover");
			dojo.attr(img, {
				"aria-label" : navContainer.getAttribute("buttonTitle")
			});
			
			var imgClickHandler = dojo.connect(img, "click", img, function() {
					//must toggle state
					setNavContainerState(navContainer,dojo.attr(navContainer,"state")!="open",false,false);
				});
			ncEvents(navContainer, img, newTD, imgClickHandler);
			newTD.appendChild(img);
			
			// IV48722 - recalculate the position of the page - starting form the navigation menu
			if(dojo.isIE>0 && window.name == "activity" && document.body.dir=="rtl"){
				setNavContainerState(navContainer,false,false,false);
				setNavContainerState(navContainer,true,false,false);
			}
		}
		catch(error) {
		}
	}
	dojo.attr(sectionRow.parentNode.parentNode, {
		"cellSpacing" : "0"
	});

	if (dojo.attr(navContainer, "pagenav") == "true") {
		navContainer.className += " fixedNavContainer";
	} else {
		navContainer.className += " navContainer";
	}
	return navContainer;
}

/**
 * Change size state of a navContrainer expanded / collapsed
 * 
 * @param img
 * @param navContainer
 * @param state
 */
function setNavContainerState(navContainer, state, dragged, init) {
	var collapseDir = dojo.attr(navContainer, "collapseDir");
	var button = dojo.byId(navContainer.id + "_collapseImage");
	var setOpen = dojo.attr(navContainer, "state");
	if(undef(state)) {
		state = setOpen;
	}
	var navCol = dojo.byId(dojo.attr(navContainer, "navcolId"));
	if(dragged || state=="icon") {
		dojo.attr( navContainer, {
			"maxwidth" : parseInt(navContainer.style.width)
		});
	}
	if(state=="icon" && navIconWidth ==0 ) {
		state = "closed";
	}
	var search = dojo.query('DIV[navSearch="true"]', navContainer.parentNode)[0];
	dojo.publish("navContainerState", state);
	if(button) {
		button.setAttribute('state',state);
		if (state=="open" || state=="icon") {
			dojo.style(search, {'display':''});
			dojo.style(button, {
				"backgroundImage" : dojo.style(button, "backgroundImage").replace("collapsed", "expanded")
			});
			dojo.attr(button, {
				"title" : navContainerStrings.collapse,
				"aria-label" : navContainerStrings.collapse
			});
			var width = (state=="icon")? navIconWidth : parseInt(navContainer.getAttribute("maxwidth"));
			if(state!="icon") {
				if(dragged) {
					width=navContainer.clientWidth;
				}
			}
			var navMenuItems = dojo.query(".navMenus", navContainer);
			for(var tnIndex=0;tnIndex<navMenuItems.length;tnIndex++) {
				var tn = navMenuItems[tnIndex];
				if(state=="icon") {
					appendClass(tn,"hide");
				}
				else {
					removeClass(tn,"hide");
				}
			}
			navContainer.closest('.sectioncol').style.maxWidth = width+'px';
			navContainer.closest('.navContainer').style.maxWidth = width+'px';
			dojo.style(navContainer, {
				"visibility" : "visible",
				"overflow" : "hidden",
				"width" : width+"px",
				"position" : "",
				"left" : "0px",
				"display" : "block"
			});
		} 
		else if (state=="closed") {
			width = 1;
			dojo.style(search, {'display':'none'});
			dojo.style(button, {
				"backgroundImage" : dojo.style(button, "backgroundImage").replace("expanded", "collapsed")
			});
			dojo.attr(button, {
				"title" : navContainerStrings.expand,
				"aria-label" : navContainerStrings.expand
			});
		    navContainer.closest('.sectioncol').style.maxWidth = width+'px';
			navContainer.closest('.navContainer').style.maxWidth = width+'px';
			dojo.style(navContainer, {
				"visibility" : "hidden",
				"overflow" : "hidden",
				"width" : width+"px",
				"position" : "relative",
				"left" : "-1px"
			});
		}
		dojo.attr( navContainer, {
			"state":state
		});
		if(!init){
			queueManager.queueEvent(new Event("setleftnavwidth", APPTARGET, width, REQUESTTYPE_HIGHASYNC), "text/json", "json", null, null);
		}
	}
	sizePanes(navContainer,false);
}

/**
 * Allows min/max toggle when header is clicked
 * @param sectionId
 * @return
 */
function toggleSectionState(sectionId) {
	var containerId = dojo.attr(dojo.byId(sectionId), "containerId");
	var navSections = eval(containerId + "navSections");
	var navSec = navSections[sectionId];
	switch(navSec.state) {
		case 'max':
			toggleSectionView(null, sectionId, 'min');
			break;
		case 'normal':
			toggleSectionView(null, sectionId, 'max');
			break;
		case 'min':
			toggleSectionView(null, sectionId, 'normal');
			break;
	}
}

function setSectionState(section, state){
	if(!section.history){
		section.history = new Array(section.state);
	}
	section.history.push(state); //last one is always current
	section.state = state;
}

function previousSectionStates(sectionOrContainerId) {
	var navSections;
	try {
		navSections = eval(sectionOrContainerId + "navSections");
	} catch(error){
		sectionOrContainerId = dojo.attr(dojo.byId(sectionOrContainerId), "containerId");
		navSections = eval(sectionOrContainerId + "navSections");
	}
	if(navSections){
		for (id in navSections) {
			var secDef = navSections[id];
			secDef.history.pop(); //remove current
			setSectionState(secDef, secDef.history.pop()); 
		}
		sizePanes(sectionOrContainerId,true);
	}
}

/**
 * expand / collapse navsections
 * 
 * @param sectionId
 * @param type
 */
function toggleSectionView(element, sectionId, type, event) {
	if(!element.getAttribute('aria-pressed')) {
		var parentEl = element.parentElement;
		for (let inputEl of parentEl.children) {
			if(inputEl.getAttribute('aria-pressed')){
				inputEl.removeAttribute('aria-pressed');
			}
		}
		element.setAttribute('aria-pressed', true);
	}
	var containerId = dojo.attr(dojo.byId(sectionId), "containerId");
	var navSections = eval(containerId + "navSections");
	var navSec = navSections[sectionId];
	if(type==navSec.state) {
		return;
	}
	if(type == 'normal' && navSec.state == 'max'){
		previousSectionStates(containerId);
	}
	if (!SCREENREADER) {
		dojo.forEach(dojo.query("> div.hoverButton", dojo.byId(sectionId + "_head")), function(element) {
			dojo.style(element, {
				"display" : "none"
			});
		});
	}
	if (type == 'max') {
		for (id in navSections) {
			setSectionState(navSections[id], 'min');
		}
		navSec.history.pop(); //remove min that was just added
		setSectionState(navSec, type);
		sizePanes(containerId,true);
	} 
	else if (type == 'normal') {
		for (id in navSections) {
			var secDef = navSections[id];
			if (secDef.state == "max"){
				setSectionState(secDef, 'normal');
			}
		}
		setSectionState(navSec, type);
		sizePanes(containerId,true);
	}
	else if (type == 'min') {
		var totalOpen = 0;
		for (id in navSections) {
			var secDef = navSections[id];
			if (secDef.state != "min") {
				totalOpen++;
			}
		}
		if (totalOpen == 2) {
			for (id in navSections) {
				var secDef = navSections[id];
				if (secDef.state != "min") {
					setSectionState(secDef, 'max');
				}
			}
		}
		else if (totalOpen == 1) {
			var navSectionsOrder = eval(containerId + "navSectionsOrder");
			var pos = dojo.indexOf(navSectionsOrder, sectionId);
			for (id in navSections) {
				setSectionState(navSections[id], 'min');
			}
			if(!SCREENREADER) {
				if (pos == 0) {
					setSectionState(navSections[navSectionsOrder[1]], 'max');
				}
				else {
					setSectionState(navSections[navSectionsOrder[pos-1]], 'max');
				}
			}
		}
		setSectionState(navSec, type);
		sizePanes(containerId,true);
	}
	var section = dojo.byId(sectionId);
	if(!event || event.type!='keyup'){
		programmaticEvent( section,"mouseout");
		programmaticEvent( section,"mouseover");
	}
	updateSectionIcons(section);
}

/**
 * Used as callback for fetchNavigationMenu
 * 
 * @param responseObj
 * @param ioArgs
 * @return
 */
function fillNavSection(responseObj, ioArgs) {
	var events = dojo.fromJson(ioArgs.args.content.events);
	var id = events[0].navSectionId;
	var render = events[0].render;
	var cName= events[0].classname;
	var section = dojo.byId(id);
	if (undef(responseObj)) {
		section.style.display = "none";
		return;
	}
	if (!SCREENREADER) {
		dojo.connect(section, "mouseover", section, function(event) {
			hideSectionIcons(this);
			showSectionIcons(this);
		});
		dojo.connect(section, "mouseout", section, function(event) {
			hideSectionIcons(this);
		});
		hideExtraHovers(section);
	}
	if (responseObj)
		responseObj = (responseObj.menu) ? responseObj.menu : responseObj;
	responseObj["labelledBy"]=ioArgs.labelledBy;
	var holder = dojo.byId(id + "_content");
	dojo.require("dojox.html");
	dojo.addOnLoad(function(){
		buildNavigationMenu(0, id, id, responseObj, holder, cName, events[0].showImages, events[0].isQuery);
		var navSections = eval(events[0].containerId + "navSections");
		//if(!render || render != "true") {
			sizePanes(events[0].containerId,true);
		//}
	});
}

/**
 * Builds new navsection content menus
 * 
 * @param level
 * @param id
 * @param menuDef
 * @param holder
 * @param className
 * @return
 */
function buildNavigationMenu(level, navId, id, menuDef, holder, className, showImages, isQuery) {
	var mcId = id+"_MC";
	var navMenus = topLevelMenus[mcId];
	if(!navMenus) {
		navMenus = new MenuController(mcId);
	}
 	holder = dojo.query("div", holder)[0];
 	navMenus.holder = holder;
	while (holder.hasChildNodes()) {
		holder.removeChild(holder.lastChild);
	}
	if(className && className!="") {
		className = " "+ className;
	}
	menuDef.baseClassName = "navMenus" + className;
	menuDef.hasSubClassName = "navSubmenu" + className;
	menuDef.subClassName = "subNavMenus"+ className;
	menuDef.staticMenu = true;
	menuDef.id = id+"_menu";
	var menu = navMenus.buildMenuLevel(0,APPTARGET,menuDef);
	if(undef(menu)) {
		return;
	}
	holder.appendChild(menu);
	navMenus.trueTop = menu.id;
	dojo.style(holder, {"overFlow":"hidden"});
	dojo.style(menu, {"display":"", "position":"static"});
	return;
}
	

/**
 * expand / collapse tree items
 * 
 * @param img
 * @param sectionId
 * @return
 */
function toggleMenuView(imgOrId, sectionOrId) {
	var section = dojo.byId(sectionOrId);
	var img = dojo.byId(imgOrId);
	if (section.style.display == "none" || section.offsetHeight == 1) {
		dojo.attr(img, {
			"src" : img.src.replace("collapsed", "expanded")
		});
		img.src = img.src.replace("collapsed", "expanded");
		dojo.style(section, {
			"display" : ""
		});
	} else {
		dojo.attr(img, {
			"src" : img.src.replace("expanded", "collapsed")
		});
		img.src = img.src.replace("expanded", "collapsed");
		dojo.style(section, {
			"display" : "none"
		});
	}
}

/**
 * Add an item to the new navsection content menu
 * 
 * @param level
 * @param itemIndex
 * @param ul
 * @param item
 * @param className
 * @return
 */
function treeItemHighlight(event) {
	var div = getSourceElement(event);
	if (div.tagName != "DIV") {
		div = findParent(div, "DIV", 1);
	}
	switch (event.type) {
	case "focus":
	case "mouseover":
		div.className = div.className.replace("acMenuDIV", "acMenuhoverDIV");
		dojo.forEach(dojo.query("span", div), function(element) {
			dojo.attr(element, {
				"origclass" : element.className
			});
			dojo.attr(element, {
				"class" : "acMenuHoverText"
			});
			dojo.connect(element, "mouseover", element, function(event) {
				cancelEvent(event);
			}, true);
			dojo.style(element, {
				"pointerEvents" : "none"
			});
		});
		break;
	case "mouseout":
	case "blur":
		div.className = div.className.replace("acMenuhoverDIV", "acMenuDIV");
		dojo.forEach(dojo.query("span", div), function(element) {
			dojo.attr(element, {
				"class" : dojo.attr(element, "origclass")
			});
		});
		break;
	}
}

function addTreeStyleMenuItem(navId, id, level, itemIndex, ul, item, className, showImages, isQuery) {
	var li = document.createElement("li");
	if(item.disabled) {
		dojo.attr(li, {"enabled":"false","aria-disabled":"true"});
	}
	dojo.attr(li, {
		"role" : "presentation",
		"class" : "acMenuLI",
		"id": ul.id + item.id
	});
	var div = document.createElement("div");
	div.id = li.id + "_div";
	dojo.connect(div, "focus", div, function(event) {
		treeItemHighlight(event);
	});
	dojo.connect(div, "mouseover", div, function(event) {
		treeItemHighlight(event);
	});
	dojo.connect(div, "blur", div, function(event) {
		treeItemHighlight(event);
	});
	dojo.connect(div, "mouseout", div, function(event) {
		treeItemHighlight(event);
	});
	div.className = "acMenuDIV";
	if (item.sub) {
		var plusMinus = document.createElement("img");
		plusMinus.id = li.id + "_plusminus";
		plusMinus.src = IMAGE_PATH + "tasknav/" + ((item.sub) ? "node_collapsed.png" : "node_nochildren.png");
		plusMinus.className = "acMenuImg";
        plusMinus.alt = "";
		dojo.attr(plusMinus, {
			"wait" : IMAGE_PATH + "tasknav/node_collapsed.png"
		});
		div.appendChild(plusMinus);
	} else {
		if(document.body.dir=="rtl") {
			dojo.style(div, {
				"paddingRight" : "13px"
			});
		}
		else {
			dojo.style(div, {
				"paddingLeft" : "13px"
			});
		}

	}
	if (level == 0 && showImages) {
		var menuImage = document.createElement("img");
		menuImage.className = "acMenuImg";
        menuImage.alt = "";
		if(item.disabled) {
			menuImage.className += " opacity_20";
		}
		menuImage.src = IMAGE_PATH + ((undef(item.image)) ? "menus/blank.gif" : item.image);
		div.appendChild(menuImage);
	}
	var tNode = document.createElement("SPAN");
	tNode.id = li.id + "_label";
	dojo.style(tNode, {
		"display" : "inline",
		"whiteSpace" : "normal",
		"role":"presentation"
	});
	div.appendChild(tNode);
	if (level > 0) {
		tNode.className = "acSubMenuText";
	}
	if(item.disabled) {
		tNode.className += " opacity_20";
	}
	tNode.innerHTML = item.text;// uncomment for text wrapping test + " | this
	// is some extra text to make things really
	// long";
	if (level == 0 && itemIndex == 0)
		div.tabIndex = "0";
	dojo.attr(div, {
		"role" : "treeitem",
		"aria-labelledby" : tNode.id
	});
	if(!undef(item.sub)) {
		dojo.attr(div, {
			"aria-expanded" : "false",
			"aria-haspopup": "true"
		});
	}
	li.appendChild(div);
	if (item.showborder) { // use item.border for all defined menu seps
		li.className = li.className + " acMenuItemBorder";
	}
	var addItem = true;
	if (item.sub) {
		var sub = document.createElement("ul");
		sub.id = ul.id + "_" + item.sub.id;

		var toggleEvent = "toggleMenuView('" + plusMinus.id + "','" + sub.id + "_menu')";
		dojo.attr(li, {
			"nodeinfo" : dojo.toJson( {
				"children" : true,
				"childrenfilled" : true,
				"state" : "closed",
				"node" : null,
				"primaryevent" : "",
				"toggleevent" : {
					"jsevent" : toggleEvent
				}
			})
		});
		if(document.body.dir=="rtl") {
			dojo.style(sub, {
				"display" : "none",
				"marginRight": "20px"
			});			
		}
		else {
			dojo.style(sub, {
				"display" : "none",
				"marginLeft": "20px"
			});
		}
		

		dojo.attr(sub, {
			"navId" : navId,
			"role" : "group",
			"aria-live" : "polite" 
		});
		li.appendChild(sub);
		item.sub.menuevent = item.menuevent;
		var subLength = buildNavigationMenu((level + 1), navId, sub.id, item.sub, sub, className);
		if (subLength == 0) {
			addItem = false;
		}
		dojo.connect(li, "click", li, function(event) {
			if(dojo.attr(li, "enabled")!="false") {
				toggleMenuView(plusMinus.id, sub.id);
			}
			hideAllMenusNF();
			stopBubble(event);
		});
	} else {
		if(undef(item.eventvalue)) {
			item.eventvalue = "";
		}
		dojo.attr(li, {
			"nodeinfo" : dojo.toJson( {
				"children" : false,
				"childrenfilled" : false,
				"state" : "closed",
				"node" : null,
				"primaryevent" : {
					"eventtype" : ((item.mxevent) ? item.mxevent : item.menuevent),
					"targetid" : ((item.eventtarget) ? item.eventtarget : navId),
					"value" : (!isQuery) ? item.eventvalue : (item.eventvalue+"{:}"+item.text)
				},
				"toggleevent" : ""
			})
		});

		dojo.connect(li, "click", li, function(event) {
			stopBubble(event);
			hideAllMenusNF();
			if(dojo.attr(li, "enabled")!="false") {
				processEvent(dojo.fromJson(this.getAttribute("nodeinfo")).primaryevent);
			}
		});
		if(item.mxevent) {
			dojo.attr(li, {"eventtype" : item.mxevent});
			dojo.subscribe("appeventstate", null, function(message){
				if(message.eventtype!=dojo.attr(li, "eventtype"))
					return;
				if(message.enabled){
					dojo.forEach(dojo.query("img", li), function(element) {
						removeClass(element, "opacity_20");
					});
					dojo.forEach(dojo.query("span", li), function(element) {
						removeClass(element, "opacity_20");
						dojo.attr(element, {
							"disabled" : ""
						});
					});
					dojo.attr(li, {
						"enabled" : "true",
						"ariaDisabled" : "false"
					});
				}
				else {
					dojo.forEach(dojo.query("img", li), function(element) {
						appendClass(element, "opacity_20");
					});
					dojo.forEach(dojo.query("span", li), function(element) {
						appendClass(element, "opacity_20");
						dojo.attr(element, {
							"disabled" : "disabled"
						});
					});
					dojo.attr(li, {
						"enabled" : "false",
						"ariaDisabled" : "true"
					});
				}
			});
		}
	}
	if (addItem) {
		dojo.attr(li, {
			"navId" : navId
		});
		ul.appendChild(li);
	}
	return li;
}

/**
 * Creates and queues event to getData from Navsection.java using XHR
 * 
 * @param id
 * @param containerId
 * @return
 */
function fetchNavigationMenu(id, containerId, showImages, tryCache, isQuery) {
	var holder = dojo.byId(id + "_content");
	var section = dojo.byId(id);
	if (!holder || !section)
		return;
	var navSections = eval(containerId + "navSections");
	if (tryCache) {
		var menuCache = localStorage["menucache_"+SECURE_USERNAME+"_"+USERLANG];
		if (menuCache) {
			menuCache = dojo.fromJson(menuCache);
			gotoMenu = menuCache["goto"];
			if (gotoMenu) {
				if (gotoMenu.items[0].eventvalue == "startcntr") {
					gotoMenu.items.shift();
				}
				fromCache = true;
				fillNavSection(gotoMenu, {
					'args' : {
						'content' : {
							'events' : "[{'navSectionId':'" + id + "','containerId':'" + containerId + "','showImages':" + showImages + "}]"
						}
					}
				});
				return;
			}
		}
	}
	var fetchEvent = new Event("getData", id, "", REQUESTTYPE_HIGHASYNC);
	var menu = dojo.byId(id + "_menu");
	if (menu) {
		var inner = dojo.byId(holder.id + "_inner");
		if (inner) {
			inner.removeChild(menu);
		}
	}
	fetchEvent["navSectionId"] = id;
	fetchEvent["containerId"] = containerId;
	fetchEvent["showImages"] = showImages;
	fetchEvent["isQuery"] = isQuery;
	queueManager.queueEvent(fetchEvent, "text/json", "json", fillNavSection, null);
}



/* Generic helper methods */

/**
 * Walk up DOM a specified number of levels looking for a particular tagname
 */
function findParent(element, tagName, levels) {
	tagName = tagName.toUpperCase();
	var level = 0;
	element = element.parentNode;
	while (element.tagName != tagName || level < levels) {
		if (element.tagName == tagName) {
			level++;
			if (level == levels)
				break;
		}
		element = element.parentNode;
	}
	return element;
}

/*
 * Returns the number of members within a Javascript object
 */
function getObjectLength(object) {
	var element_count = 0;
	for (e in object) {
		element_count++;
	}
	return element_count;
}

function hideExtraHovers(section) {
	dojo.connect(section, "mouseout", section, function(event) {
		dojo.forEach(dojo.query(".hoverButton", dojo.byId(this.id + "_head")), function(element) {
			dojo.style(element, {
				"display" : "none"
			});
		});
	});
}

function ncEvents(navContainer, img, newTD, oldImgHandler) {
	dojo.disconnect(oldImgHandler);
	dojo.connect(img, "click", img, function(event) {
		///must toggle state
		var state = dojo.attr(navContainer,"state");
		switch (state)
		{
			case "open":
				state="icon";
				break;
			case "icon":
				state="closed";
				break;
			case "closed":
				state="open";
		}
		setNavContainerState(navContainer, state, false, false);
		stopBubble(event);
	});
	dojo.connect(img, "mouseup", img, function(event) {
		
	});
	dojo.connect(img, "mousedown", img, function(event) {
		stopBubble(event);
	});

	dojo.style(newTD, {
		"cursor" : "col-resize"
	});
	dojo.connect(newTD, "mousedown", newTD, function(event) {
		hideAllMenus();
		this.style.opacity = '0.4';
		// cancel out any text selections 
		document.body.focus(); 
		appendClass(document.body,"dragging");
		// prevent text selection in IE 
		var bodyOnSelectStart = document.body.onselectstart;
		document.body.onselectstart = function () { return false; };
		var moveHandle = dojo.connect(document, "mousemove", document, function(event) {
			hideAllMenus();
			appendClass(document.body,"dragged");
			document.body.onselectstart = function () { return false; };
			var width = parseInt(event.clientX) - 5;
			if(document.body.dir == "rtl") {
				width = parseInt(document.body.clientWidth - event.clientX) - 5;
			}
			dojo.attr(navContainer,{"maxwidth":width});
			var tbSpacer = dojo.byId("apptoolbarSpacer");
			if(tbSpacer){
				dojo.style(tbSpacer, {
					"width" : (width+10)+"px"
				});
			}
			dojo.style(navContainer, {
				"visibility" : "visible",
				"overflow" : "hidden",
				"width" : width+"px",
				"position" : "",
				"left" : "0px"
			});
			var navWidth = parseInt(navContainer.style.width);
			navContainer.closest('.sectioncol').style.maxWidth = width+'px';
			navContainer.closest('.navContainer').style.maxWidth = width+'px';
			var navMenuItems = dojo.query(".navMenus", navContainer);
			var tooSmall = navWidth < navIconWidth + 20;
			for(var tnIndex=0;tnIndex<navMenuItems.length;tnIndex++) {
				var tn = navMenuItems[tnIndex];
				if(tooSmall) {
					appendClass(tn,"hide");
				}
				else {
					removeClass(tn,"hide");
				}
			}
			sizeAnchors(navContainer, navWidth);
			sizePanes(navContainer, false);
		}); 
		dojo.connect(document, "mouseover", document, function(event) { return false; });
		var docHandler = dojo.connect(document, "mouseup", newTD, function(event) {
			this.style.opacity = '1';
			dojo.disconnect(moveHandle);
			dojo.disconnect(docHandler);
			removeClass(document.body,"dragging");
			document.body.onselectstart = bodyOnSelectStart;
			var navWidth = parseInt(navContainer.style.width);
			if(parseInt(navContainer.style.width) < 25) {
				navContainer.style.width=parseInt(navIconWidth+1)+"px";
				setNavContainerState(navContainer,"icon",true,false);
			}
			else {
				setNavContainerState(navContainer,"open",true,false);
			}
			var width = parseInt(event.clientX) - 5;
			if(document.body.dir == "rtl") {
				width = parseInt(document.body.clientWidth - event.clientX) - 5;
			}
		});
	});
}

function sizeAnchors(navContainer, width){
	var navMenuAnchors = dojo.query(".dojoxEllipsis", navContainer);
	for(var naIndex=0;naIndex<navMenuAnchors.length;naIndex++) {
		var navAnchor = navMenuAnchors[naIndex];
		if(navAnchor.className.indexOf("navTitle") ==0) {
			continue;
		}
		dojo.removeAttr(navAnchor, "width");
		var section = navAnchor.parentNode.parentNode.parentNode.parentNode;
		var newWidth = width;
		var scrolled = (section.scrollHeight > section.clientHeight);
		if(scrolled) {
			if(dojo.isIE > 0){
				newWidth -= 7;
			}
			else {
				newWidth -= 15;
			}
		}
		newWidth-= (parseInt(dojo.style(navAnchor, "paddingRight")) + parseInt(dojo.style(navAnchor, "paddingLeft")) + 6);
		if(newWidth<0) {
			newWidth = 1; //IE issue
		}
		dojo.style(navAnchor, {"width": newWidth + "px"});
	}
}

function navHeadingKey(event, sectionId){
	var section = dojo.byId(sectionId)
	if(!section){
		return;
	}
	
	var navSections = eval(dojo.attr(section, "containerid") + "navSections");
	var secDef = navSections[sectionId];
	var state = secDef.state;
	switch(event.keyCode)
	{
		case KEYCODE_LEFT_ARROW:
		case KEYCODE_UP_ARROW:
			toggleSectionView(null, sectionId, 'min', event);
			cancelEvent(event);
			break;
		case KEYCODE_RIGHT_ARROW:
		case KEYCODE_DOWN_ARROW:
			toggleSectionView(null, sectionId, 'normal', event);
			cancelEvent(event);
			break;
		default:
	}
}

function updateSectionIcons(section){
	hideSectionIcons(section);
	showSectionIcons(section);
}

function hideSectionIcons(section){
	dojo.forEach(dojo.query("> div.hoverButton", dojo.byId(section.id + "_head")), function(element) {
		dojo.style(element, {
			"display" : "none"
		});
	});
}

function showSectionIcons(section){
	var navSections = eval(dojo.attr(section, "containerId") + "navSections");
	var navSectionsOrder = eval(dojo.attr(section, "containerId") + "navSectionsOrder");
	if (navSectionsOrder.length > 1) {
		var col = dojo.byId(section.id + "_" + navSections[section.id].state);
		if (col) {
			dojo.style(col, {
				"display" : "inline"
			});
		}
	}
}